/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the license.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.kelp.common;

// ToolBox imports
import org.enhydra.tool.ToolBoxInfo;
import org.enhydra.tool.common.PathHandle;
import org.enhydra.tool.common.FileUtil;
import org.enhydra.tool.common.ExtensionFilter;
import org.enhydra.tool.configure.ConfigTool;

// Kelp imports
import org.enhydra.kelp.common.Constants;
import org.enhydra.kelp.common.importer.JavaFilter;
import org.enhydra.kelp.common.node.OtterTemplateNode;
import org.enhydra.kelp.common.node.OtterProject;

// Standard imports
import java.io.BufferedReader;
import java.io.IOException;
import java.io.File;
import java.io.FileFilter;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.util.Vector;

//
public class Backward {

    // strings not to be resourced
    private final static String RELATIVE_CLASSES = "= ../classes";

    //
    private Vector              phVector = new Vector();

    public static boolean createIsWebApp(String[] sourcePaths) {
        boolean            webApp = true;
        PresentationFilter filter = new PresentationFilter();

        for (int i = 0; i < sourcePaths.length; i++) {
            File found = FileUtil.findFirst(filter, sourcePaths[i]);

            if (found != null) {
                webApp = filter.isWebApp();
                break;
            }
        }
        return webApp;
    }

    public static String createDefaultInputPath(String in,
                                                String[] sourcePaths) {
        PathHandle handle = null;
        File       file = null;
        String     out = new String(in);

        file = new File(out);
        if (file.isDirectory()) {

            // done
        } else if (file.getParentFile().exists()) {
            file = new File(file.getParentFile(), Constants.DIR_INPUT);
            if (file.exists()) {
                out = file.getAbsolutePath();

                // done
            } else {
                ExtensionFilter filter = new ExtensionFilter();
                StringBuffer    mkIn = new StringBuffer();

                mkIn.append('.');
                mkIn.append(Constants.TYPE_MK);
                mkIn.append('.');
                mkIn.append(Constants.TYPE_IN);
                filter.setDirectoryValid(false);
                filter.addExtension(Constants.TYPE_IN);
                filter.addExclusion(mkIn.toString());
                for (int i = 0; i < sourcePaths.length; i++) {
                    file = FileUtil.findFirst(filter, sourcePaths[i]);
                    if (file != null) {
                        out = file.getParent();
                        break;
                    }
                }
            }
        }
        out = PathHandle.createPathString(out);
        return out;
    }

    public static String createDefaultBootstrapPath(String in,
            OtterProject project) {
        String          out = new String(in);
        String          deployPath = new String();
        String          inputPath = new String();
        File            file = new File(out);
        BootstrapFilter filter = null;

        if (file.isFile()) {
            out = PathHandle.createPathString(file);

            // done
        } else {
            deployPath = project.getDeployRootPath();
            filter = new BootstrapFilter(false);
            file = FileUtil.findFirst(filter, deployPath);
            if (file != null && file.isFile()) {
                out = PathHandle.createPathString(file);

                // done
            } else {
                inputPath = project.getDeployInputPath();
                filter = new BootstrapFilter(true);
                file = FileUtil.findFirst(filter, inputPath);
                if (file != null && file.isFile()) {
                    out = PathHandle.createPathString(file);
                    out = deployPath
                          + out.substring(inputPath.length(),
                                          out.length() - 3);
                }
            }
        }
        return out;
    }

    public static String[][] createReplacementTable(String[][] tableIn,
            OtterTemplateNode[] templates) {
        String[][] tableOut = new String[0][2];
        String[]   placeholders = new String[0];
        Backward   back = null;

        back = new Backward();
        placeholders = back.findPlaceholders(templates);
        if (placeholders.length == 0) {
            tableOut = tableIn;
        } else {
            tableOut = new String[tableIn.length + placeholders.length][2];
            for (int i = 0; i < placeholders.length; i++) {
                tableOut[i][0] = placeholders[i];

                // strings not to be resourced
                if (placeholders[i].equals(Backward.RELATIVE_CLASSES)) {
                    tableOut[i][1] = "= @JAVA_DEPLOY_PATH@/../classes";
                } else if (placeholders[i].equals("@CLASSES@")) {
                    tableOut[i][1] = "@JAVA_DEPLOY_PATH@/../classes";
                } else if (placeholders[i].equals("@SERVLET_CONF_DIR@")) {
                    tableOut[i][1] = "@JAVA_DEPLOY_PATH@/servlet";
                } else if (placeholders[i].equals("@OUTPUT@")) {
                    tableOut[i][1] = "@JAVA_DEPLOY_PATH@";
                } else {
                    tableOut[i][1] = placeholders[i];
                }
            }
            for (int i = 0; i < tableIn.length; i++) {
                tableOut[i + placeholders.length][0] = tableIn[i][0];
                tableOut[i + placeholders.length][1] = tableIn[i][1];
            }
        }
        return tableOut;
    }

    private String[] findPlaceholders(OtterTemplateNode[] templates) {
        String placeholders[] = new String[0];

        phVector.clear();
        for (int i = 0; i < templates.length; i++) {
            findPlaceholdersInFile(templates[i].getFilePath());
        }
        placeholders = new String[phVector.size()];
        placeholders = (String[]) phVector.toArray(placeholders);
        phVector.clear();
        return placeholders;
    }

    private void findPlaceholdersInFile(String path) {
        BufferedReader reader = null;
        File           file = null;
        String         line = null;

        file = new File(path);
        if (file.isFile()) {
            try {
                reader = new BufferedReader(new FileReader(file));
                line = reader.readLine();
                while (line != null) {
                    findRelativeClasses(line);
                    findPlaceholdersInLine(line);
                    line = reader.readLine();
                }
                reader.close();
            } catch (FileNotFoundException e) {
                e.printStackTrace();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    private void findRelativeClasses(String line) {
        int index = line.indexOf(Backward.RELATIVE_CLASSES);

        if (index > 0) {
            if (!phVector.contains(Backward.RELATIVE_CLASSES)) {
                phVector.insertElementAt(Backward.RELATIVE_CLASSES, 0);
            }
        }
    }

    private void findPlaceholdersInLine(String line) {
        String       search = new String(line);
        String[]     exclude = ConfigTool.getSuffixArray();
        StringBuffer ph = new StringBuffer();
        int          index = -1;
        boolean      start = false;
        boolean      include = true;

        index = search.indexOf('@');
        while (index > -1) {
            if (start) {
                ph.setLength(0);
                ph.append('@');
                ph.append(search.substring(0, index));
                ph.append('@');
                if (!phVector.contains(ph.toString())) {
                    include = true;
                    for (int i = 0; i < exclude.length; i++) {
                        if (ph.toString().endsWith(exclude[i])) {
                            include = false;
                            break;
                        }
                    }
                    if (include) {
                        phVector.addElement(ph.toString());
                    }
                }
            }
            if (search.length() > index) {
                search = search.substring(index + 1);
            } else {
                search = new String();
            }
            start = (!start);
            index = search.indexOf('@');
        }
    }

}

//
class BootstrapFilter implements FileFilter {
    private boolean template = false;

    public BootstrapFilter(boolean t) {
        template = true;
    }

    public boolean accept(File file) {
        boolean      accept = false;
        String       path = file.getAbsolutePath().toLowerCase();
        StringBuffer multiserverBuf = new StringBuffer();
        StringBuffer servletBuf = new StringBuffer();
        StringBuffer kernelBuf = new StringBuffer();
        StringBuffer bootstrapBuf = new StringBuffer();
        StringBuffer bootstrap4Buf = new StringBuffer();

        //
        multiserverBuf.append(Constants.FILE_MULTISERVER_CONF);
        servletBuf.append(Constants.FILE_SERVLET_CONF);
        kernelBuf.append(Constants.FILE_KERNEL_CONF);
        bootstrapBuf.append(Constants.FILE_BOOTSTRAP_CONF);
        bootstrap4Buf.append(Constants.FILE_BOOTSTRAP4_CONF);
        if (template) {
            multiserverBuf.append('.');
            multiserverBuf.append(Constants.TYPE_IN);
            servletBuf.append('.');
            servletBuf.append(Constants.TYPE_IN);
            kernelBuf.append('.');
            kernelBuf.append(Constants.TYPE_IN);
            bootstrapBuf.append('.');
            bootstrapBuf.append(Constants.TYPE_IN);
            bootstrap4Buf.append('.');
            bootstrap4Buf.append(Constants.TYPE_IN);
        }
        if (file.isFile()) {
            if (ToolBoxInfo.isEnhydra3()) {
                if (path.endsWith(multiserverBuf.toString())
                        || path.endsWith(servletBuf.toString())) {
                    accept = true;
                }
            } else {
                if (path.endsWith(kernelBuf.toString())
                        || path.endsWith(bootstrapBuf.toString())
                        || path.endsWith(bootstrap4Buf.toString())) {
                    accept = true;
                }
            }
        }
        return accept;
    }

}

//
class PresentationFilter extends JavaFilter {

    // strings not to be resourced
    private final static String HTTP_PRESENTATION = "HttpPresentation";
    private final static String HTTP_SERVLET = "HttpServlet";

    //
    private boolean             webApp = true;

    public boolean accept(File f) {
        boolean accept = super.accept(f);

        if (accept && f.canRead()) {
            accept = false;
            BufferedReader reader = null;
            String         line = new String();

            try {
                reader = new BufferedReader(new FileReader(f));
                line = reader.readLine();
                while (line != null) {
                    if (line.indexOf(HTTP_PRESENTATION) > -1) {
                        webApp = false;
                        accept = true;
                        break;
                    } else if (line.indexOf(HTTP_SERVLET) > -1) {
                        webApp = true;
                        accept = true;
                        break;
                    }
                    line = reader.readLine();
                }
                reader.close();
            } catch (IOException e) {
                e.printStackTrace();
                accept = false;
            }
        }
        return accept;
    }

    protected boolean isWebApp() {
        return webApp;
    }

}
