/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.kelp.common.deployer;

// ToolBox
import org.enhydra.tool.archive.ArchiveException;
import org.enhydra.tool.archive.JarPlan;
import org.enhydra.tool.archive.wizard.ArchiveWizard;
import org.enhydra.tool.common.PathHandle;

// JDK
import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import java.util.EventObject;
import java.util.ArrayList;
import java.util.Arrays;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.event.CellEditorListener;
import javax.swing.table.TableCellEditor;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableModel;

//
public class ArchiveTablePanel extends JPanel {
    private BorderLayout layoutMain;
    private JScrollPane  scroll;
    private JTable       table;
    private String       defWorkRoot = null;

    public ArchiveTablePanel() {
        try {
            jbInit();
            pmInit();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    //
    protected void clearAll() {
        table.getModel().removeTableModelListener(table);
        table.setModel(new DefaultTableModel());
        removeAll();
    }

    protected void addRow() {
        ArchiveWizard wizard = null;

        wizard = new ArchiveWizard();
        wizard.setWorkingRoot(getDefWorkingRoot());
        wizard.setMostRecentlyUsed(false);
        if (wizard.showDialog(getTopLevelAncestor()) == JOptionPane.CANCEL_OPTION) {
           // cancel
        } else {
            int index = -1;

            index = getModel().addPlan(wizard.getPlan());
            table.setRowSelectionInterval(index, index);
        }
    }

    protected void editRow() throws ArchiveException {
        ArchiveWizard wizard = null;
        JarPlan       plan = null;
        int           index = -1;

        index = table.getSelectedRow();
        if (index > -1) {
            wizard = new ArchiveWizard();
            wizard.setMostRecentlyUsed(false);
            wizard.setWorkingRoot(getDefWorkingRoot());
            plan = getModel().getPlan(index);
            wizard.setPlan(plan);
            if (wizard.showDialog(getTopLevelAncestor()) == JOptionPane.CANCEL_OPTION) {
                // cancel
            } else {
                getModel().setPlan(wizard.getPlan(), index);
                table.setRowSelectionInterval(index, index);
            }
        }
    }

    protected void removeSelection() {
        int row = -1;

        row = table.getSelectedRow();
        if (row > -1) {
            getModel().removeRow(row);
        }
    }

    protected void moveSelectionDown() {
        int count = -1;
        int row = -1;

        count = getModel().getRowCount();
        row = table.getSelectedRow();
        if (row > -1 && (row < (count - 1))) {
            getModel().moveRow(row, row, row + 1);
            table.setRowSelectionInterval(row + 1, row + 1);
        }
    }

    protected void moveSelectionUp() {
        int row = -1;

        row = table.getSelectedRow();
        if (row > 0) {
            getModel().moveRow(row, row, row - 1);
            table.setRowSelectionInterval(row - 1, row - 1);
        }
    }

    protected void setPlans(JarPlan[] plans) {
        getModel().setPlans(plans);
    }

    protected JarPlan[] getPlans() {
        return getModel().getPlans();
    }

    protected void setDefWorkingRoot(String r) {
        defWorkRoot = PathHandle.createPathString(r);
    }

    protected String getDefWorkingRoot() {
        if (defWorkRoot == null) {
            setDefWorkingRoot(System.getProperty("user.dir"));
        }
        return defWorkRoot;
    }

    //
    private ArchiveTableModel getModel() {
        return (ArchiveTableModel) table.getModel();
    }

    private void initCellColors(Component comp, boolean isSelected, int row) {
        if ((row == table.getSelectedRow()) || isSelected) {
            comp.setBackground(SystemColor.textHighlight);
            comp.setForeground(SystemColor.textHighlightText);
        } else {
            comp.setBackground(SystemColor.info);
            comp.setForeground(SystemColor.infoText);
        }
    }

    private void pmInit() {
        CheckCell        check = null;
        TextCellRenderer textRenderer = null;

        table.setModel(new ArchiveTableModel());
        table.getTableHeader().setEnabled(false);
        table.getTableHeader().setReorderingAllowed(false);
        table.getTableHeader().setUpdateTableInRealTime(false);
        table.setCellSelectionEnabled(false);
        table.setColumnSelectionAllowed(false);
        table.setRowSelectionAllowed(true);
        table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        table.getModel().addTableModelListener(table);
        check = new CheckCell();
        textRenderer = new TextCellRenderer();
        table.setSelectionMode(DefaultListSelectionModel.SINGLE_SELECTION);
        table.getColumnModel().getColumn(0).setCellRenderer(check);
        table.getColumnModel().getColumn(0).setCellEditor(check);
        table.getColumnModel().getColumn(0).setPreferredWidth(40);
        table.getColumnModel().getColumn(1).setCellRenderer(textRenderer);
        table.getColumnModel().getColumn(1).setPreferredWidth(250);
    }

    private void jbInit() throws Exception {
        scroll = (JScrollPane) Beans.instantiate(getClass().getClassLoader(),
                                                 JScrollPane.class.getName());
        table = (JTable) Beans.instantiate(getClass().getClassLoader(),
                                           JTable.class.getName());
        layoutMain =
            (BorderLayout) Beans.instantiate(getClass().getClassLoader(),
                                             BorderLayout.class.getName());
        scroll.getViewport().add(table, null);
        this.setForeground(UIManager.getColor("text"));
        this.setLayout(layoutMain);
        this.add(scroll, BorderLayout.CENTER);
    }

    private class CheckCell extends DefaultTableCellRenderer
        implements TableCellEditor {
        private JCheckBox         check = null;
        private DefaultCellEditor editor = null;

        public CheckCell() {
            super();
        }

        public void addCellEditorListener(CellEditorListener l) {
            editor.addCellEditorListener(l);
        }

        public void removeCellEditorListener(CellEditorListener l) {
            editor.removeCellEditorListener(l);
        }

        public Object getCellEditorValue() {
            return editor.getCellEditorValue();
        }

        public boolean isCellEditable(EventObject anEvent) {
            return editor.isCellEditable(anEvent);
        }

        public boolean shouldSelectCell(EventObject anEvent) {
            return editor.shouldSelectCell(anEvent);
        }

        public void cancelCellEditing() {
            editor.cancelCellEditing();
        }

        public boolean stopCellEditing() {
            return editor.stopCellEditing();
        }

        public Component getTableCellRendererComponent(JTable table,
                Object value, boolean isSelected, boolean hasFocus, int row,
                int column) {
            if (isSelected) {
                table.setRowSelectionInterval(row, row);
            }
            check = createCheck(table, value, isSelected, hasFocus, row,
                                column, false);
            initCellColors(check, isSelected, row);
            editor = new DefaultCellEditor(check);
            return editor.getTableCellEditorComponent(table, value,
                                                      isSelected, row,
                                                      column);
        }

        public Component getTableCellEditorComponent(JTable table,
                Object value, boolean isSelected, int row, int column) {
            table.setRowSelectionInterval(row, row);
            check = createCheck(table, value, isSelected, false, row, column,
                                true);
            initCellColors(check, isSelected, row);
            editor = new DefaultCellEditor(check);
            return editor.getTableCellEditorComponent(table, value,
                                                      isSelected, row,
                                                      column);
        }

        protected synchronized JCheckBox createCheck(JTable table,
                Object value, boolean isSelected, boolean hasFocus, int row,
                int column, boolean edit) {
            JCheckBox cb = null;

            cb = new JCheckBox();
            cb.setFocusPainted(true);
            cb.setHorizontalAlignment(SwingConstants.CENTER);
            return cb;
        }

    }

    //
    private class TextCellRenderer extends DefaultTableCellRenderer {
        public Component getTableCellRendererComponent(JTable table,
                Object value, boolean isSelected, boolean hasFocus, int row,
                int column) {
            Component comp = null;

            if (isSelected) {
                table.setRowSelectionInterval(row, row);
            }
            comp = super.getTableCellRendererComponent(table, value,
                                                       isSelected, hasFocus,
                                                       row, column);
            if (comp instanceof JLabel && value instanceof JarPlan) {
                JLabel  lab = (JLabel) comp;
                JarPlan plan = (JarPlan) value;

                lab.setHorizontalAlignment(SwingConstants.LEFT);
                lab.setText(plan.getArchivePath());
                lab.setToolTipText(lab.getText());
                initCellColors(lab, isSelected, row);
            }
            return comp;
        }

    }
}
