/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */

package org.enhydra.kelp.common.deployer;

// ToolBox imports
import org.enhydra.tool.ToolBoxInfo;

// Kelp imports
import org.enhydra.kelp.common.Constants;
import org.enhydra.kelp.common.PathUtil;
import org.enhydra.kelp.common.node.OtterProject;

// Standard imports
import java.io.File;
import java.io.FileWriter;
import java.io.PrintWriter;

public class StartServerGenerator {
    private OtterProject project = null;
    private File         directory;
    private File         file = null;

    /**
     * Constructor declaration
     *
     *
     * @param p
     */
    public StartServerGenerator(OtterProject p) {
        project = p;
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    public File getFile() {
        String path = project.getSourcePathArray()[0];
        File dir = new File(path);
        file = new File(dir, Constants.FILE_START_SERVER);
        return file;
    }

    /**
     * Write a StartServer.java file for launching and stopping the server
     * with a simple graphical interface. This is useful in environments such
     * as the JDeveloper where you cannot run an arbitrary class from a project.
     *
     * @throws IOException
     */
    public void create() throws java.io.IOException {
        if ((!getFile().exists()) || project.isDeployOverwrite()) {
            PrintWriter  out = new PrintWriter(new FileWriter(getFile()));

            // strings not to be translated
            out.println(' ');
            out.println("// ");
            out.println("//  You can run this class to start the Enhydra ");
            out.println("//  application server from your IDE.");
            out.println("// ");
            out.println("//  Note: The <eas_root>/tool/lib/toolbox.jar ");
            out.println("//  must be in your class path to compile or run ");
            out.println("//  this class.");
            out.println("// ");
            out.println(' ');
            out.println("public class StartServer extends org.enhydra.tool.boot.StartServer {"); //nores
            out.println(' ');
            out.println("  public static void main(String[] args) {"); // nores
            out.println("     org.enhydra.tool.boot.StartServer.main(args);");
            out.println("  }");
            out.println(' ');
            out.println('}');
            out.println(' ');
            out.flush();
            out.close();
        }
    }


}
