/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.kelp.common.node;

// Toolbox imports
import org.enhydra.tool.common.PathHandle;
import org.enhydra.tool.ToolBoxInfo;

// Enhydra imports
import org.enhydra.xml.xmlc.compiler.Parse;
import org.enhydra.xml.xmlc.XMLCException;

// Kelp imports
import org.enhydra.kelp.common.Constants;
import org.enhydra.kelp.common.PropUtil;
import org.enhydra.kelp.common.bridge.XMLCConnectionFactory;
import org.enhydra.kelp.common.bridge.MetaDataHandler;
import org.enhydra.kelp.common.map.Mapper;
import org.enhydra.kelp.common.node.OtterProject;

// Standard imports
import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.lang.reflect.Constructor;
import java.lang.reflect.Method;
import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Vector;
import java.util.StringTokenizer;
import java.util.ResourceBundle;

/**
 * Class declaration
 *
 *
 * @author Paul Mahar
 */
public class OtterXMLCNode implements OtterDocumentNode {

    // strings not to be resourced
    static ResourceBundle     res =
        ResourceBundle.getBundle("org.enhydra.kelp.common.Res");    // nores
    private final String      POUND = "#";                          // nores
    private final String      DOMFACTORY = "-domfactory";           // nores
    private final String      IMPL = "Impl";                        // nores

    //
    public final static int   CLASS_NAME_DEFAULT = 1;
    public final static int   CLASS_NAME_CUSTOM = 2;
    public final static int   CLASS_NAME_MAPPED = 3;

    //
    private MetaDataHandler   metaDataHandler;
    private OtterDocumentNode docNode;

    /**
     * Create a node to use when calling XMLC.
     *
     */
    public OtterXMLCNode(OtterDocumentNode d) {
        docNode = d;
        initMetaDataHandler();
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    public String getXMLCOptionFilePath() {
        return docNode.getXMLCOptionFilePath();
    }

    /**
     * Method declaration
     *
     *
     * @param n
     */
    public void setXMLCOptionFilePath(String n) {
        docNode.setXMLCOptionFilePath(n);
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    public String getXMLCParameters() {
        String       params = new String();
        StringBuffer buf = new StringBuffer();
        int          index = -1;

        params = docNode.getXMLCParameters();
        params = PropUtil.removeQuotes(params);
        return params;
    }

    /**
     * Method declaration
     *
     *
     * @param p
     */
    public void setXMLCParameters(String p) {
        docNode.setXMLCParameters(p);
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    public boolean isSelected() {
        return docNode.isSelected();
    }

    /**
     * Method declaration
     *
     *
     * @param b
     */
    public void setSelected(boolean b) {
        docNode.setSelected(b);
        if (b) {
            setStatic(false);
        }
    }

    public boolean isStatic() {
        return docNode.isStatic();
    }

    public void setStatic(boolean b) {
        docNode.setStatic(b);
        if (b) {
            setSelected(false);
        }
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    public String getFilePath() {
        return docNode.getFilePath();
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    public String getCustomClassName() {
        String custom = getProperty(PropertyKeys.NAME_XMLC_CUSTOM);

        if (custom == null) {
            custom = new String();
        }
        return custom;
    }

    /**
     * Method declaration
     *
     *
     * @param n
     */
    public void setCustomClassName(String n) {
        setProperty(PropertyKeys.NAME_XMLC_CUSTOM, n);
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    public Object getNativeNode() {
        return docNode.getNativeNode();
    }

    /**
     * Method declaration
     *
     *
     * @param o
     */
    public void setNativeNode(Object o) {
        docNode.setNativeNode(o);
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    public OtterProject getProject() {
        return docNode.getProject();
    }

    /**
     * Method declaration
     *
     *
     * @param n
     *
     * @return
     */
    public String getProperty(String n) {
        return docNode.getProperty(n);
    }

    /**
     * Method declaration
     *
     *
     * @param n
     * @param v
     */
    public void setProperty(String n, String v) {
        docNode.setProperty(n, v);
    }

    /**
     * Method declaration
     *
     *
     * @param n
     * @param v
     */
    public void setProperty(String n, int v) {
        docNode.setProperty(n, v);
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    public int getClassNameType() {

        // When getting the project scope overrides
        // local type.
        int    type;
        int    projectScope;
        String stringType = getProperty(PropertyKeys.NAME_XMLC_TYPE);

        projectScope = getProject().getMapScope();
        type = PropUtil.stringToInt(stringType, CLASS_NAME_MAPPED);
        switch (type) {
        case CLASS_NAME_DEFAULT:
        case CLASS_NAME_CUSTOM:
            if (projectScope == OtterProject.MAP_SCOPE_ALL) {
                type = CLASS_NAME_MAPPED;
                setProperty(PropertyKeys.NAME_XMLC_TYPE, type);
            }
            break;
        case CLASS_NAME_MAPPED:
            if (projectScope == OtterProject.MAP_SCOPE_NONE) {
                type = CLASS_NAME_DEFAULT;
                setProperty(PropertyKeys.NAME_XMLC_TYPE, type);
            }
            break;
        }
        return type;
    }

    /**
     * Method declaration
     *
     *
     * @param newType
     */
    public void setClassNameType(int newType) {

        // If conflict exists with project scope,
        // change project scope.
        int oldType = getClassNameType();
        int projectScope = getProject().getMapScope();

        setProperty(PropertyKeys.NAME_XMLC_TYPE, newType);
        syncProjectToType(newType);
        if ((newType != oldType)
                || (newType == OtterXMLCNode.CLASS_NAME_MAPPED)) {
            initClassName();
        }
    }

    /**
     * Method declaration
     *
     *
     * @param forPrint
     *
     * @return
     */
    private String[] getAllParameters(boolean forPrint) {
        StringBuffer    buf = new StringBuffer();
        String[]        parameters = new String[0];
        StringTokenizer tokenizer = null;
        int             count = 0;
        int             index = 0;

        // File - Node
        if (getXMLCParameters() != null) {
            buf.append(getXMLCParameters().trim());
        }
        if (forPrint) {
            buf.append('#');
        } else {
            buf.append(' ');
        }

        // Package - Folder
        OtterNode packNode = null;
        String    packParam = new String();

        packNode = docNode.getParent();
        if (packNode == null || packNode instanceof OtterProject) {

            // skip it.
        } else {
            packParam = packNode.getXMLCParameters();
        }
        if ((packParam != null) && (packParam.trim().length() > 0)) {
            buf.append(packParam);
            if (forPrint) {
                buf.append('#');
            } else {
                buf.append(' ');
            }
        }

        // Project
        if (docNode.getProject().getXMLCParameters() != null) {
            buf.append(docNode.getProject().getXMLCParameters().trim());
        }
        if (forPrint) {
            tokenizer = new StringTokenizer(buf.toString().trim(), POUND);
        } else {
            tokenizer = new StringTokenizer(buf.toString().trim());
        }
        count = tokenizer.countTokens();
        parameters = new String[tokenizer.countTokens()];
        while (tokenizer.hasMoreTokens()) {
            parameters[index] = tokenizer.nextToken();
            index++;
        }

        // Add domFactory if required
        parameters = addDomFactory(parameters);
        return parameters;
    }

    private String[] addDomFactory(String[] in) {
        PathHandle path = null;
        String[]   out = in;
        ArrayList  list = null;
        boolean    found = false;

        path = PathHandle.createPathHandle(docNode.getFilePath());
        for (int i = 0; i < in.length; i++) {
            if (in[i].trim().equalsIgnoreCase(DOMFACTORY)) {
                found = true;
                break;
            }
        }
        if (found) {

            // don't overwride
        } else if (path.hasExtension(Constants.TYPE_HTML)
                   || path.hasExtension(Constants.TYPE_HTM)) {

            // use default
        } else if (path.hasExtension(Constants.TYPE_WML)
                   && ToolBoxInfo.isClassAvailable(ToolBoxInfo.WML_FACTORY)) {
            list = new ArrayList(Arrays.asList(in));
            list.add(DOMFACTORY);
            list.add(ToolBoxInfo.WML_FACTORY);
            out = (String[]) list.toArray(out);
        } else if (path.hasExtension(Constants.TYPE_CHTML)
                   && ToolBoxInfo.isClassAvailable(ToolBoxInfo.CHTML_FACTORY)) {
            list = new ArrayList(Arrays.asList(in));
            list.add(DOMFACTORY);
            list.add(ToolBoxInfo.CHTML_FACTORY);
            out = (String[]) list.toArray(out);
        } else if (path.hasExtension(Constants.TYPE_XHTML)
                   && ToolBoxInfo.isClassAvailable(ToolBoxInfo.XHTML_FACTORY)) {
            list = new ArrayList(Arrays.asList(in));
            list.add(DOMFACTORY);
            list.add(ToolBoxInfo.XHTML_FACTORY);
            out = (String[]) list.toArray(out);
        }
        return out;
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    private String[] getAllOptionFilenames() {
        StringBuffer    buf = new StringBuffer();
        File            file = null;
        PathHandle      handle = null;
        String          filename = new String();
        String[]        filenames = new String[0];
        StringTokenizer tokenizer = null;
        int             count = 0;
        int             index = 0;

        // Project
        handle =
            PathHandle.createPathHandle(docNode.getProject().getXMLCOptionFilePath());
        if (handle.isFile() && handle.hasExtension(Constants.TYPE_XMLC)) {
            buf.append(handle.getPath());
        }

        // Package
        OtterNode parent = docNode.getParent();

        if (parent == null || parent instanceof OtterProject) {

            // skip it
        } else {
            handle =
                PathHandle.createPathHandle(parent.getXMLCOptionFilePath());
            if (handle.isFile() && handle.hasExtension(Constants.TYPE_XMLC)) {
                buf.append('#');
                buf.append(handle.getPath());
            }
        }
        buf.append('#');

        // Node
        handle = PathHandle.createPathHandle(getXMLCOptionFilePath());
        if (handle.isFile() && handle.hasExtension(Constants.TYPE_XMLC)) {
            buf.append(handle.getPath());
        }
        tokenizer = new StringTokenizer(buf.toString().trim(), POUND);
        count = tokenizer.countTokens();
        filenames = new String[tokenizer.countTokens()];
        while (tokenizer.hasMoreTokens()) {
            filenames[index] = tokenizer.nextToken();
            index++;
        }
        return filenames;
    }

    /**
     * Method declaration
     *
     *
     * @param type
     */
    private void syncProjectToType(int type) {
        int projectScope = getProject().getMapScope();

        switch (type) {
        case CLASS_NAME_CUSTOM:
        case CLASS_NAME_DEFAULT:
            if (projectScope == OtterProject.MAP_SCOPE_ALL) {
                getProject().setMapScope(OtterProject.MAP_SCOPE_SELECTED);
            }
            break;
        case CLASS_NAME_MAPPED:
            if (projectScope == OtterProject.MAP_SCOPE_NONE) {
                getProject().setMapScope(OtterProject.MAP_SCOPE_SELECTED);
            }
            break;
        }
    }

    /**
     * Save Exception information for reporting. This lets exceptions from
     * multiple nodes to be reported in a batch.
     *
     * @param e
     * An exception that occured while invoking XMLC.
     */
    public void setException(Throwable e) {
        if (e == null) {
            docNode.setException(null);
        } else {
            docNode.setException(new LocalException(e));
        }
    }

    /**
     * Get an exception that occured when invoking XMLC. Null if the node
     * has not been compiled or was compiled without error.
     */
    public Throwable getException() {
        return docNode.getException();
    }

    public OtterNode getParent() {
        return docNode.getParent();
    }

    public void save() {
        docNode.save();
    }

    /**
     * Get the XMLC options selected this HTML/WML file.
     */
    public MetaDataHandler getMetaDataHandler() {
        return metaDataHandler;
    }

    /**
     * Add the generated Java source file to the IDE project. Do not call
     * this when using Win32 JBuilder as it can't handle the width of source
     * lines generated by XMLC.
     */
    public void replaceGeneratedSource(boolean forRecomp) {
        if ((metaDataHandler.getJavaClassSource().exists())
                && (getProject().isOpenBuild())) {
            int count = 1;

            if (forRecomp) {
                count = 2;
            }
            String[]         source = new String[count];
            OtterNodeFactory factory = null;

            source[0] =
                metaDataHandler.getJavaClassSource().getAbsolutePath();
            factory = getProject().getNodeFactory();
            if (forRecomp) {
                source[1] =
                    metaDataHandler.getJavaInterfaceSource().getAbsolutePath();
            }
            factory.replaceGeneratedSource(getProject(), docNode, source);
        }
    }

    /**
     * Method declaration
     *
     */
    public void initProjectOptions() {
        metaDataHandler.setKeepGeneratedSource(true);    // always keep Java source
        metaDataHandler.setCompileSource(false);    // never compile Java source
				setXMLCOptionFilePath(docNode.getProject().getSourcePath()+getResourceRelativePath()+"/resources/options.xmlc");
        // loging options
        metaDataHandler.setPrintAccessorInfo(getProject().isPrintAccessorInfo());
        metaDataHandler.setPrintDocumentInfo(getProject().isPrintDocumentInfo());
        metaDataHandler.setPrintDOM(getProject().isPrintDOM());
        metaDataHandler.setPrintParseInfo(getProject().isPrintParseInfo());
        metaDataHandler.setVerbose(getProject().isVerbose());

        // Call XMLC with the do not call javac option.
    }

    public String getGenerateToRoot() {
        String       projectGenTo = getProject().getGenerateToDirectory();
        StringBuffer buf = new StringBuffer();

        if (projectGenTo == null) {
            buf.append(docNode.getProject().getSourcePathOf(docNode));
        } else {
            buf.append(projectGenTo);
        }
        return buf.toString();
    }

    /**
     * Initialize the XMLC options.
     *
     * @throws XMLCException
     * Thrown when options are invalid.
     *
     */
    private void initMetaDataHandler() {
        metaDataHandler = XMLCConnectionFactory.createMetaDataHandler();
        initClassName();
        initProjectOptions();
        metaDataHandler.setInputDocument(getFilePath());
        initJavaSourceFile();
    }

    /**
     * Method declaration
     *
     *
     * @exception IOException
     * @exception XMLCException
     */
    private void parseOptions() throws XMLCException, IOException {
        metaDataHandler.parse(getAllOptionFilenames(),
                              getAllParameters(false), (PrintWriter) null,
                              this);
    }

    /**
     * Method declaration
     *
     */
    private void initJavaSourceFile() {
        StringBuffer filename = new StringBuffer();
        String       className = new String();
        String       packageName = new String();
        File         source = null;
        int          index = -1;

        filename.append(getGenToDirectory().toString());
        filename.append(File.separator);
        className = metaDataHandler.getClassName();
        packageName = metaDataHandler.getPackageName();
        if ((packageName == null) || (packageName.trim().length() == 0)) {
            index = -1;
        } else {
            index = className.indexOf(packageName);
        }
        if (index > -1) {
            className =
                className.substring(metaDataHandler.getPackageName().length()
                                    + 1);
        }
        index = className.lastIndexOf('.');
        if (index > -1) {
            className = className.substring(index + 1);
        }
        filename.append(className);
        filename.append('.');
        filename.append(Constants.TYPE_JAVA);
        source = new File(filename.toString());
        metaDataHandler.setJavaClassSource(source, this);
    }

    /**
     * Get the generate to directory and create the directory if needed.
     * This is where the Java source files are created.
     */
    private File getGenToDirectory() {
        String genTo = getGenToPath();
        File   genDir = new File(genTo);

        if (genDir.exists()) {
            if (!genDir.isDirectory()) {
                System.err.println(res.getString("Unable_to_write") + genTo);
            }
        } else {
            genDir.mkdirs();
        }
        return genDir;
    }

    /**
     * Get the generate to directory as a string. This is where XMLC will create
     * Java source files.
     */
    private String getGenToPath() {
        String       packagePath = getPackagePath();
        StringBuffer buf = new StringBuffer();

        buf.append(getGenerateToRoot());
        if (packagePath.length() > 0) {
            buf.append(File.separator);
            buf.append(packagePath);
        }
        return buf.toString();
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    private String getPackagePath() {
        String packPath = new String();

        if (metaDataHandler.getPackageName() != null) {
            packPath = metaDataHandler.getPackageName();
        } else if (metaDataHandler.getClassName() != null) {
            int index = metaDataHandler.getClassName().lastIndexOf('.');

            if (index > -1) {
                packPath = metaDataHandler.getClassName().substring(0, index);
            }
        }
        packPath = packPath.replace('.', File.separatorChar);
        return packPath;
    }

    /**
     * Create a class name for the given HTML/WML file and map the
     * package name as needed. Mapping allows users to  specify
     * a package name to use for all the HTML/WML files in a directory.
     */
    private void initClassName() {
        Mapper mapper = new Mapper(getProject());
        String className = mapper.getClassName(this);

        metaDataHandler.setClassName(className);
    }

    private String getResourceRelativePath() {
        StringBuffer buf = new StringBuffer();
        String       path = new String();

        buf.append(File.separator);
		  path = metaDataHandler.getPackageName();
		  int index = path.lastIndexOf("presentation");
		  if(index !=-1) {
		  		path = path.substring(0,index-1);	
		  }		
        path = path.replace('.', File.separatorChar);
        buf.append(path);
        return buf.toString();
    }


    private String baseOut() {
        StringBuffer buf = new StringBuffer();
        String       path = new String();

        buf.append(getProject().getClassOutputPath());
        buf.append(File.separator);
        path = metaDataHandler.getClassName();
        path = path.replace('.', File.separatorChar);
        buf.append(path);
        return buf.toString();
    }

    /**
     * Get the class file that we expect after XMLC has generated
     * a java file and called javac to compile it.
     */
    public File getClassFile() {
        File         file = null;
        StringBuffer buf = new StringBuffer();

        buf.append(baseOut());
        buf.append('.');
        buf.append(Constants.TYPE_CLASS);
        file = new File(buf.toString());
        return file;
    }

    public File getOptionFileForRecomp() {
        File         file = null;
        StringBuffer buf = new StringBuffer();

        buf.append(baseOut());
        buf.append('.');
        buf.append(Constants.TYPE_XMLC);
        file = new File(buf.toString());
        return file;
    }

    /**
     * True if we can find the Java source file that we expect XMLC to create.
     * This is used to in reporting compilation results.
     */
    public boolean isNewJavaFound() {
        boolean newFound = false;

        if (metaDataHandler.getKeepGeneratedSource()) {
            newFound = metaDataHandler.getJavaClassSource().exists();
        }
        return newFound;
    }

    /**
     * Method declaration
     *
     *
     * @exception IOException
     * @exception XMLCException
     */
    public void preCompile() throws XMLCException, IOException {
        parseOptions();

        // clear out any existing impl or standard classes.
        File         implSource = null;
        StringBuffer implName = new StringBuffer();
        int          index = -1;

        implName.append(metaDataHandler.getJavaClassSource());
        index = implName.toString().lastIndexOf('.');
        implName.insert(index, IMPL);
        implSource = new File(implName.toString());
        if (implSource.exists()) {
            implSource.delete();
        }
        if (metaDataHandler.getJavaClassSource().exists()) {
            metaDataHandler.getJavaClassSource().delete();
        }

        // reset names if needed
        if (metaDataHandler.getRecompilation()) {
            metaDataHandler.setJavaInterfaceSource(metaDataHandler.getJavaClassSource(),
                                                   this);
            metaDataHandler.setJavaClassSource(implSource, this);
        }
        docNode.preCompile();
    }

    /**
     * Class declaration
     *
     * @author Paul Mahar
     */
    class LocalException extends Exception {
        private Throwable baseException = null;

        /**
         * Constructor declaration
         *
         *
         * @param e
         */
        public LocalException(Throwable e) {
            baseException = e;
        }

        /**
         * Method declaration
         *
         *
         * @return
         */
        public String getMessage() {
            StringBuffer buf = new StringBuffer();
            String[]     parameters = getAllParameters(true);
            String[]     filenames = getAllOptionFilenames();

            buf.append(baseException.getMessage());
            buf.append('\n');
            if (parameters.length >= 1) {
                buf.append(Constants.TAB2);
                buf.append(res.getString("Command_line"));
                buf.append('\n');
                for (int i = 0; i < parameters.length; i++) {
                    buf.append(Constants.TAB4);
                    buf.append(parameters[i]);
                    buf.append('\n');
                }
            }
            if (filenames.length >= 1) {
                buf.append(Constants.TAB2);
                if (filenames.length >= 2) {
                    buf.append(res.getString("Option_files"));
                } else {
                    buf.append(res.getString("Option_file"));
                }
                buf.append('\n');
                for (int i = 0; i < filenames.length; i++) {
                    buf.append(Constants.TAB4);
                    buf.append(filenames[i]);
                }
            }
            return buf.toString().trim();
        }

    }
}
