/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.kelp.common.swing;

// Kelp imports
import org.enhydra.kelp.common.event.SelectionPanelEvent;
import org.enhydra.kelp.common.event.SelectionPanelListener;
import org.enhydra.kelp.common.node.OtterFileNode;

// Standard imports
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.ResourceBundle;
import java.util.ArrayList;
import java.util.Arrays;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.DefaultListModel;
import java.awt.*;
import java.beans.*;

//
public class FileNodeSelectionPanel extends JPanel {

    //
    public static ResourceBundle     res =
        ResourceBundle.getBundle("org.enhydra.kelp.common.Res");    // nores

    // strings not to be resourced
    private final String             SELECT = ">";
    private final String             SELECT_ALL = ">>";
    private final String             REMOVE = "<";
    private final String             REMOVE_ALL = "<<";

    //
    private JLabel                   labelAvailable;
    private JLabel                   labelSelected;
    private OtterFileNode[]          nodes = new OtterFileNode[0];
    private LocalButtonListener      buttonListener = null;
    private JScrollPane              scrollAvailable;
    private JScrollPane              scrollSelected;
    private JList                    listAvailable;
    private JList                    listSelected;
    private JButton                  buttonAdd;
    private JButton                  buttonAddAll;
    private JButton                  buttonRemove;
    private JButton                  buttonRemoveAll;
    private GridBagLayout            layoutSelect;
    private JPanel                   panelSelect;
    private JPanel                   panelOption;
    private DefaultListModel         modelAvailable = new DefaultListModel();
    private DefaultListModel         modelSelected = new DefaultListModel();
    private GridBagLayout            layoutOption;
    private JCheckBox                checkShowFull;
    private SelectionPanelListener[] selectionPanelListeners =
        new SelectionPanelListener[0];
    private GridBagLayout            layoutMain;

    /**
     * Constructor declaration
     *
     */
    public FileNodeSelectionPanel() {
        try {
            jbInit();
            pmInit();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    // override JPanel
    public void setEnabled(boolean enable) {
        buttonAdd.setEnabled(enable);
        buttonAddAll.setEnabled(enable);
        buttonRemove.setEnabled(enable);
        buttonRemoveAll.setEnabled(enable);
        listAvailable.setEnabled(enable);
        listSelected.setEnabled(enable);
        checkShowFull.setEnabled(enable);
        super.setEnabled(enable);
    }

    public void clearAll() {
        buttonAdd.removeActionListener(buttonListener);
        buttonAddAll.removeActionListener(buttonListener);
        buttonRemove.removeActionListener(buttonListener);
        buttonRemoveAll.removeActionListener(buttonListener);
        checkShowFull.removeActionListener(buttonListener);
        listAvailable.setModel(new DefaultListModel());
        listSelected.setModel(new DefaultListModel());
        removeAll();
        modelAvailable = null;
        modelSelected = null;
        nodes = null;
        selectionPanelListeners = null;
        buttonListener = null;
    }

    // event methods
    public synchronized void addSelectionPanelListener(SelectionPanelListener l) {
        ArrayList list = null;

        list = new ArrayList(Arrays.asList(selectionPanelListeners));
        if (!list.contains(l)) {
            list.add(l);
            list.trimToSize();
            selectionPanelListeners = new SelectionPanelListener[list.size()];
            selectionPanelListeners =
                (SelectionPanelListener[]) list.toArray(selectionPanelListeners);
        }
        list.clear();
    }

    public synchronized void removeSelectionPanelListener(SelectionPanelListener l) {
        ArrayList list = null;

        list = new ArrayList(Arrays.asList(selectionPanelListeners));
        if (list.contains(l)) {
            list.remove(l);
            list.trimToSize();
            selectionPanelListeners = new SelectionPanelListener[list.size()];
            selectionPanelListeners =
                (SelectionPanelListener[]) list.toArray(selectionPanelListeners);
        }
        list.clear();
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    public OtterFileNode[] getNodes() {
        return nodes;
    }

    /**
     * Method declaration
     *
     *
     * @param source
     */
    public void setNodes(OtterFileNode[] n) {
        nodes = n;
        fillLists();
        fireSelectionEvent();
    }

    public String[] getSelectText() {
        String[] s = new String[2];

        s[0] = labelAvailable.getText();
        s[1] = labelSelected.getText();
        return s;
    }

    public void setSelectText(String[] s) {
        labelAvailable.setText(s[0]);
        labelSelected.setText(s[1]);
    }

    //
    // PROTECTED
    //
    protected void fillLists() {
        int           count = getNodes().length;
        OtterFileNode source = null;

        if (getModelSelected().getSize() > 0) {
            getModelSelected().clear();
        }
        if (getModelAvailable().getSize() > 0) {
            getModelAvailable().clear();
        }
        for (int i = 0; i < count; i++) {
            LocalListNode listNode = null;

            source = getNodes()[i];
            listNode = new LocalListNode(source.getFilePath());
            if (source.isSelected()) {
                getModelSelected().addElement(listNode);
            } else {
                getModelAvailable().addElement(listNode);
            }
        }
    }

    protected DefaultListModel getModelSelected() {
        return modelSelected;
    }

    protected DefaultListModel getModelAvailable() {
        return modelAvailable;
    }

    //
    // PRIVATE
    //
    private void pmInit() {
        buttonListener = new LocalButtonListener();
        buttonAdd.addActionListener(buttonListener);
        buttonAddAll.addActionListener(buttonListener);
        buttonRemove.addActionListener(buttonListener);
        buttonRemoveAll.addActionListener(buttonListener);
        checkShowFull.addActionListener(buttonListener);
        checkShowFull.setSelected(false);
    }

    private void jbInit() throws Exception {
        layoutSelect =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        panelSelect = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                                 JPanel.class.getName());
        panelOption = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                                 JPanel.class.getName());
        scrollAvailable =
            (JScrollPane) Beans.instantiate(getClass().getClassLoader(),
                                            JScrollPane.class.getName());
        scrollSelected =
            (JScrollPane) Beans.instantiate(getClass().getClassLoader(),
                                            JScrollPane.class.getName());
        labelAvailable =
            (JLabel) Beans.instantiate(getClass().getClassLoader(),
                                       JLabel.class.getName());
        labelSelected =
            (JLabel) Beans.instantiate(getClass().getClassLoader(),
                                       JLabel.class.getName());
        listAvailable = (JList) Beans.instantiate(getClass().getClassLoader(),
                                                  JList.class.getName());
        listSelected = (JList) Beans.instantiate(getClass().getClassLoader(),
                                                 JList.class.getName());
        buttonAdd = (JButton) Beans.instantiate(getClass().getClassLoader(),
                                                JButton.class.getName());
        buttonAddAll =
            (JButton) Beans.instantiate(getClass().getClassLoader(),
                                        JButton.class.getName());
        buttonRemove =
            (JButton) Beans.instantiate(getClass().getClassLoader(),
                                        JButton.class.getName());
        buttonRemoveAll =
            (JButton) Beans.instantiate(getClass().getClassLoader(),
                                        JButton.class.getName());
        layoutOption =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        checkShowFull =
            (JCheckBox) Beans.instantiate(getClass().getClassLoader(),
                                          JCheckBox.class.getName());
        layoutMain =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        listAvailable.setModel(modelAvailable);
        listSelected.setModel(modelSelected);
        labelAvailable.setText(res.getString("Available_Pages"));
        labelSelected.setText(res.getString("Selected_Pages"));
        buttonAdd.setMargin(new Insets(1, 10, 1, 10));
        buttonAdd.setText(SELECT);
        buttonAddAll.setMargin(new Insets(1, 10, 1, 10));
        buttonAddAll.setText(SELECT_ALL);
        buttonRemove.setMargin(new Insets(1, 10, 1, 10));
        buttonRemove.setText(REMOVE);
        buttonRemoveAll.setMargin(new Insets(1, 10, 1, 10));
        buttonRemoveAll.setText(REMOVE_ALL);
        checkShowFull.setText(res.getString("Show_full"));
        scrollAvailable.getViewport().add(listAvailable);
        scrollSelected.getViewport().add(listSelected);
        panelSelect.setLayout(layoutSelect);
        panelSelect.add(labelAvailable,
                        new GridBagConstraints(0, 0, 1, 1, 0.1, 0.0,
                                               GridBagConstraints.WEST,
                                               GridBagConstraints.HORIZONTAL,
                                               new Insets(5, 5, 2, 5), 0, 0));
        panelSelect.add(scrollAvailable,
                        new GridBagConstraints(0, 1, 1, 4, 0.4, 0.0,
                                               GridBagConstraints.WEST,
                                               GridBagConstraints.BOTH,
                                               new Insets(3, 3, 5, 3), 0, 0));
        panelSelect.add(buttonAdd,
                        new GridBagConstraints(1, 1, 1, 1, 0.1, 0.0,
                                               GridBagConstraints.CENTER,
                                               GridBagConstraints.NONE,
                                               new Insets(8, 2, 3, 2), 14,
                                               0));
        panelSelect.add(buttonAddAll,
                        new GridBagConstraints(1, 2, 1, 1, 0.1, 0.0,
                                               GridBagConstraints.CENTER,
                                               GridBagConstraints.NONE,
                                               new Insets(3, 2, 3, 2), 6, 0));
        panelSelect.add(buttonRemove,
                        new GridBagConstraints(1, 3, 1, 1, 0.1, 0.0,
                                               GridBagConstraints.CENTER,
                                               GridBagConstraints.NONE,
                                               new Insets(3, 2, 3, 2), 14,
                                               0));
        panelSelect.add(buttonRemoveAll,
                        new GridBagConstraints(1, 4, 1, 1, 0.1, 0.0,
                                               GridBagConstraints.CENTER,
                                               GridBagConstraints.NONE,
                                               new Insets(3, 2, 8, 2), 6, 0));
        panelSelect.add(labelSelected,
                        new GridBagConstraints(2, 0, 1, 1, 0.1, 0.0,
                                               GridBagConstraints.WEST,
                                               GridBagConstraints.HORIZONTAL,
                                               new Insets(5, 5, 2, 5), 0, 0));
        panelSelect.add(scrollSelected,
                        new GridBagConstraints(2, 1, 1, 4, 0.4, 0.0,
                                               GridBagConstraints.EAST,
                                               GridBagConstraints.BOTH,
                                               new Insets(3, 3, 5, 3), 0, 0));
        panelOption.setLayout(layoutOption);
        panelOption.add(checkShowFull,
                        new GridBagConstraints(0, 0, 1, 1, 0.1, 0.0,
                                               GridBagConstraints.WEST,
                                               GridBagConstraints.HORIZONTAL,
                                               new Insets(5, 5, 0, 5), 0, 0));
        this.setLayout(layoutMain);
        this.add(panelSelect,
                 new GridBagConstraints(0, 0, 1, 1, 0.1, 0.0,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.BOTH,
                                        new Insets(0, 0, 0, 0), 0, 0));
        this.add(panelOption,
                 new GridBagConstraints(0, 1, 1, 1, 0.1, 0.0,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.HORIZONTAL,
                                        new Insets(0, 0, 0, 0), 0, 0));
    }

    private void addSelected() {
        Object[] elements = listAvailable.getSelectedValues();

        for (int i = 0; i < elements.length; i++) {
            LocalListNode listNode = (LocalListNode) elements[i];

            modelSelected.addElement(listNode);
            modelAvailable.removeElement(listNode);
            for (int j = 0; j < getNodes().length; j++) {
                if (getNodes()[j].getFilePath().equals(listNode.getFullname())) {
                    getNodes()[j].setSelected(true);
                }
            }
        }
        fireSelectionEvent();
    }

    private void addAllElements() {
        for (int i = 0; i < modelAvailable.size(); i++) {
            modelSelected.addElement(modelAvailable.elementAt(i));
        }
        modelAvailable.removeAllElements();
        for (int i = 0; i < getNodes().length; i++) {
            getNodes()[i].setSelected(true);
        }
        fireSelectionEvent();
    }

    private void removeSelected() {
        Object[] elements = listSelected.getSelectedValues();

        for (int i = 0; i < elements.length; i++) {
            LocalListNode listNode = (LocalListNode) elements[i];

            modelAvailable.addElement(listNode);
            modelSelected.removeElement(listNode);
            for (int j = 0; j < getNodes().length; j++) {
                if (getNodes()[j].getFilePath().equals(listNode.getFullname())) {
                    getNodes()[j].setSelected(false);
                }
            }
        }
        fireSelectionEvent();
    }

    private void removeAllElements() {
        for (int i = 0; i < modelSelected.size(); i++) {
            modelAvailable.addElement(modelSelected.elementAt(i));
        }
        modelSelected.removeAllElements();
        for (int i = 0; i < getNodes().length; i++) {
            getNodes()[i].setSelected(false);
        }
        fireSelectionEvent();
    }

    private void fireSelectionEvent() {
        SelectionPanelEvent event = null;

        event = new SelectionPanelEvent(this, modelAvailable, modelSelected);
        for (int i = 0; i < selectionPanelListeners.length; i++) {
            selectionPanelListeners[i].onSelection(event);
        }
    }

    private class LocalButtonListener implements ActionListener {
        public void actionPerformed(ActionEvent event) {
            Object source = event.getSource();

            if (source == buttonAdd) {
                addSelected();
            } else if (source == buttonAddAll) {
                addAllElements();
            } else if (source == buttonRemove) {
                removeSelected();
            } else if (source == buttonRemoveAll) {
                removeAllElements();
            } else if (source == checkShowFull) {
                checkShowFull.isSelected();
                fillLists();
            }
        }

    }

    //
    private class LocalListNode {
        private String full = new String();

        public LocalListNode(String fullName) {
            full = fullName;
        }

        public String toString() {
            String s = full;

            if (!checkShowFull.isSelected()) {
                File f = new File(full);

                s = f.getName();
            }
            return s;
        }

        public String getFullname() {
            return full;
        }

    }
}
