/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.kelp.common.xmlc;

// ToolBox imports
import org.enhydra.tool.common.ButtonPanel;

// Standard imports
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.*;
import javax.swing.*;

//
public class XMLCButtonPanel extends ButtonPanel {

    //
    private GridBagLayout       layoutMain;
    private JButton             buttonCompile = null;
    private JButton             buttonClose = null;
    private JButton             buttonHelp = null;
    private JButton             buttonAbout = null;
    private JPanel              panelFiller = null;
    private LocalButtonListener buttonListener = null;

    public final static String[] getOptions() {
        final String[] options = {
            ButtonPanel.COMMAND_COMPILE, ButtonPanel.COMMAND_CLOSE,
            ButtonPanel.COMMAND_HELP, ButtonPanel.COMMAND_ABOUT
        };

        return options;
    }

    public XMLCButtonPanel() {
        try {
            jbInit();
            pmInit();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    // override ButtonPanel
    public void removeHelp() {
        remove(buttonHelp);
        invalidate();
        doLayout();
    }

    //
    //
    private void pmInit() {
        buttonListener = new LocalButtonListener();
        buttonCompile.addActionListener(buttonListener);
        buttonClose.addActionListener(buttonListener);
        buttonAbout.addActionListener(buttonListener);
        buttonHelp.addActionListener(buttonListener);
        buttonCompile.setText(ButtonPanel.COMMAND_COMPILE);
        buttonClose.setText(ButtonPanel.COMMAND_CLOSE);
        buttonAbout.setText(ButtonPanel.COMMAND_ABOUT);
        buttonHelp.setText(ButtonPanel.COMMAND_HELP);

        //
        buttonCompile.setActionCommand(buttonCompile.getText());
        buttonClose.setActionCommand(buttonClose.getText());
        buttonAbout.setActionCommand(buttonAbout.getText());
        buttonHelp.setActionCommand(buttonHelp.getText());
    }

    private void jbInit() throws Exception {
        layoutMain =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        buttonCompile =
            (JButton) Beans.instantiate(getClass().getClassLoader(),
                                        JButton.class.getName());
        buttonClose = (JButton) Beans.instantiate(getClass().getClassLoader(),
                                                  JButton.class.getName());
        buttonAbout = (JButton) Beans.instantiate(getClass().getClassLoader(),
                                                  JButton.class.getName());
        buttonHelp = (JButton) Beans.instantiate(getClass().getClassLoader(),
                                                 JButton.class.getName());
        panelFiller = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                                 JPanel.class.getName());
        buttonCompile.setText("COMPILE");
        buttonCompile.setEnabled(false);
        buttonClose.setText("CLOSE");
        buttonAbout.setText("ABOUT");
        buttonHelp.setText("HELP");
        this.setLayout(layoutMain);
        this.add(buttonCompile,
                 new GridBagConstraints(2, 0, 1, 1, 0.1, 0.0,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.NONE,
                                        new Insets(5, 0, 5, 5), 0, 0));
        this.add(buttonClose,
                 new GridBagConstraints(3, 0, 1, 1, 0.1, 0.0,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.NONE,
                                        new Insets(5, 5, 5, 5), 0, 0));
        this.add(buttonAbout,
                 new GridBagConstraints(5, 0, 1, 1, 0.1, 0.0,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.NONE,
                                        new Insets(5, 5, 5, 5), 0, 0));
        this.add(buttonHelp,
                 new GridBagConstraints(4, 0, 1, 1, 0.1, 0.0,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.NONE,
                                        new Insets(5, 5, 5, 5), 0, 0));
        this.add(panelFiller,
                 new GridBagConstraints(1, 0, 1, 1, 1.0, 0.0,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.HORIZONTAL,
                                        new Insets(0, 0, 0, 0), 50, 0));
    }

    private class LocalButtonListener implements ActionListener {

        /**
         * ActionListener implementation.
         *
         * @param event
         * Event that triggered this listener.
         *
         */
        public void actionPerformed(ActionEvent e) {
            Object      source = null;
            ActionEvent event = null;

            source = e.getSource();
            if (source == buttonCompile) {
                event = new ActionEvent(buttonCompile, ButtonPanel.COMPILE,
                                        buttonCompile.getActionCommand());
            } else if (source == buttonClose) {
                event = new ActionEvent(buttonClose, ButtonPanel.CLOSE,
                                        buttonClose.getActionCommand());
            } else if (source == buttonHelp) {
                event = new ActionEvent(buttonHelp, ButtonPanel.HELP,
                                        buttonHelp.getActionCommand());
            } else if (source == buttonAbout) {
                event = new ActionEvent(buttonAbout, ButtonPanel.ABOUT,
                                        buttonAbout.getActionCommand());
            }
            if (event != null) {
                notifyActionListeners(event);
            }
        }

    }
}
