/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Lisa Reese
 *
 */

package org.enhydra.kelp.forte;

import org.openide.options.SystemOption;
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;

import org.enhydra.kelp.common.node.PropertyKeys;
import org.enhydra.kelp.common.PropUtil;

import java.util.Hashtable;
import java.io.*;
/** Options for something or other.
 *
 * @author  rees0234
 */
public class DeploySettings extends SystemOption implements PropertyKeys{

    static final long serialVersionUID = 8806955980938332585L;
    
    protected Hashtable nodeProps = null;

    public String displayName () {
        return NbBundle.getMessage (DeploySettings.class, "LBL_deploy_settings");
    }

    public HelpCtx getHelpCtx () {
        return HelpCtx.DEFAULT_HELP;
        // If you provide context help then use:
        // return new HelpCtx (DeploySettings.class);
    }

    /** Default instance of this system option, for the convenience of associated classes. */
    public static DeploySettings getDefault ()
    {
        return (DeploySettings) findObject (DeploySettings.class, true);
    }

   public void writeExternal (ObjectOutput out) throws IOException {
        out.writeObject(propertyValue(DEPLOY_CONTENT_TYPES));
        out.writeObject(propertyValue(DEPLOY_CONTENT_ENABLE));
        out.writeObject(propertyValue(DEPLOY_TYPE));
        out.writeObject(propertyValue(DEPLOY_BUILD));
        out.writeObject(propertyValue(DEPLOY_ECHO));
        out.writeObject(propertyValue(DEPLOY_OVERWRITE));
        out.writeObject(propertyValue(DEPLOY_PATH_ROOT));
        out.writeObject(propertyValue(DEPLOY_PATH_BOOTSTRAP));
        out.writeObject(propertyValue(DEPLOY_INPUT_FILTER));
        out.writeObject(propertyValue(DEPLOY_INPUT_ENABLE));
        out.writeObject(propertyValue(DEPLOY_PATH_INPUT));
        out.writeObject(propertyValue(DEPLOY_PATH_RESOURCE));
        out.writeObject(propertyValue(DEPLOY_TABLE_LENGTH));
        out.writeObject(propertyValue(DEPLOY_TABLE_FIND));
        out.writeObject(propertyValue(DEPLOY_TABLE_REPLACE));
        out.writeObject(propertyValue(DEPLOY_STARTUP_JAVA));
        out.writeObject(propertyValue(DEPLOY_STARTUP_RUN));
        out.writeObject(propertyValue(ARCHIVE_PROPERTIES));
        out.writeObject(propertyValue(AUTO_DEPLOY_FILE));
        out.writeObject(nodeProps);
        out.writeObject(propertyValue(NAME_WORKING_DIR));
        out.writeObject(propertyValue(NAME_AUTO_PACK));
        out.writeObject(propertyValue(NAME_LIBRARIES));
        out.writeObject(propertyValue(NAME_GENTO));
    }

   public void readExternal (ObjectInput in) throws IOException, ClassNotFoundException {
        setProperty(DEPLOY_CONTENT_TYPES, (String)in.readObject());
        setProperty(DEPLOY_CONTENT_ENABLE, (String)in.readObject());
        setProperty(DEPLOY_TYPE, (String)in.readObject());
        setProperty(DEPLOY_BUILD, (String)in.readObject());
        setProperty(DEPLOY_ECHO, (String)in.readObject());
        setProperty(DEPLOY_OVERWRITE, (String)in.readObject());
        setProperty(DEPLOY_PATH_ROOT, (String)in.readObject());
        setProperty(DEPLOY_PATH_BOOTSTRAP, (String)in.readObject());
        setProperty(DEPLOY_INPUT_FILTER, (String)in.readObject());
        setProperty(DEPLOY_INPUT_ENABLE, (String)in.readObject());
        setProperty(DEPLOY_PATH_INPUT, (String)in.readObject());
        setProperty(DEPLOY_PATH_RESOURCE, (String)in.readObject());
        setProperty(DEPLOY_TABLE_LENGTH, (String)in.readObject());
        setProperty(DEPLOY_TABLE_FIND, (String)in.readObject());
        setProperty(DEPLOY_TABLE_REPLACE, (String)in.readObject());
        setProperty(DEPLOY_STARTUP_JAVA, (String)in.readObject());
        setProperty(DEPLOY_STARTUP_RUN, (String)in.readObject());
        setProperty(ARCHIVE_PROPERTIES, (String)in.readObject());
        setProperty(AUTO_DEPLOY_FILE, (String)in.readObject());
        try {
            nodeProps = (Hashtable)in.readObject();
        }  catch (OptionalDataException e) {
            if (nodeProps == null)
                nodeProps = new Hashtable();
        }
        setProperty(NAME_WORKING_DIR, (String)in.readObject());
        setProperty(NAME_AUTO_PACK, (String)in.readObject());
        setProperty(NAME_LIBRARIES, (String)in.readObject());
        setProperty(NAME_GENTO, (String)in.readObject());
    }

    public void setProperty(String key, String value)
    {
        putProperty(key, value, true);
    }

    public String propertyValue(String key)
    {
        return (String)getProperty(key);
    }

    public void setNodeProperty(String key, String prop)
    {
        if (nodeProps == null)
            nodeProps = new Hashtable();

        if (prop != null)
            try {
                nodeProps.put(key, prop);
            }catch (NullPointerException e) {
                System.err.println(e);
            }
        else
            nodeProps.remove(key);
    }

    public String getNodeProperty(String key)
    {
        if (nodeProps != null)
        {
            String res = (String)nodeProps.get(key);
            return res;
        }

        return null;
    }


    public String getDeployRoot()
    {
        return propertyValue(DEPLOY_PATH_ROOT);
    }
    
    public void setDeployRoot(String root)
    {
        setProperty(DEPLOY_PATH_ROOT, root);
    }
    
    public boolean isGlobal()
    {
        return false;
    }
}
