/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Lisa Reese
 *
 */

package org.enhydra.kelp.forte.node;

import org.openide.actions.*;
import org.openide.cookies.*;
import org.openide.loaders.*;
import org.openide.nodes.*;
import org.openide.util.HelpCtx;
import org.openide.TopManager;
import org.openide.text.EditorSupport;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileSystem;
import org.openide.filesystems.FileStateInvalidException;

//import org.netbeans.modules.html.*;

import org.enhydra.kelp.forte.services.*;
import org.enhydra.kelp.forte.XMLCSettings;

import org.enhydra.kelp.common.PathUtil;

import java.io.File;


/** Represents a XMLC object in the Repository.
 *
 * @author  rees0234
 */
public class XMLCDataObject extends MultiDataObject {

    private static final long serialVersionUID = -556335052709447405L;

    protected XMLCSettings settings = null;
    
    public XMLCDataObject (FileObject pf, XMLCDataLoader loader) throws DataObjectExistsException
    {
        super (pf, loader);
        init ();
    }

    private void init ()
    {
        CookieSet cookies = getCookieSet ();
        cookies.add(new XMLCEditorSupport(this));
        cookies.add(new ViewSupport(getPrimaryEntry()));
        settings = XMLCSettings.getDefault();
    }

    public HelpCtx getHelpCtx ()
    {
        return HelpCtx.DEFAULT_HELP;
        // If you add context help, change to:
        // return new HelpCtx (XMLCDataObject.class);
    }

    protected Node createNodeDelegate ()
    {
        return new XMLCDataNode (this);
    }

        /** Creates a Node.Cookie of given class. The method
     * may be called more than once.
     */
   public Node.Cookie createCookie(Class klass)
   {
//        if (CompilerCookie.class.isAssignableFrom(klass)) {
//            MultiDataObject.Entry entry = getPrimaryEntry();
//            return new XMLCCompilerSupport.Compile(entry);
//            }
         if (klass == XMLCEditorSupport.class)
         {
            return new XMLCEditorSupport(this);
         }
         else if (klass == ViewSupport.class) 
         {
            return new ViewSupport(getPrimaryEntry());
         }

        return null;
    }
    
    public String getPath()
    {
        FileObject file = getPrimaryFile();
        StringBuffer res = new StringBuffer();
        FileSystem fs = null;
        try
        {
           fs = file.getFileSystem();
        } catch (FileStateInvalidException e)
        {
           System.err.println(e);
           return "invalid";
        }

        res.append(fs.getSystemName());
        String pack = file.getPackageNameExt(File.separatorChar, '.');
        res.append(File.separatorChar);
        res.append(pack);
         
        return res.toString();
    }

    public String getProperty(String key)
    {
       String path = PathUtil.compressPathRelativeToProject(new ForteProject(), getPath());
       return (String)settings.getNodeProperty(path + key);
    }

    public void setProperty(String key, String value)
    {
        String path = PathUtil.compressPathRelativeToProject(new ForteProject(), getPath());
        settings.setNodeProperty(path + key, value);
    }
    
    static final class ViewSupport implements ViewCookie 
    {
        /** entry */   
      private MultiDataObject.Entry primary;
        
        /** Constructs new ViewSupport */
      public ViewSupport(MultiDataObject.Entry primary) {
            this.primary = primary;
        }
        
         public void view () {
             try {
                 TopManager.getDefault ().showUrl (primary.getFile ().getURL ());
             } catch (FileStateInvalidException e) {
                 e.printStackTrace(System.err);
             }
         }
    }
    
}
