/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.kelp.jbuilder.properties;

// JBuilder
import com.borland.primetime.properties.PropertyPage;
import com.borland.primetime.help.HelpTopic;
import com.borland.primetime.node.TextFileNode;
import com.borland.primetime.help.ZipHelpTopic;

// Toolbox
import org.enhydra.tool.common.ToolException;

// AddinCore
import org.enhydra.kelp.KelpInfo;
import org.enhydra.kelp.common.node.OtterTemplateNode;
import org.enhydra.kelp.common.properties.TemplateNodePropertyPanel;

// AddinJBuilder
import org.enhydra.kelp.jbuilder.node.PrimeTemplateNode;

// JDK
import java.awt.BorderLayout;
import java.lang.ref.WeakReference;

//
public class TemplateNodePropertyPage extends PropertyPage {

    // string not to be resourced
    private final String              HELP_LINK = KelpInfo.getAddinHelpURL()
            + "#confnode";    // nores

    //
    private TemplateNodePropertyPanel nodePanel = null;
    private WeakReference             nodeRef = null;
    private BorderLayout              layoutMain = null;

    public TemplateNodePropertyPage(Object topic) {
        if (topic instanceof TextFileNode) {
            try {
                jbInit();
                pmInit((TextFileNode) topic);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    public void readProperties() {
        if (nodeRef != null) {
            nodePanel.read((OtterTemplateNode) nodeRef.get());
        }
    }

    public HelpTopic getHelpTopic() {
        ZipHelpTopic t = new ZipHelpTopic(null, HELP_LINK);

        return t;
    }

    public void writeProperties() {
        nodePanel.save();
    }

    public TemplateNodePropertyPage() {
        try {
            jbInit();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    //
    private void pmInit(TextFileNode nativeNode) throws ToolException {
        OtterTemplateNode otterNode;

        otterNode = new PrimeTemplateNode(nativeNode);
        nodePanel.read(otterNode);
        nodeRef = new WeakReference(otterNode);
    }

    private void jbInit() throws Exception {
        layoutMain = new BorderLayout();
        nodePanel = new TemplateNodePropertyPanel();
        nodePanel.setIconView(true);
        this.setLayout(layoutMain);
        this.add(nodePanel, BorderLayout.CENTER);
    }

}
