/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.kelp.jbuilder.build;

// JBuilder imports
import com.borland.primetime.build.BuildTask;
import com.borland.primetime.build.BuildProcess;
import com.borland.primetime.node.Project;
import com.borland.primetime.node.TextFileNode;
import com.borland.jbuilder.node.JBProject;

// Kelp imports
import org.enhydra.kelp.common.event.WriteEvent;
import org.enhydra.kelp.common.event.WriteListener;
import org.enhydra.kelp.common.node.OtterXMLCNode;
import org.enhydra.kelp.common.node.OtterProject;
import org.enhydra.kelp.common.xmlc.XMLCNodeBuilder;
import org.enhydra.kelp.jbuilder.node.PrimeProject;

// Standard imports
import java.util.Vector;
import java.util.ResourceBundle;

/**
 * BuildTask implementation for calling XMLC.
 *
 */
public class XMLCBuildTask extends BuildTask implements WriteListener {
    static ResourceBundle      res =
        ResourceBundle.getBundle("org.enhydra.kelp.jbuilder.Res");    // nores
    public static final Object SHARED_KEY = new String();
    private BuildProcess       buildProcess;
    private Vector             nodeVector = new Vector();
    private Vector             writeListeners = new Vector();
    private XMLCNodeBuilder    nodeBuilder;
    private int                messageCount = 0;
    private int                allNodeCount = 0;

    /**
     * Constructor declaration
     *
     */
    public XMLCBuildTask() {}

    public void increment() {
        allNodeCount++;
    }

    /**
     * Generate the DOM class using XMLC.
     */
    public boolean build(BuildProcess buildProcess) {
        this.buildProcess = buildProcess;
        //OtterProject  otterProject = new PrimeProject((JBProject) project);
        int           nodeCount = nodeVector.size();
        int           compileCount = nodeCount;
        OtterXMLCNode current = null;

        if ((allNodeCount < 3) ) {

            // If build not selected, only continue for single node build
        } else {
            nodeVector.clear();
            nodeCount = 0;
            compileCount = 0;
        }

        // remove upToDate nodes from vector if not a rebuild.
       // if (!rebuild) {
            for (int i = 0; i < nodeCount; i++) {
                current = (OtterXMLCNode) nodeVector.elementAt(i);
                if (upToDate(current)) {
                    nodeVector.setElementAt(new Object(), i);
                    compileCount--;
                }
            }
      //  }
        OtterXMLCNode[] nodes = new OtterXMLCNode[compileCount];

        compileCount = 0;
        for (int i = 0; i < nodeCount; i++) {
            if (nodeVector.elementAt(i) instanceof OtterXMLCNode) {
                if (compileCount < nodes.length) {
                    nodes[compileCount] =
                        (OtterXMLCNode) nodeVector.elementAt(i);
                    compileCount++;
                }
            }
        }
        WriteListener[] w = new WriteListener[1];
        boolean         recomp = false;

        w[0] = this;
        buildProcess.fireBuildStatus(res.getString("Generating_DOM"), true);
        nodeBuilder = new XMLCNodeBuilder(this, true);
        //nodeBuilder.setProject(otterProject);
        nodeBuilder.setNodes(nodes);
        nodeBuilder.setEcho(false);
        nodeBuilder.buildInCurrentThread();
        for (int i = 0; i < nodes.length; i++) {
            recomp = nodes[i].getMetaDataHandler().getRecompilation();
            nodes[i].replaceGeneratedSource(recomp);
        }
        return true;
    }

    /**
     * Build messages.
     *
     */
    public void onWrite(WriteEvent e) {
        if (buildProcess != null && messageCount < 200) {
            buildProcess.fireBuildMessage(e.getString(), e.getString());
            messageCount++;
            if (messageCount == 200) {
                String message = res.getString("Reached_maximum_XMLC");

                buildProcess.fireBuildMessage(message, message);
            }
        }
    }

    /**
     * Collect list of nodes to build.
     *
     */
    public void addNode(OtterXMLCNode n) {
        nodeVector.addElement(n);
    }

    /**
     * Check if node up to date.
     *
     */
    private boolean upToDate(OtterXMLCNode xmlcNode) {
        boolean      built = false;
        TextFileNode docNode = null;

        docNode = (TextFileNode) xmlcNode.getNativeNode();
        if (docNode.isModified()) {
            try {
                docNode.save();
            } catch (Exception e) {
                e.printStackTrace();
            }
        } else if (!xmlcNode.getClassFile().exists()) {}
        else if (xmlcNode.getClassFile().lastModified()
                 > docNode.getUrl().getFileObject().lastModified()) {
            built = true;
        }
        return built;
    }

}
