/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 * Travis Shook
 *
 */
package org.enhydra.kelp.jdeveloper.node;

// ToolBox
import org.enhydra.tool.ToolBoxInfo;
import org.enhydra.tool.common.FileUtil;
import org.enhydra.tool.common.PathHandle;

// JDeveloper
import borland.jbuilder.addin.Container;
import borland.jbuilder.addin.HtmlNode;
import borland.jbuilder.addin.Node;
import borland.jbuilder.addin.Project;
import borland.jbuilder.addin.Url;

// AddinCore
import org.enhydra.kelp.common.Constants;
import org.enhydra.kelp.common.PathUtil;
import org.enhydra.kelp.common.deployer.RunParameters;
import org.enhydra.kelp.common.node.OtterDocumentNode;
import org.enhydra.kelp.common.node.OtterTemplateNode;
import org.enhydra.kelp.common.node.OtterTextFileNode;
import org.enhydra.kelp.common.node.OtterProject;
import org.enhydra.kelp.common.node.OtterNode;
import org.enhydra.kelp.common.node.OtterFileNode;
import org.enhydra.kelp.common.node.OtterNodeFactory;
import org.enhydra.kelp.common.node.PropertyKeys;

// JDK
import java.io.File;
import java.util.StringTokenizer;
import java.util.ArrayList;

//
public class OracleProject extends OtterProject implements OtterNode {

    // string not to be resourced
    private final String      ROOT_CONTAINER = "RootContainer";      // nores
    private final String      DEFAULT_RUN_PATH =
        "DefaultRunnablePath";                                       // nores
    private final String      RUN_PARAMETERS = "RunParameters";      // nores
    private final String      VM_PARAMETERS = "JavaVmParameters";    // nores

    //
    private Project           nativeProject = null;
    private OracleNodeFactory factory = null;

    public String getCodeGenDefaultDestination() {
        PathHandle ph = null;
        File       file = null;

        file = new File(getRootPath());
        if (file.getParentFile() == null) {
            ph = PathHandle.createPathHandle(file);
        } else {
            ph = PathHandle.createPathHandle(file.getParentFile());
        }
        return ph.getPath();
    }

    public String getNativeProjectFilePath() {
        String nativePath = nativeProject.getUrl().getPath();

        return PathHandle.createPathString(nativePath);
    }

    public boolean isDefaultProject() {
        return false;
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    public OtterTextFileNode getFirstPolicy() {
        PolicyNodeReader reader = new PolicyNodeReader();

        return reader.getFirstNode(nativeProject);
    }

    public String getWorkingPath() {
        String     path = new String();
        PathHandle handle = null;

        path = nativeProject.getWorkingDirectory();
        handle = PathHandle.createPathHandle(path);
        if (handle.isDirectory()) {
            path = handle.getPath();
        } else {
            setWorkingPath(getRootPath());
        }
        return path;
    }

    public void setWorkingPath(String inPath) {
        String path = FileUtil.toWindowsPath(inPath);

        nativeProject.setWorkingDirectory(path);
    }

    public OtterFileNode[] getAllInput() {
        String[]        ext = new String[0];
        OtterFileNode[] input = new OtterFileNode[0];
        OtterFileNode[] nodes = new OtterFileNode[0];
        ArrayList       list = new ArrayList();
        PathHandle      nodePath = null;
        PathHandle      inputPath = null;

        inputPath = PathHandle.createPathHandle(getDeployInputPath());
        if (isDeployInputFilter()) {
            ext = new String[1];
            ext[0] = Constants.TYPE_IN;
        }
        nodes = findFileNodesByType(ext);
        for (int i = 0; i < nodes.length; i++) {
            nodePath = PathHandle.createPathHandle(nodes[i].getFilePath());
            if (inputPath.parentOf(nodePath)) {
                list.add(nodes[i]);
            }
        }
        input = (OtterFileNode[]) list.toArray(input);
        list.clear();
        return input;
    }

    // override OtterProject
    public boolean isDeployStartupJava() {
        return true;
    }

    // override OtterProject
    public boolean isDeployStartupJavaReadOnly() {
       return true;
    }

    /**
     * Method declaration
     *
     *
     * @param extensions
     *
     * @return
     */
    public OtterFileNode[] findFileNodesByType(String[] extensions) {
        NodeReader reader = new NodeReader(extensions);

        return reader.getNodes(nativeProject);
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    public OtterTextFileNode[] getAllDeploymentDescs() {
        DeploymentNodeReader reader = new DeploymentNodeReader();

        return reader.getNodes(nativeProject);
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    public OtterDocumentNode[] getAllDocuments() {
        DocumentNodeReader reader = new DocumentNodeReader();

        return reader.getNodes(this);
    }

    /**
     * Constructor declaration
     *
     *
     * @param p
     */
    public OracleProject() {
        factory = new OracleNodeFactory();
    }

    public OracleProject(Project p) {
        this();
        setNativeProject(p);
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    public OtterNodeFactory getNodeFactory() {
        return factory;
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    public Object getNativeProject() {
        return nativeProject;
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    public OtterProject getProject() {
        return this;
    }

    public OtterNode getParent() {
        return null;
    }

    /**
     * Method declaration
     *
     *
     * @param p
     */
    public void setNativeProject(Object p) {
        nativeProject = (Project) p;
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    public Object getNativeNode() {
        return getNativeProject();
    }

    /**
     * Method declaration
     *
     *
     * @param n
     */
    public void setNativeNode(Object n) {
        setNativeProject(n);
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    public boolean isOpenBuild() {
        return false;
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    public String getClassOutputPath() {
        String out = null;

        if (nativeProject != null) {
            out = nativeProject.getOutputDirectory();
        }
        return PathHandle.createPathString(out);
    }

    public void setClassOutputPath(String p) {
        nativeProject.BeginUpdate();
        nativeProject.setOutputDirectory(FileUtil.toWindowsPath(p));
        nativeProject.save(true);
        nativeProject.EndUpdate();
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    public String getSourcePath() {
        String out = null;

        if (nativeProject != null) {
            out = nativeProject.getProjectSourcePath();
        }
        return PathHandle.createPathString(out);
    }

    /**
     * Method declaration
     *
     *
     * @param pathAsArray
     */
    public void setSourcePathArray(String[] pathAsArray) {
        StringBuffer path = new StringBuffer();

        for (int i = 0; i < pathAsArray.length; i++) {
            pathAsArray[i] = FileUtil.toWindowsPath(pathAsArray[i]);
            path.append(pathAsArray[i]);
            path.append(';');
        }
        nativeProject.BeginUpdate();
        nativeProject.setProjectSourcePath(path.toString());
        nativeProject.save(true);
        nativeProject.EndUpdate();
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    public String[] getSourcePathArray() {
        String[]  pathAsArray = null;
        String    pathAsString = getSourcePath();
        ArrayList pathAsVector = new ArrayList();
        int       at = pathAsString.indexOf(';');
        int       size = 0;

        while (at > -1) {
            pathAsVector.add(pathAsString.substring(0, at).trim());
            pathAsString = pathAsString.substring(at + 1).trim();
            at = pathAsString.indexOf(';');
        }
        pathAsVector.add(pathAsString.trim());
        size = pathAsVector.size();
        if (size > 0) {
            pathAsArray = new String[size];
            for (int i = 0; i < size; i++) {
                pathAsArray[i] = (String) pathAsVector.get(i);
                pathAsArray[i] = PathHandle.createPathString(pathAsArray[i]);
            }
        }
        return pathAsArray;
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    public String getClassPath() {
        String out = null;

        if (nativeProject != null) {
            out = nativeProject.getClassPath();
            out = out.replace('\\', '/');
        }
        return out;
    }

    /**
     * Method declaration
     *
     *
     * @param property
     *
     * @return
     */
    public String getProperty(String property) {
        String out = null;

        if (nativeProject != null) {
            out = nativeProject.getProperty(property);
        }
        return out;
    }

    /**
     * Method declaration
     *
     *
     * @param property
     * @param value
     */
    public void setProperty(String property, String value) {
        if (nativeProject != null) {
            nativeProject.BeginUpdate();
            if (property.trim().equalsIgnoreCase(PropertyKeys.NAME_LIBRARIES)) {
                StringTokenizer tokenizer = null;
                String[]        libs = new String[0];

                // string not to be resourced
                final String    TOKEN = (new String()) + ';';

                tokenizer = new StringTokenizer(value, TOKEN);
                libs = new String[tokenizer.countTokens()];
                for (int i = 0; i < libs.length; i++) {
                    libs[i] = tokenizer.nextToken();
                }
                nativeProject.setLibraries(libs);
            } else {
                nativeProject.setProperty(property, value);
            }
            nativeProject.save(true);
            nativeProject.EndUpdate();
        }
    }

    /**
     * Method declaration
     *
     *
     * @param property
     * @param i
     */
    public void setProperty(String property, int i) {
        String value = Integer.toString(i);

        setProperty(property, value);
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    public String getRootPath() {
        String path = null;
        File   dir = null;

        path = PathHandle.createPathString(getNativeProjectFilePath());
        path = path.substring(0, path.lastIndexOf('.'));
        dir = new File(path);
        dir.mkdirs();
        return path;
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    public char getFileSeparatorChar() {
        return File.separatorChar;
    }

    /**
     * Method declaration
     *
     */
    public void configureRunClass() {
        StringBuffer  buf = new StringBuffer();
        RunParameters param = new RunParameters(this);

        // App parameters
        nativeProject.setProperty(RUN_PARAMETERS, param.getAppParameters());

        // Run class
        buf.append(getSourcePathArray()[0]);
        buf.append(File.separator);
        buf.append(Constants.FILE_START_SERVER);
        nativeProject.setProperty(DEFAULT_RUN_PATH, buf.toString());
        if (!ToolBoxInfo.isEnhydra3()) {
            nativeProject.setProperty(VM_PARAMETERS, param.getVMParameters());
        }
    }

    public void save() {
        nativeProject.save(true);
    }

    /**
     * Class declaration
     *
     *
     * @author Paul Mahar
     */
    class DeploymentNodeReader {
        private ArrayList list = new ArrayList();

        /**
         * Constructor declaration
         *
         */
        protected DeploymentNodeReader() {}

        /**
         * Method declaration
         *
         *
         * @param nativeProject
         *
         * @return
         */
        protected OtterTextFileNode[] getNodes(Project nativeProject) {
            OtterTextFileNode[] nodes;

            recursiveRead(nativeProject);
            nodes = new OtterTextFileNode[list.size()];
            for (int i = 0; i < nodes.length; i++) {
                nodes[i] = (OtterTextFileNode) list.get(i);
            }
            return nodes;
        }

        /**
         * Method declaration
         *
         *
         * @param container
         */
        private void recursiveRead(Container container) {
            String     filename = null;
            Node[]     nativeNodes = container.getChildren();
            File       current = null;
            PathHandle handle = null;

            for (int i = 0; i < nativeNodes.length; i++) {
                if (nativeNodes[i] instanceof Container) {
                    if (!nativeNodes[i].toString().equalsIgnoreCase(ROOT_CONTAINER)) {
                        recursiveRead((Container) nativeNodes[i]);
                    }
                } else {
                    filename = nativeNodes[i].getFilename();
                    current = new File(filename);
                    handle = PathHandle.createPathHandle(filename);
                    if (handle.isFile()
                            && (handle.endsWith(Constants.FILE_WEB_XML)
                                || handle.endsWith(Constants.FILE_EJB_JAR_XML))) {
                        OtterTextFileNode imageNode = null;

                        imageNode = new OracleTextFileNode(nativeNodes[i]);
                        list.add(imageNode);
                    }
                }
            }
        }

    }

    /**
     * Class declaration
     *
     *
     * @author Paul Mahar
     */
    class DocumentNodeReader {
        private ArrayList    list = new ArrayList();
        private OtterProject otterProject = null;

        /**
         * Method declaration
         *
         *
         * @param nativeProject
         *
         * @return
         */
        protected OtterDocumentNode[] getNodes(OtterProject p) {
            OtterDocumentNode[] nodes = new OtterDocumentNode[0];

            otterProject = p;
            list.clear();
            recursiveRead((Project) otterProject.getNativeProject());
            nodes = new OtterDocumentNode[list.size()];
            nodes = (OtterDocumentNode[]) list.toArray(nodes);
            list.clear();
            return nodes;
        }

        /**
         * Method declaration
         *
         *
         * @param container
         */
        private void recursiveRead(Container container) {
            PathHandle path = null;
            Node[]     nativeNodes = container.getChildren();

            for (int i = 0; i < nativeNodes.length; i++) {
                if (nativeNodes[i] instanceof Container) {
                    if (!nativeNodes[i].toString().equalsIgnoreCase(ROOT_CONTAINER)) {
                        recursiveRead((Container) nativeNodes[i]);
                    }
                } else {
                    path =
                        PathHandle.createPathHandle(nativeNodes[i].getFilename());
                    if (path.getFile().isFile()
                            && path.hasExtension(otterProject.getDocTypes())) {
                        OtterDocumentNode newNode = null;

                        newNode = new OracleDocumentNode(nativeNodes[i]);
                        list.add(newNode);
                    }
                }
            }
        }

    }

    /**
     * Class declaration
     *
     *
     * @author Paul Mahar
     */
    class NodeReader {
        private ArrayList list = new ArrayList();
        private String[]  extensions = new String[0];

        /**
         * Constructor declaration
         *
         *
         * @param exts
         */
        protected NodeReader(String[] exts) {
            extensions = exts;
        }

        /**
         * Method declaration
         *
         *
         * @param nativeProject
         *
         * @return
         */
        protected OtterFileNode[] getNodes(Project nativeProject) {
            OtterFileNode[] nodes;

            recursiveRead(nativeProject);
            nodes = new OtterFileNode[list.size()];
            nodes = (OtterFileNode[]) list.toArray(nodes);
            list.clear();
            return nodes;
        }

        /**
         * Method declaration
         *
         *
         * @param container
         */
        private void recursiveRead(Container container) {
            Node[]     nativeNodes = container.getChildren();
            PathHandle current = null;

            for (int i = 0; i < nativeNodes.length; i++) {
                if (nativeNodes[i] instanceof Container) {
                    if (!nativeNodes[i].toString().equalsIgnoreCase(ROOT_CONTAINER)) {
                        recursiveRead((Container) nativeNodes[i]);
                    }
                } else {
                    current =
                        PathHandle.createPathHandle(nativeNodes[i].getFilename());
                    if (current.getFile().exists()) {
                        if (current.hasExtension(extensions)
                                || extensions.length == 0) {
                            OtterNode newNode = null;

                            newNode = new OracleNode(nativeNodes[i]);
                            list.add(newNode);
                        }
                    }
                }
            }
        }

    }
    class PolicyNodeReader {
        private OtterTextFileNode node = null;

        /**
         * Method declaration
         *
         *
         * @param nativeProject
         *
         * @return
         */
        protected OtterTextFileNode getFirstNode(Project nativeProject) {
            recursiveRead(nativeProject);
            return node;
        }

        /**
         * Method declaration
         *
         *
         * @param container
         */
        private void recursiveRead(Container container) {
            String     filename = null;
            Node[]     nativeNodes = container.getChildren();
            PathHandle handle = null;

            for (int i = 0; i < nativeNodes.length; i++) {
                if (nativeNodes[i] instanceof Container) {
                    if (!nativeNodes[i].toString().equalsIgnoreCase(ROOT_CONTAINER)) {
                        recursiveRead((Container) nativeNodes[i]);
                    }
                } else {
                    filename = nativeNodes[i].getFilename();
                    handle = PathHandle.createPathHandle(filename);
                    if (handle.isFile()
                            && handle.endsWith(Constants.FILE_JAVA_POLICY)) {
                        node = new OracleTextFileNode(nativeNodes[i]);
                        break;
                    }
                }
            }
        }

    }
}
