/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.kelp.jdeveloper.properties;

// ToolBox imports
import org.enhydra.tool.common.FileUtil;
import org.enhydra.tool.common.PathHandle;

// Oracle imports
import borland.jbuilder.addin.IdeEnvironment;
import borland.jbuilder.addin.Context;
import borland.jbuilder.addin.MenuItem;
import borland.jbuilder.addin.MenuItemAdaptor;
import borland.jbuilder.addin.MenuListener;
import borland.jbuilder.addin.MenuLocations;
import borland.jbuilder.addin.MenuManager;
import borland.jbuilder.addin.MenuCategory;
import borland.jbuilder.addin.VersionedAddin;

// Kelp imports
import org.enhydra.kelp.common.Constants;
import org.enhydra.kelp.common.node.OtterDocumentNode;
import org.enhydra.kelp.common.node.OtterProject;
import org.enhydra.kelp.common.node.OtterXMLCNode;
import org.enhydra.kelp.jdeveloper.node.OracleDocumentNode;
import org.enhydra.kelp.jdeveloper.node.OracleProject;

// Standard imports
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.util.ResourceBundle;

//
public class ContextMenuListener implements VersionedAddin, MenuListener {

    // string not to be resourced
    static ResourceBundle     res =
        ResourceBundle.getBundle("org.enhydra.kelp.jdeveloper.Res"); // mpres
    private final String      VERSION = "2.0";    // nores

    //
    private MenuManager       menuManager = null;
    private OtterDocumentNode docNode = null;

    // VersionedAddin
    public String version() {
        return VERSION;
    }

    public void startup(IdeEnvironment ide) {
        menuManager = ide.getMenuManager();
        menuManager.addMenuListener(this);
    }

    // MenuListener
    public void menuActivated(MenuCategory menuCategory, Context context) {}

    public void contextMenuCreation(MenuCategory menuCategory,
                                    Context context) {
        MenuItem             menuItem = null;
        XMLCMenuItemListener listener = null;
        OtterProject         otterProject = null;
        PathHandle           path = null;
        boolean              createMenu = false;

        if (context.getNode() == null || context.getProject() == null) {

            // done
        } else if (context.getNode().getFilename() != null) {
            otterProject = new OracleProject(context.getProject());
            path =
                PathHandle.createPathHandle(context.getNode().getFilename());
            if (path.hasExtension(otterProject.getDocTypes())) {
                createMenu = true;
            }
        }
        if (createMenu) {
            docNode = new OracleDocumentNode(context.getNode());
            listener = new XMLCMenuItemListener();
            menuItem =
                menuManager.createMenuItem(res.getString("XMLC_Properties_"),
                                           Constants.XMLC, listener);
            menuManager.insertMenuItem(menuItem, menuCategory, 0);
        }
    }

    private void openPropertyDialog() {
        XMLCNodePropertyDialog dialog = null;
        Frame                  frame = null;

        frame = IdeEnvironment.getInstance().getIdeFrame();
        dialog = new XMLCNodePropertyDialog(frame);
        dialog.read(new OtterXMLCNode(docNode));
        dialog.show();
    }

    class XMLCMenuItemListener extends MenuItemAdaptor {
        public void actionPerformed(ActionEvent e) {
            openPropertyDialog();
        }

    }
}
