/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.kelp.jdeveloper.properties;

// JDeveloper
import oracle.jdeveloper.wizard.common.JDeveloperDialog;

// AddinCore
import org.enhydra.kelp.KelpInfo;
import org.enhydra.kelp.common.Constants;
import org.enhydra.kelp.common.node.OtterXMLCNode;
import org.enhydra.kelp.common.properties.XMLCNodePropertyPanel;

// JDK
import java.awt.BorderLayout;
import java.awt.Frame;
import java.lang.ref.WeakReference;
import java.util.ResourceBundle;

/**
 * Class declaration
 *
 *
 * @author Paul Mahar
 */
public class XMLCNodePropertyDialog extends JDeveloperDialog {

    // strings not to be resourced
    static ResourceBundle         res =
        ResourceBundle.getBundle("org.enhydra.kelp.jdeveloper.Res");    // nores
    private final String          HELP_LINK = KelpInfo.getAddinHelpURL()
                                              + "#xmlcnode";    // nores

    //
    private static final String   TITLE = res.getString("XMLC_Properties");
    private WeakReference         nodeRef = null;
    private XMLCNodePropertyPanel nodePanel = null;
    private LocalActionTab        actionTab = null;
    private BorderLayout          layoutMain = null;

    /**
     * Constructor declaration
     *
     */
    public XMLCNodePropertyDialog() {
        this(new Frame());
    }

    /**
     * Constructor declaration
     *
     *
     * @param topic
     */
    public XMLCNodePropertyDialog(Frame parent) {
        super(parent, XMLCNodePropertyDialog.TITLE, false);
        construct();
    }

    public void show() {
        addNotifyCreateParams();
        addNotify();
        pack();
        centerOnScreen();
        super.show();
    }

    /**
     * Method declaration
     *
     */
    public void readProperties() {
        nodePanel.read(getNode());
    }

    /**
     * Method declaration
     *
     */
    public void writeProperties() {
        nodePanel.save();
    }

    protected void read(OtterXMLCNode inNode) {
        nodePanel.read(inNode);
        nodeRef = new WeakReference(inNode);
    }

    private OtterXMLCNode getNode() {
      OtterXMLCNode node = null;
      if (nodeRef != null) {
        node = (OtterXMLCNode) nodeRef.get();
      }
      return node;
    }

    private void closeDialog() {
        setVisible(false);
        hide();
        removeAll();
        dispose();
    }

    private void construct() {
        try {
            jbInit();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * Method declaration
     *
     *
     * @exception Exception
     */
    private void jbInit() throws Exception {
        nodePanel = new XMLCNodePropertyPanel();
        actionTab = new LocalActionTab();
        layoutMain = new BorderLayout();
        actionTab.addTab(Constants.XMLC, nodePanel);
        getContentPane().setLayout(layoutMain);
        getContentPane().add(actionTab, BorderLayout.CENTER);
    }

    class LocalActionTab extends ActionTabPane {
        protected void doCancel() {
            closeDialog();
        }

        protected void doOK() {
            nodePanel.save();
            closeDialog();
        }

    }
}
