/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 * Travis Shook
 *
 */
package org.enhydra.kelp.jdeveloper.tool;

// JDeveloper imports
import borland.jbuilder.addin.IdeEnvironment;
import borland.jbuilder.addin.Project;

// ToolBox
import org.enhydra.tool.common.SwingUtil;

// ToolBox imports
import org.enhydra.tool.common.event.HelpEvent;
import org.enhydra.tool.common.event.HelpListener;

// Kelp imports
import org.enhydra.kelp.common.Constants;
import org.enhydra.kelp.common.ResUtil;
import org.enhydra.kelp.common.node.OtterProject;
import org.enhydra.kelp.common.deployer.CoreDeployTool;
import org.enhydra.kelp.jdeveloper.node.OracleProject;

// Standard imports
import java.awt.Component;
import java.awt.Dialog;
import java.awt.Frame;

/**
 * This is the class that gets registered with the IDE through the
 * jbuilder.properties file.
 */
public class DeployTool extends AbstractBasicTool implements HelpListener {

    // string not to be resourced
    private final String HELP_LINK = "deploywizard";    // nores

    /**
     * Constructor declaration
     *
     */
    public DeployTool() {}

    /**
     * Wizard interface implementation, run the wizard in response to this call.
     * @param ide Access to all AddIn interfaces
     * @param context Current node, project, browser
     */
    public void invoke() {
        IdeEnvironment ide = null;
        Frame          frame = null;
        CoreDeployTool coreTool = null;
        OtterProject   otterProject = null;

        ide = IdeEnvironment.getInstance();
        frame = ide.getIdeFrame();
        nativeProject = ide.getBrowser().getWorkspace().getActiveProject();
        String ver = System.getProperty(Constants.JAVA_VERSION);

        if (ver.startsWith(Constants.JDK11)) {
            ide.messageBox(ResUtil.format(res.getString("Kelp_requires_Java_2"), ver),
                           IdeEnvironment.MSG_Error, IdeEnvironment.BTN_OK);
            nativeProject = null;
        }
        try {
            verifyAllClassPaths();
        } catch (Exception e) {
            ide.messageBox(e.getMessage(), IdeEnvironment.MSG_Error,
                           IdeEnvironment.BTN_OK);
            nativeProject = null;
        }
        if (nativeProject != null) {
            nativeProject.save(false);    // save all file changes in project
            otterProject = new OracleProject(nativeProject);
            coreTool = new CoreDeployTool();
            coreTool.setProject(otterProject);
            coreTool.addHelpListener(this);
            coreTool.showDialog(frame);
            frame.requestFocus();
        }
    }

    public void onHelp(HelpEvent event) {
        browseKelpHelp(HELP_LINK);
    }

    /**
     * Wizard interface method, returns the name of this wizard.
     * @return Name of the wizard
     */
    public String getTitle() {
        return CoreDeployTool.getDefaultTitle();
    }

}
