/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.kelp.jdeveloper.wizard;

// ToolBox imports
import org.enhydra.tool.ToolBoxInfo;
import org.enhydra.tool.common.IconSet;
import org.enhydra.tool.common.ToolException;
import org.enhydra.tool.common.PathHandle;
import org.enhydra.tool.common.SwingUtil;
import org.enhydra.tool.common.ExtensionFilter;

// JDeveloper
import borland.jbuilder.addin.Context;
import borland.jbuilder.addin.IdeEnvironment;
import borland.jbuilder.addin.Project;
import borland.jbuilder.addin.VersionedAddin;
import borland.jbuilder.addin.Wizard;
import borland.jbuilder.addin.WizardManager;

// AddinCore imports
import org.enhydra.kelp.KelpInfo;
import org.enhydra.kelp.common.Constants;
import org.enhydra.kelp.common.ResUtil;

// AddinJDeveloper
import org.enhydra.kelp.jdeveloper.HelpBrowser;

// JDK
import java.awt.Container;
import java.awt.Image;
import java.io.File;
import java.io.Serializable;
import java.util.ResourceBundle;
import java.util.Properties;

//
abstract public class AbstractBasicWizard implements Serializable,
        VersionedAddin, Wizard {
    static ResourceBundle res =
        ResourceBundle.getBundle("org.enhydra.kelp.jdeveloper.Res");    // nores

    // string not to be resourced
    private final String  VERSION = "2.0";    // nores

    //
    protected Project     nativeProject = null;

    public AbstractBasicWizard() {}

    abstract public void invoke(IdeEnvironment ide, Context context);

    /**
     * Wizard interface method, returns the text for the menu label
     * @return Menu label text
     */
    public String getMenuLabel() {
        return ResUtil.format(res.getString("_0_dots"), getName());
    }

    /**
     * Wizard interface method, returns information about the type of Wizard this is.
     * @return An integer representing combined flags defined by the
     * Wizard interface.
     */
    public int getInfo() {
        return Wizard.PRODUCES_FILE + Wizard.REQUIRES_PROJECT;
    }

    /**
     * Wizard interface method
     * @return little Otter
     */

    /**
     * Wizard interface method
     * @return Currently returns big Otter
     */
    public Image getBigImage() {
        Image image = null;

        if (KelpInfo.isToolBoxVersionInSynch()) {
            image = IconSet.getLargeIcon().getImage();
        }
        return image;
    }

    public Image getSmallImage() {
        Image image = null;

        if (KelpInfo.isToolBoxVersionInSynch()) {
            image = IconSet.getSmallIcon().getImage();
        }
        return image;
    }

    /**
     * Entry point exposed by the Wizard interface for early initialization.
     * Registers this Wizard object with the IDE's WizardManager object
     * @$param ide Access to all AddIn interfaces
     */
    public void startup(IdeEnvironment ide) {
        WizardManager e = ide.getWizardManager();

        e.registerWizard(this);
    }

    /**
     * For the VersionAddin Interface.
     */
    public String version() {
        return VERSION;
    }

    public void browseKelpHelp(String anchor) {
        HelpBrowser browser = new HelpBrowser();
        browser.open(anchor);
    }

}
