/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Travis Shook
 */
package org.enhydra.kelp.jdeveloper.wizard;

// ToolBox imports
import org.enhydra.tool.common.IconSet;
import org.enhydra.tool.common.FileUtil;
import org.enhydra.tool.common.wizard.TBWizardPage;
import org.enhydra.tool.common.wizard.TBWizardPanel;
import org.enhydra.tool.common.ProgressMeter;
import org.enhydra.tool.codegen.ValidationException;
import org.enhydra.tool.codegen.wizard.CodeGenPanel;

// JDeveloper imports
import borland.jbuilder.addin.Addin;
import borland.jbuilder.addin.Context;
import borland.jbuilder.addin.Container;
import borland.jbuilder.addin.IdeEnvironment;
import borland.jbuilder.addin.Node;
import borland.jbuilder.addin.Project;
import borland.jbuilder.addin.VersionedAddin;
import borland.jbuilder.addin.Wizard;
import oracle.jdeveloper.wizard.common.ButtonPanel;
import oracle.jdeveloper.wizard.common.StandardWizardDialog;
import oracle.jdeveloper.wizard.common.OracleWizardPanel;

// Kelp imports
import org.enhydra.kelp.KelpInfo;
import org.enhydra.kelp.common.Constants;
import org.enhydra.kelp.common.ResUtil;
import org.enhydra.kelp.common.node.OtterProject;
import org.enhydra.kelp.common.node.PropertyKeys;
import org.enhydra.kelp.common.codegen.ProjectGenUtil;
import org.enhydra.kelp.common.codegen.CodeGenException;
import org.enhydra.kelp.jdeveloper.node.OracleProject;

// Standard imports
import java.awt.BorderLayout;
import java.awt.Image;
import java.awt.Frame;
import java.io.File;
import java.util.Properties;
import java.util.ResourceBundle;
import javax.swing.JPanel;

/**
 * Class declaration
 *
 *
 * @author Paul Mahar
 */
abstract public class AbstractCodeGenWizard extends AbstractBasicWizard {
    static ResourceBundle res =
        ResourceBundle.getBundle("org.enhydra.kelp.jdeveloper.Res");    // nores

    // string not to be resourced
    private final String  LIBS = "Enhydra;SunTools;Toolbox";    // nores
    private final String  HELP_LINK = "appwizard";      // nores

    //
    private String        imageFileName = new String();
    private Image         bigImage = null;
    private ProjectGenUtil    util = null;
    abstract public String getDisplayName();
    abstract public String[] getSteps();

    /**
     * Method declaration
     *
     *
     * @param ide
     * @param context
     */
    public void invokeWizard(IdeEnvironment ide, Context context,
                             ProjectGenUtil pgUtil) throws CodeGenException {
        CodeGenWizardDialog dialog = null;
        util = pgUtil;
        ide.getWorkspace().BeginUpdate();
        ide.getWorkspace().save(true);
        ide.getWorkspace().reload();
        ide.getWorkspace().EndUpdate();
        if (context != null) {
            nativeProject = context.getProject();
            if (nativeProject == null
                    || nativeProject.getType().equals(Container.ROOT_TYPE)) {
                nativeProject = null;
                String mess = new String();

                mess = ResUtil.format(res.getString("Project_not_found"),
                                      getDisplayName());
                ide.messageBox(mess, IdeEnvironment.MSG_Warning,
                               IdeEnvironment.BTN_OK);
            }
            if (nativeProject != null) {
                OtterProject otterProject = null;
                StringBuffer path = new StringBuffer();

                otterProject = new OracleProject(nativeProject);
                util.init(otterProject);
                dialog = new CodeGenWizardDialog(ide, getName(),
                                                 nativeProject);
                dialog.show();
            }
        }
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    public String toString() {
        return getDisplayName();
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    public String getName() {
        return getDisplayName();
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    public String getMenuLabel() {
        return null;
    }

    //
    //
    protected void verifyJava2() throws Exception {
        String ver = null;

        ver = System.getProperty(Constants.JAVA_VERSION);
        if (ver.startsWith(Constants.JDK11)) {
            String mess = new String();

            mess = ResUtil.format(res.getString("Kelp_requires_Java_2"), ver);
            throw new Exception(mess);
        }
    }

    //
    //
    private void setupNativeProject(OtterProject otterProject) {

        // Note steps should be added to replacement list!!!!
        IdeEnvironment ide = null;

        ide = IdeEnvironment.getInstance();
        nativeProject.BeginUpdate();
        nativeProject.setHTMLRootDirectory(FileUtil.toWindowsPath(otterProject.getWorkingPath()));
        nativeProject.setHTMLSrcDirectory(Constants.DIR_SRC);
        otterProject.setProperty(PropertyKeys.NAME_LIBRARIES, LIBS);
        nativeProject.save(true);
        nativeProject.EndUpdate();
        ide.getWorkspace().BeginUpdate();
        ide.getWorkspace().save(true);
        ide.getWorkspace().reload();
        ide.getWorkspace().EndUpdate();
    }

    private void openReadme() {
        Node[] list = new Node[0];
        Node   node = null;

        list = nativeProject.getChildren();
        for (int i = 0; i < list.length; i++) {
            if (list[i].getFilename().toLowerCase().endsWith(Constants.FILE_README)) {
                node = list[i];
                break;
            }
        }
        list = new Node[0];
        try {
            IdeEnvironment.getInstance().getBrowser().selectUrl(node.getUrl());
            node.showViewerWindow();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private class CodeGenWizardDialog extends StandardWizardDialog {
        private OtterProject otterProject = null;

        public CodeGenWizardDialog(IdeEnvironment ide, String title,
                                   Project nativeProject) {
            super(ide, title, nativeProject);
            otterProject = new OracleProject(nativeProject);
            pmInit();
        }

        public void show() {
            first();
            pack();
            centerOnScreen();
            super.show();
        }

        public void cancel() {
            closeDialog();
            super.cancel();
        }

        public void help() {
            browseKelpHelp(HELP_LINK);
        }

        /**
         * Method declaration
         *
         *
         */
        public boolean finish() {
            boolean       valid = false;
            ProgressMeter progress = null;

            if (getPanel() instanceof LocalWizardPage) {
                LocalWizardPage localPage = (LocalWizardPage) getPanel();

                valid = localPage.validatePanel();
            }
            if (valid) {
                try {
                    String       javaPath = new String();
                    StringBuffer buf = new StringBuffer();

                    javaPath = System.getProperty("java.home");
                    util.getGenerator().setJavaPath(javaPath);
                    if (getSteps().length > 0) {
                       util.getGenerator().setAddinSteps(getSteps());
                    }
                    util.generate();
                    setupNativeProject(otterProject);
                    buf.append(ResUtil.format(res.getString("The_0_Wizard"),
                                              getDisplayName()));
                    buf.append('\n');
                    buf.append(Constants.TAB4);
                    buf.append(nativeProject.getFilename());
                    buf.append('\n');
                    buf.append(ResUtil.format(res.getString("Refer_to_the_readme"),
                                              getDisplayName()));
                    ide.messageBox(buf.toString(),
                                   IdeEnvironment.MSG_Information,
                                   IdeEnvironment.BTN_OK);
                    openReadme();
                    valid = true;
                } catch (CodeGenException e) {
                    e.printStackTrace(System.err);
                    ide.messageBox(e.getMessage(),
                                   IdeEnvironment.MSG_Warning,
                                   IdeEnvironment.BTN_OK);
                    valid = false;
                }
            }
            closeDialog();
            return valid;
        }

        //
        //
        private void pmInit() {
            CodeGenPanel[] panels = new CodeGenPanel[0];

            try {
                panels = util.getWizardPanels();
            } catch (CodeGenException e) {
                e.printStackTrace(System.err);
                panels = new CodeGenPanel[0];
            }
            if (KelpInfo.isToolBoxVersionInSynch()) {
                setImage(IconSet.getOtterWizard().getImage());
            }
            for (int i = 0; i < panels.length; i++) {
                LocalWizardPage page = null;

                page = new LocalWizardPage();
                page.setPanel(panels[i]);
                addPanel(page);
            }
        }

        private void closeDialog() {
            setVisible(false);
            removeAll();
            dispose();
        }

    }

    /**
     * Class declaration
     *
     *
     * @author Paul Mahar
     */
    private class LocalWizardPage extends OracleWizardPanel {
        private CodeGenPanel  cgPanel = null;
        private TBWizardPanel tbPanel = null;
        private TBWizardPage  tbPage = null;

        /**
         * Constructor declaration
         *
         */
        public LocalWizardPage() {
            super();
            pmInit();
        }

        /**
         * Method declaration
         *
         */
        public void enter() {
            setEnabled(true);
            setVisible(true);
            add(tbPage, BorderLayout.CENTER);
            enableFinish(true);
        }

        public boolean leave() {
            boolean valid = validatePanel();

            if (valid) {
                setEnabled(false);
                setVisible(false);
                remove(tbPage);
                enableFinish(false);
            }
            return valid;
        }

        /**
         * Method declaration
         *
         */
        public boolean validatePanel() {
            boolean valid = true;

            try {
                cgPanel.validateOptionSet();
            } catch (ValidationException e) {
                IdeEnvironment.getInstance().messageBox(this, e.getMessage(),
                                                        IdeEnvironment.MSG_Error,
                                                        IdeEnvironment.BTN_OK);
                valid = false;
            }
            return valid;
        }

        public String getTitle() {
            return tbPage.getPageTitle();
        }

        protected void setPanel(CodeGenPanel codePanel) {
            cgPanel = codePanel;
            tbPanel.add(cgPanel, BorderLayout.CENTER);
            tbPage.setInstructions(cgPanel.getInstructions());
            tbPage.setPageTitle(cgPanel.getPageTitle());
        }

        protected JPanel getPanel() {
            return cgPanel;
        }

        //
        //
        private void pmInit() {
            BorderLayout layoutMain = null;
            BorderLayout layoutTBPanel = null;

            layoutMain = new BorderLayout();
            layoutTBPanel = new BorderLayout();
            layoutMain.setHgap(5);
            layoutMain.setVgap(5);
            setLayout(layoutMain);
            setVisible(false);
            tbPage = new TBWizardPage();
            tbPanel = new TBWizardPanel();
            tbPanel.setLayout(layoutTBPanel);
            tbPage.add(tbPanel);
        }

        private void enableFinish(boolean enable) {
            if (getTopLevelAncestor() instanceof StandardWizardDialog) {
                StandardWizardDialog dialog = null;
                ButtonPanel          buttons = null;

                dialog = (StandardWizardDialog) getTopLevelAncestor();
                buttons = dialog.getButtonBar();
                buttons.enableButton(StandardWizardDialog.FINISH_BUTTON,
                                     enable);
            }
        }

    }
}
