/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 * Travis Shook
 *
 */
package org.enhydra.kelp.jdeveloper.wizard;

// AddinCore
import org.enhydra.kelp.KelpInfo;
import org.enhydra.kelp.common.codegen.Enhydra3AppUtil;
import org.enhydra.kelp.common.codegen.AppGenUtil;
import org.enhydra.kelp.common.codegen.CodeGenException;

// ToolBox
import org.enhydra.tool.ToolBoxInfo;

// JDeveloper
import borland.jbuilder.addin.Context;
import borland.jbuilder.addin.IdeEnvironment;

// JDK
import java.util.ArrayList;
import java.util.ResourceBundle;

//
public class Enhydra3AppWizard extends AbstractCodeGenWizard {

    //
    public String getDisplayName() {
        return Enhydra3AppUtil.getDisplayName();
    }

    /**
     * Method declaration
     *
     *
     * @param ide
     * @param context
     */
    public void invoke(IdeEnvironment ide, Context context) {
        AppGenUtil agUtil = null;

        try {
            KelpInfo.verifyIDEClassPath();
            verifyJava2();
            agUtil = new Enhydra3AppUtil();
            invokeWizard(ide, context, agUtil);
        } catch (CodeGenException e) {
            e.printStackTrace(System.err);
            ide.messageBox(e.getMessage(), IdeEnvironment.MSG_Error,
                           IdeEnvironment.BTN_OK);
        } catch (Exception e) {
            ide.messageBox(e.getMessage(), IdeEnvironment.MSG_Error,
                           IdeEnvironment.BTN_OK);
        }
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    public String[] getSteps() {
        ArrayList steps = new ArrayList();
        String[] array = new String[0];

        if (! ToolBoxInfo.isEnhydra3()) {
          steps.add(res.getString("readmeEn0"));
        }

        steps.add(res.getString("readmeEn1"));
        steps.add(res.getString("readmeEn2"));
        steps.add(res.getString("readmeEn3"));
        steps.add(res.getString("readmeEn4"));
        steps.add(res.getString("readmeEn5"));
        steps.trimToSize();
        array = new String[steps.size()];
        array = (String[]) steps.toArray(array);
        steps.clear();
        return array;
    }

}
