/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.multiServer.launch.capture;

// Standard imports
import java.io.PrintStream;
import java.io.OutputStream;
import java.util.Vector;

//
public class Printer extends PrintStream {
    private static Vector listeners = new Vector();

    public Printer(int type) {
        super(new CaptureOutputStream(type));
    }

    // event methods
    public static synchronized void addPrintListener(PrintListener l) {
        listeners.addElement(l);
    }

    public static synchronized void removePrintListener(PrintListener l) {
        if (listeners.contains(l)) {
            listeners.removeElement(l);
        }
    }

    protected static synchronized void notifyPrintListeners(Object source,
            String s, int type) {
        int           count = 0;
        PrintListener listener = null;
        PrintEvent    event = new PrintEvent(source, type, s);

        count = listeners.size();
        for (int i = 0; i < count; i++) {
            listener = (PrintListener) listeners.elementAt(i);
            listener.onPrint(event);
        }
    }

}
class CaptureOutputStream extends OutputStream {
    private StringBuffer lineBuffer = new StringBuffer();
    private int          type = PrintEvent.OUTPUT;

    public CaptureOutputStream(int t) {
        super();
        type = t;
    }

    public synchronized void write(int i) {
        byte[] b = new byte[1];

        b[0] = (byte) i;
        write(b);
    }

    /**
     * Method declaration
     *
     *
     * @param str
     *
     * @see
     */
    public synchronized void write(byte[] b) {
        write(b, 0, b.length);
    }

    public synchronized void write(byte[] b, int off, int len) {
        String  str = new String(b);
        String  out = str.substring(off, (off + len));
        boolean writeLine = false;

        if (true) {
            if (out.length() > 1) {
                char end1 = out.charAt(out.length() - 2);
                char end2 = out.charAt(out.length() - 1);

                if ((int) end1 == 13 && (int) end2 == 10) {
                    if (out.length() < 3) {
                        out = "";
                    } else {
                        out = out.substring(0, out.length() - 2);
                    }
                    writeLine = true;
                }
                if (out.endsWith("\n")) {
                    writeLine = true;
                }
            }
            if (writeLine) {
                Printer.notifyPrintListeners(this,
                                             lineBuffer.toString() + out,
                                             type);
                lineBuffer.setLength(0);
            } else {
                lineBuffer.append(out);
            }
        } else {
            Printer.notifyPrintListeners(this, out, type);
        }
    }

}
