/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.tool.archive;

// Toolbox Imports
import org.enhydra.tool.common.FileUtil;
import org.enhydra.tool.common.PathHandle;
import org.enhydra.tool.common.ResUtil;

// Standard imports
import java.io.File;
import java.io.FileFilter;
import java.util.ResourceBundle;
import java.util.Arrays;
import java.util.ArrayList;

//
public class EarPlan extends JarPlan {

    //
    private String           appName = new String();
    private String           appDesc = "Application description";    // default
    private String[]         eModules = new String[0];
    private WebApplication[] webApps = new WebApplication[0];
    private boolean          alt = false;
    private boolean          copyClient = true;

    public EarPlan() {}

    public boolean isAlt() {
        return alt;
    }

    public void setAlt(boolean a) {
        alt = a;
    }

    public boolean isCopyClient() {
        return copyClient;
    }

    public void setCopyClient(boolean c) {
        copyClient = c;
    }

    public String getAppName() {
        return appName;
    }

    public void setAppName(String n) {
        appName = n;
    }

    public String getDescription() {
        return appDesc;
    }

    public void setDescription(String d) {
        appDesc = d.replace('\n', ' ');
    }

    public String[] getEnterpriseModules() {
        return eModules;
    }

    public void setEnterpriseModules(String[] in) {
        eModules = in;
    }

    public WebApplication[] getWebApplications() {
        return webApps;
    }

    public void setWebApplications(WebApplication[] apps) {
        webApps = apps;
    }

    public void addWebApplication(String war) {
        ArrayList list = null;

        list = new ArrayList(Arrays.asList(webApps));
        list.add(new WebApplication(war));
        list.trimToSize();
        webApps = new WebApplication[list.size()];
        webApps = (WebApplication[]) list.toArray(webApps);
        list.clear();
    }

    public void addEnterpriseModule(String mod) {
        ArrayList list = null;

        list = new ArrayList(Arrays.asList(eModules));
        list.add(mod);
        list.trimToSize();
        eModules = new String[list.size()];
        eModules = (String[]) list.toArray(eModules);
        list.clear();
    }

    public Module[] getAllModules() {
        ArrayList list = new ArrayList();
        String[]  ePaths = getEnterpriseModules();
        Module[]  mods = new Module[ePaths.length];

        for (int i = 0; i < mods.length; i++) {
            mods[i] = new Module(ePaths[i]);
        }
        list.addAll(Arrays.asList(mods));
        list.addAll(Arrays.asList(getWebApplications()));
        list.trimToSize();
        mods = new Module[list.size()];
        mods = (Module[]) list.toArray(mods);
        list.clear();
        return mods;
    }

    public boolean equals(Object comp) {
        EarPlan plan = null;
        boolean equal = false;

        if (comp instanceof EarPlan) {
            plan = (EarPlan) comp;
            equal = super.equals(comp);
        }
        if (equal) {
            if (plan.getAppName() == null && getAppName() == null) {}
            else if (plan.getAppName().equals(getAppName())) {}
            else {
                equal = false;
            }
        }
        if (equal) {
            if (plan.getDescription() == null && getDescription() == null) {}
            else if (plan.getDescription().equals(getDescription())) {}
            else {
                equal = false;
            }
        }
        if (equal) {
            if (!(plan.isAlt() == isAlt())) {
                equal = false;
            }
        }
        if (equal) {
            if (!(plan.isCopyClient() == isCopyClient())) {
                equal = false;
            }
        }
        if (equal) {
            if (!Arrays.equals(plan.getWebApplications(),
                               getWebApplications())) {
                equal = false;
            }
        }
        if (equal) {
            if (!Arrays.equals(plan.getEnterpriseModules(),
                               getEnterpriseModules())) {
                equal = false;
            }
        }
        return equal;
    }

}
