/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.tool.archive;

// Toolbox Imports
import org.enhydra.tool.common.FileUtil;
import org.enhydra.tool.common.PathHandle;
import org.enhydra.tool.common.ResUtil;

// Standard imports
import java.io.File;
import java.io.FileFilter;
import java.util.Arrays;
import java.util.ResourceBundle;

//
public class WarPlan extends JarPlan {

    //
    private String   contentRoot = null;
    private String[] contentFiles = null;

    public WarPlan() {
        super();
        Descriptor[] initDD = new Descriptor[2];

        initDD[0] = new Descriptor(true, Descriptor.WEB);
        initDD[1] = new Descriptor(false, Descriptor.ENHYDRA_WEB);
        setDescriptors(initDD);

    }

    public String getContentRoot() {
        return contentRoot;
    }

    public String[] getContentFiles() {
        return contentFiles;
    }

    public void setContentFiles(String[] contPaths) {
        if (isEqual(getContentFileArray(), contPaths)) {
            contentFiles = null;
        } else {
            contentFiles = contPaths;
        }
    }

    public void setContentRoot(String in) throws ArchiveException {
        if (in == null) {
            throw new ArchiveException(res.getString("Content_directory_is"));
        } else {
            PathHandle path = PathHandle.createPathHandle(in);

            if (isValidate()) {
                if (!path.isDirectory()) {
                    throw new ArchiveException(ResUtil.format(res.getString("Content_directory_not"),
                                                              path));
                }
            }
            contentRoot = path.getPath();
        }
    }

    public File[] getContentFileArray() {
        File[] files = new File[0];

        fileList.clear();
        setFillFilter(new ContentFilter());
        if (getContentRoot() == null) {

            // done
        } else if (contentFiles == null) {
            fillFileList(new File(getContentRoot()));
        } else {
            for (int i = 0; i < contentFiles.length; i++) {
                File cursor = new File(contentFiles[i]);

                if (getFillFilter().accept(cursor)) {
                    fileList.add(cursor);
                }
            }
        }
        fileList.trimToSize();
        files = new File[fileList.size()];
        files = (File[]) fileList.toArray(files);
        fileList.clear();
        return files;
    }

    public boolean equals(Object comp) {
        WarPlan plan = null;
        boolean equal = false;

        if (comp instanceof WarPlan) {
            plan = (WarPlan) comp;
            equal = super.equals(comp);
        }
        if (equal) {
            if (plan.getContentRoot() == null && getContentRoot() == null) {}
            else if (plan.getContentRoot().equals(getContentRoot())) {}
            else {
                equal = false;
            }
        }
        if (equal) {
            if (!Arrays.equals(plan.getContentFiles(), getContentFiles())) {
                equal = false;
            }
        }
        return equal;
    }

    //
    private class ContentFilter implements FileFilter {
        private String[]   ddPaths = new String[0];
        private PathHandle rootPath = null;

        public ContentFilter() {
            super();
            ddPaths = new String[getDescriptors().length];
            rootPath = PathHandle.createPathHandle(getContentRoot());
        }

        public boolean accept(File file) {
            boolean    include = false;
            PathHandle filePath = null;

            for (int i = 0; i < ddPaths.length; i++) {
                ddPaths[i] = getDescriptors()[i].getPath();
            }
            filePath = PathHandle.createPathHandle(file);
            if (file.isDirectory()) {
                include = true;
            } else if (rootPath.parentOf(filePath)) {
                include = true;
                for (int i = 0; i < ddPaths.length; i++) {
                    if (filePath.equals(ddPaths[i])) {
                        include = false;
                        break;
                    }
                }
            }
            return include;
        }

    }
}
