/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.tool.archive.wizard;

// ToolBox
import org.enhydra.tool.archive.ArchiveException;
import org.enhydra.tool.archive.JarPlan;
import org.enhydra.tool.common.PathHandle;

// JDK
import javax.swing.*;
import javax.swing.border.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.*;
import java.beans.*;
import java.io.File;
import java.util.ResourceBundle;

/**
 * This class defines the type of panel you can add to an
 * OpenTool's BasicWizardPage or an CodeGenPage.
 */
public class ArchiveSelectionPanel extends ArchivePanel {

    //
    transient private GridBagLayout     layoutMain;
    transient private BorderLayout      layoutDescription;
    transient private JComboBox         comboArchive;
    transient private JLabel            labelArchive;
    transient private JPanel            panelDescription;
    transient private JTextArea         textDescription;
    transient private LocalItemListener itemListener = null;
    transient private ArchiveType       selection = null;
    transient private JPanel            panelFiller;

    /**
     * Create an ArchivePanel for use in an OpenTool
     * BasicWizardPage or an ArchivePage.
     */
    public ArchiveSelectionPanel() {
        try {
            jbInit();
            pmInit();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    public void validatePlan() throws ArchiveException {

        // no-op
    }

    public void readPlan(JarPlan plan) throws ArchiveException {

        // no-op
    }

    public void writePlan(JarPlan plan) throws ArchiveException {

        // no-op
    }

    /**
     * Get the title to use on the current page.
     *
     * @return
     * A string to place at the top of a CodeGen wizard panel.
     */
    public String getPageTitle() {
        return "Archive Type";
    }

    /**
     * Get the instructions for selecting a generator.
     *
     * @return
     * A string to place below the page title.
     */
    public String getInstructions() {
        StringBuffer buf = new StringBuffer();

        buf.append("Choose an archive type and view a description of ");
        buf.append("your selection.");
        return buf.toString();
    }

    /**
     * Get the type selected by the user.
     *
     * @return
     * The archive type
     */
    public ArchiveType getSelection() {
        return selection;
    }

    // /
    // / PRIVATE METHODS
    // /

    /**
     * Method declaration
     *
     *
     * @exception Exception
     */
    private void jbInit() throws Exception {
        panelDescription =
            (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                       JPanel.class.getName());
        labelArchive = (JLabel) Beans.instantiate(getClass().getClassLoader(),
                                                  JLabel.class.getName());
        textDescription =
            (JTextArea) Beans.instantiate(getClass().getClassLoader(),
                                          JTextArea.class.getName());
        comboArchive =
            (JComboBox) Beans.instantiate(getClass().getClassLoader(),
                                          JComboBox.class.getName());
        layoutMain =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        layoutDescription =
            (BorderLayout) Beans.instantiate(getClass().getClassLoader(),
                                             BorderLayout.class.getName());
        panelFiller = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                                 JPanel.class.getName());
        labelArchive.setMaximumSize(new Dimension(80, 20));
        labelArchive.setMinimumSize(new Dimension(80, 20));
        labelArchive.setPreferredSize(new Dimension(80, 20));
        labelArchive.setLabelFor(comboArchive);
        labelArchive.setText("Archive type:");
        textDescription.setDisabledTextColor(SystemColor.controlText);
        textDescription.setBackground(SystemColor.text);
        textDescription.setBounds(2, 2, 2, 2);
        textDescription.setMargin(new Insets(5, 5, 5, 5));
        textDescription.setEnabled(false);
        textDescription.setEditable(false);
        textDescription.setWrapStyleWord(true);
        textDescription.setFont(labelArchive.getFont());
        textDescription.setLineWrap(true);
        panelDescription.setLayout(layoutDescription);
        panelDescription.setBorder(BorderFactory.createEtchedBorder());
        panelDescription.setMaximumSize(new Dimension(220, 70));
        panelDescription.setMinimumSize(new Dimension(220, 70));
        panelDescription.setPreferredSize(new Dimension(220, 70));
        comboArchive.setMaximumSize(new Dimension(100, 21));
        comboArchive.setMinimumSize(new Dimension(100, 21));
        comboArchive.setPreferredSize(new Dimension(100, 21));
        panelDescription.add(textDescription, BorderLayout.CENTER);
        this.setLayout(layoutMain);
        this.add(panelFiller,
                 new GridBagConstraints(3, 0, 1, 1, 0.1, 0.1,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.BOTH,
                                        new Insets(0, 0, 0, 0), 0, 0));
        this.add(comboArchive,
                 new GridBagConstraints(1, 0, 2, 1, 0.5, 0.5,
                                        GridBagConstraints.WEST,
                                        GridBagConstraints.NONE,
                                        new Insets(5, 5, 5, 5), 100, 0));
        this.add(labelArchive,
                 new GridBagConstraints(0, 0, 1, 1, 0.5, 0.5,
                                        GridBagConstraints.WEST,
                                        GridBagConstraints.NONE,
                                        new Insets(5, 10, 5, 0), 60, 15));
        this.add(panelDescription,
                 new GridBagConstraints(0, 1, 4, 1, 0.5, 0.5,
                                        GridBagConstraints.NORTHWEST,
                                        GridBagConstraints.VERTICAL,
                                        new Insets(5, 5, 5, 5), 150, 80));
    }

    private void pmInit() {
        ArchiveType[] types = null;

        types = ArchiveType.getAllTypes();
        comboArchive.removeAllItems();
        for (int i = 0; i < types.length; i++) {
            comboArchive.addItem(types[i]);
        }
        if (types.length > 0) {
            textDescription.setText(types[0].getDescription());
            selection = types[0];
        }
        comboArchive.repaint();
        itemListener = new LocalItemListener();
        comboArchive.addItemListener(itemListener);
    }

    private class LocalItemListener implements ItemListener {
        public void itemStateChanged(ItemEvent event) {
            Object item = comboArchive.getSelectedItem();

            if (item instanceof ArchiveType) {
                selection = (ArchiveType) item;
                System.setProperty("user.dir", getWorkingRoot());
                textDescription.setText(selection.getDescription());
            } else {
                selection = null;
                textDescription.setText(new String());
            }
        }

    }
}
