/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.tool.archive.wizard;

//
// ToolBox imports
//
import org.enhydra.tool.ToolBoxInfo;
import org.enhydra.tool.archive.JarPlan;
import org.enhydra.tool.archive.ArchiveException;
import org.enhydra.tool.archive.ArchiveTool;
import org.enhydra.tool.common.PathHandle;
import org.enhydra.tool.common.SwingUtil;

//
// Standard imports
//
import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.ArrayList;
import java.util.ResourceBundle;
import java.util.jar.Manifest;
import java.util.jar.JarInputStream;

/**
 * The ArchiveDialog defines a dialog for presenting
 * archive wizards.
 */
public class E3Type extends ArchiveType {
    public final static String EXT = "jar";
    //
    protected final String getSelectionName() {
        return "Enhydra Application";
    }

    protected final String getExtension() {
        return EXT;
    }

    protected final String getType() {
        return "e3-jar";
    }


    protected final String getDescription() {
        StringBuffer buf = new StringBuffer();

        buf.append("Create a jar for an Enhydra super servlet. ");
        buf.append("Super servlet jars are compatable with Enhydra 3.");
        return buf.toString();
    }

    protected void initPanels() throws ArchiveException {
        E3FilePanel    filePanel = new E3FilePanel();
        ClassesPanel   classesPanel = new ClassesPanel();
        ArchivePanel[] newPanels = new ArchivePanel[2];

        newPanels[0] = filePanel;
        newPanels[1] = classesPanel;
        setWizardPanels(newPanels);
    }

    protected void initPlan() throws ArchiveException {
        PathHandle path = null;

        setPlan(new JarPlan());
        getPlan().setArchivePath(getDefaultArchivePath());
        path = PathHandle.createPathHandle(getPlan().getArchivePath());
        if (path.isFile()) {
            readArchive(getPlan().getArchivePath());
        }
    }

}
