/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */

package org.enhydra.tool.archive.wizard;

// ToolBox
import org.enhydra.tool.archive.ArchiveException;
import org.enhydra.tool.archive.Descriptor;
import org.enhydra.tool.archive.JarPlan;
import org.enhydra.tool.archive.WarPlan;

// JDK
import java.awt.*;
import java.beans.Beans;
import java.io.File;
import java.lang.ref.WeakReference;
import javax.swing.*;

//
public class WarFilePanel extends ArchivePanel {
    private BorderLayout layoutMain = null;
    private FilePanel    filePanel = null;

    public WarFilePanel() {
        try {
            jbInit();
            pmInit();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    public void readPlan(JarPlan plan) throws ArchiveException {
        filePanel.setArchivePath(plan.getArchivePath());
        if (plan instanceof WarPlan) {
            WarPlan wp = (WarPlan) plan;
            Descriptor[] rows = plan.getDescriptors();

            filePanel.getDDTableModel().clear();
            for (int i = 0 ; i < rows.length ; i++ ) {
              filePanel.getDDTableModel().addDescriptor(rows[i]);
            }
            filePanel.getDDTable().setRowSelectionInterval(0, 0);
        } else {
            throw new ArchiveException("Unable to read plan: expecting type WarPlan");
        }
    }

    public void writePlan(JarPlan plan) throws ArchiveException {
        validatePlan();
        plan.setArchivePath(filePanel.getArchivePath());
        plan.setDescriptors(filePanel.getDDTableModel().getDescriptors());
    }

    public void validatePlan() throws ArchiveException {
        filePanel.validatePlan();
    }

    public String getPageTitle() {
        return "Web Archive";
    }

    public String getInstructions() {
        StringBuffer buf = new StringBuffer();

        buf.append("Enter details for the web archive file.");
        return buf.toString();
    }

    // override ArchivePanel
    protected void setWizard(ArchiveWizard wizard) {
      super.setWizard(wizard);
      filePanel.setWizard(wizard);
    }

    protected void preparePanel() {
       String[] defaultDD = new String[1];
       StringBuffer buf = new StringBuffer();
       buf.append("src");
       buf.append(File.separator);
       buf.append("WEB-INF");
       buf.append(File.separator);
       buf.append("web.xml");
       defaultDD[0] = buf.toString();
       filePanel.setDefaultDD(defaultDD);
       filePanel.preparePanel();
    }

    private void pmInit() {
      filePanel.setExtension("war");
    }

    private void jbInit() throws Exception {
        layoutMain =
            (BorderLayout) Beans.instantiate(getClass().getClassLoader(),
                                             BorderLayout.class.getName());
        filePanel = (FilePanel) Beans.instantiate(getClass().getClassLoader(),
                                                  FilePanel.class.getName());
        this.setLayout(layoutMain);
        this.add(filePanel, BorderLayout.CENTER);
    }

}
