/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.tool.archive.wizard;

// ToolBox
import org.enhydra.tool.archive.WebApplication;
import org.enhydra.tool.common.PathHandle;

// JDK
import java.awt.Component;
import java.util.ArrayList;
import javax.swing.JTable;
import javax.swing.table.AbstractTableModel;

//
public class WarSelector extends OneColumnFileSelector {
    private JTable table = null;

    public WarSelector() {
        super();
        warInit();
    }

    // override OneColumnFileSelector
    protected void removeSelection() {
        int index = table.getSelectedRow();

        if ((index > -1) && (index < getTableModel().getRowCount())) {
            getTableModel().removeRow(index);
        }
    }

    // override OneColumnFileSelector
    protected void addToModel(PathHandle path) {
        int       count = getTableModel().getRowCount();
        ArrayList list = new ArrayList();

        for (int i = 0; i < count; i++) {
            list.add(getTableModel().getValueAt(i, 1).toString());
        }
        if (!list.contains(path.getPath())) {
            WebApplication webApp = null;

            webApp = new WebApplication(path.getPath());
            getTableModel().addWebApplication(webApp);
        }
    }

    protected void setWebApplications(WebApplication[] webApps) {
        int count = getTableModel().getRowCount();

        for (int i = 0; i < count; i++) {
            getTableModel().removeRow(i);
        }
        for (int i = 0; i < webApps.length; i++) {
            getTableModel().addWebApplication(webApps[i]);
        }
    }

    protected WebApplication[] getWebApplications() {
        int              count = getTableModel().getRowCount();
        WebApplication[] webApps = new WebApplication[count];

        for (int i = 0; i < count; i++) {
            webApps[i] = new WebApplication(getTableModel().getValueAt(i,
                    1).toString());
            webApps[i].setContextRoot(getTableModel().getValueAt(i,
                    0).toString());
        }
        return webApps;
    }

    private void warInit() {
        Component[] comps = getScrollPane().getViewport().getComponents();

        for (int i = 0; i < comps.length; i++) {
            getScrollPane().getViewport().remove(comps[i]);
        }
        table = new JTable();
        table.setModel(new WarTableModel());
        getScrollPane().getViewport().add(table, null);
        table.getColumnModel().getColumn(0).setPreferredWidth(40);
        table.getColumnModel().getColumn(1).setPreferredWidth(250);
        getTableModel().addTableModelListener(table);
    }

    private WarTableModel getTableModel() {
        return (WarTableModel) table.getModel();
    }

    private class WarTableModel extends AbstractTableModel {
        ArrayList tableList = new ArrayList();

        public WarTableModel() {
            super();
        }

        public String getColumnName(int column) {
            String cn = "Unknown";

            switch (column) {
            case 0:
                cn = "Context Root";
                break;
            case 1:
                cn = "Web Application";
                break;
            }
            return cn;
        }

        public void removeRow(int index) {
            tableList.remove(index);
            fireTableRowsDeleted(index, index);
        }

        public Object getValueAt(int row, int column) {
            WebApplication webApp = (WebApplication) tableList.get(row);
            Object         value = "Unknown";

            switch (column) {
            case 0:
                value = webApp.getContextRoot();
                break;
            case 1:
                value = webApp.getArchive();
                break;
            }
            return value;
        }

        public void setValueAt(Object value, int row, int column) {
            WebApplication webApp = (WebApplication) tableList.get(row);

            switch (column) {
            case 0:
                webApp.setContextRoot(value.toString());
                break;
            case 1:
                webApp.setArchive(value.toString());
                break;
            }
        }


        public int getColumnCount() {
            return 2;
        }

        public boolean isCellEditable(int row, int col) {
            return (col == 0);
        }

        public int getRowCount() {
            return tableList.size();
        }

        protected void addWebApplication(WebApplication webApp) {
            int index = tableList.size();

            tableList.add(webApp);
            fireTableRowsInserted(index, index);
        }

    }
}
