/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.tool.archive.wizard;

// ToolBox
import org.enhydra.tool.archive.ArchiveException;
import org.enhydra.tool.archive.JarPlan;
import org.enhydra.tool.archive.EarPlan;

// JDK
import java.awt.*;
import java.beans.Beans;
import java.io.File;
import javax.swing.*;

//
public class WebAppPanel extends ArchivePanel {
    private BorderLayout          layoutMain = null;
    private WarSelector selectorPanel = null;

    public WebAppPanel() {
        try {
            jbInit();
            pmInit();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    public void readPlan(JarPlan plan) throws ArchiveException {
        selectorPanel.setSelections(plan.getLibFiles());
        if (plan instanceof EarPlan) {
            EarPlan ep = (EarPlan) plan;

            selectorPanel.setWebApplications(ep.getWebApplications());
        }
    }

    public void writePlan(JarPlan plan) throws ArchiveException {
        if (plan instanceof EarPlan) {
            EarPlan ep = (EarPlan) plan;

            ep.setWebApplications(selectorPanel.getWebApplications());
        }
    }

    public void validatePlan() throws ArchiveException {
    	// always valid
    }

    public String getPageTitle() {
        return "Web Application Modules";
    }

    public String getInstructions() {
        StringBuffer buf = new StringBuffer();

        buf.append("Select web applications (.war) to include in this");
        buf.append(" enterprise application (.ear). You can modify the root");
        buf.append(" context for each web application.");
        return buf.toString();
    }

    // override ArchivePanel
    protected void setWizard(ArchiveWizard wizard) {
        super.setWizard(wizard);
        selectorPanel.setWizard(wizard);
    }

    private void pmInit() {
        String[] exts = {
            "war"
        };

        selectorPanel.setExtensions(exts);
        selectorPanel.setTitle("Web Application");
    }

    private void jbInit() throws Exception {
        layoutMain =
            (BorderLayout) Beans.instantiate(getClass().getClassLoader(),
                                             BorderLayout.class.getName());
        selectorPanel =
            (WarSelector) Beans.instantiate(getClass().getClassLoader(),
                WarSelector.class.getName());
        this.setLayout(layoutMain);
        this.add(selectorPanel, BorderLayout.CENTER);
    }

}
