/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.tool.archive.xml;

// XMLC
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.enhydra.xml.io.DOMParser;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

// ToolBox
import org.enhydra.tool.archive.ArchiveException;
//import org.enhydra.tool.archive.EjbBuilder;
import org.enhydra.tool.archive.WarBuilder;
import org.enhydra.tool.archive.Descriptor;
import org.enhydra.tool.archive.Module;
import org.enhydra.tool.archive.WebApplication;
import org.enhydra.tool.common.PathHandle;

// JDK
import java.io.InputStreamReader;
import java.io.BufferedReader;
import java.io.InputStream;
import java.io.IOException;

//
public class EarDescriptorHandler extends AbstractDescriptorHandler {
    private final String  ALT_DD = "alt-dd";
    private final String  APP = "application";
    private final String  DESC_NAME = "description";
    private final String  DISPLAY_NAME = "display-name";
    private final String  EJB = "ejb";
    private final String  MODULE = "module";
    private final String  WEB = "web";
    private final String  WEB_URI = "web-uri";
    private final String  CONTEXT_ROOT = "context-root";
    private final String  PUBLIC_ID =
        "-//Sun Microsystems, Inc.//DTD J2EE Application 1.2//EN";
    private final String  SYSTEM_ID =
        "http://java.sun.com/j2ee/dtds/application_1_2.dtd";

    //
    private String        appDesc = "Application description";    // default
    private String        appName = new String();
    private Module[]      mods = new Module[0];
    private InputStream[] mergeDescriptors = new InputStream[0];
    private boolean       altDD = true;

    //
    //
    public String getAppName() {
        return appName;
    }

    public void setAppName(String n) {
        appName = n;
    }

    public String getDescription() {
        return appDesc;
    }

    public void setDescription(String d) {
        appDesc = d;
    }

    public Module[] getAllModules() {
        return mods;
    }

    public void setAllModules(Module[] m) {
        mods = m;
    }

    public InputStream[] getMergeDescriptors() {
        return mergeDescriptors;
    }

    public void setMergeDescriptors(InputStream[] m) {
        mergeDescriptors = m;
    }

    public boolean isAlt() {
        return altDD;
    }

    public void setAlt(boolean a) {
        altDD = a;
    }

    public static void main(String[] args) {
        EarDescriptorHandler prep = new EarDescriptorHandler();

        prep.setSource("d:/test/simple/test2.xml");
        prep.setOutStream(System.out);
        try {
            prep.prep();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    protected Document getDocument() throws ArchiveException {
        DOMParser parser = new DOMParser();
        Document  d = null;
        NodeList  list = null;

        d = parser.newDocument();
        d.appendChild(d.createElement(APP));
        for (int i = 0; i < mergeDescriptors.length; i++) {
            Document            mergeDoc = null;
            InputSource         inputSource = null;
            DocTypeFilterReader reader = null;

            reader =
                new DocTypeFilterReader(new BufferedReader(new InputStreamReader(mergeDescriptors[i])));
            inputSource = new InputSource(reader);
            try {
                mergeDoc = parser.parse(inputSource);
            } catch (IOException e) {
                throw new ArchiveException(e,
                                           "Unable to parse merge descriptor");
            } catch (SAXException e) {
                throw new ArchiveException(e,
                                           "Unable to parse merge descriptor");
            }
            list = mergeDoc.getChildNodes();
            for (int j = 0; j < list.getLength(); j++) {
                Node child = null;

                child = list.item(j);
                if (child instanceof Element) {
                    Element element = (Element) child;

                    if (element.getNodeName().equals("module")) {
                        d.appendChild(element);
                    }
                }
            }
        }
        return d;
    }

    //
    protected String getSystemID() {
        return SYSTEM_ID;
    }

    protected String getPublicID() {
        return PUBLIC_ID;
    }

    protected void prepElements() {
        prepDisplayName();
        prepDescription();
        prepModules();
    }

    private void prepDescription() {
        Element element = null;

        element = lookup(getDoc().getDocumentElement(), DESC_NAME);
        if (element == null) {
            element = getDoc().createElement(DESC_NAME);
            element.appendChild(getDoc().createTextNode(getDescription()));
            getDoc().getDocumentElement().appendChild(element);
        }
    }

    private void prepDisplayName() {
        Element element = null;

        element = lookup(getDoc().getDocumentElement(), DISPLAY_NAME);
        if (element == null) {
            element = getDoc().createElement(DISPLAY_NAME);
            element.appendChild(getDoc().createTextNode(getAppName()));
            getDoc().getDocumentElement().appendChild(element);
        }
    }

    private void prepModules() {
        PathHandle ph = null;

        for (int i = 0; i < mods.length; i++) {
            Element modElement = null;

            if (mods[i] instanceof WebApplication) {
                modElement = appendWar((WebApplication) mods[i]);
            } else if (mods[i].getName().toLowerCase().endsWith(".jar")) {
                modElement = appendEjbJar(mods[i]);
            }
            if ((modElement != null) && isAlt()) {
                appendAlt(modElement, mods[i]);
            }
        }
    }

    private Element appendWar(WebApplication webApp) {
        Element mod = null;
        Element web = null;
        Element uri = null;
        Element ctx = null;

        mod = getDoc().createElement(MODULE);
        web = getDoc().createElement(WEB);
        uri = getDoc().createElement(WEB_URI);
        ctx = getDoc().createElement(CONTEXT_ROOT);
        mod.appendChild(web);
        web.appendChild(uri);
        web.appendChild(ctx);
        uri.appendChild(getDoc().createTextNode(webApp.getName()));
        ctx.appendChild(getDoc().createTextNode(webApp.getContextRoot()));
        getDoc().getDocumentElement().appendChild(mod);
        return mod;
    }

    private void appendAlt(Element modElement, Module mod) {
        Element      alt = null;
        StringBuffer buf = new StringBuffer();

        buf.append(mod.getArchive());
        buf.append('/');
        if (mod instanceof WebApplication) {
            buf.append(Descriptor.getArchivePath(Descriptor.WEB));
        } else {
            buf.append(Descriptor.getArchivePath(Descriptor.EJB));
        }
        alt = getDoc().createElement(ALT_DD);
        alt.appendChild(getDoc().createTextNode(buf.toString()));
        modElement.appendChild(alt);
    }

    private Element appendEjbJar(Module ejbMod) {
        Element mod = null;
        Element ejb = null;

        mod = getDoc().createElement(MODULE);
        ejb = getDoc().createElement(EJB);
        mod.appendChild(ejb);
        ejb.appendChild(getDoc().createTextNode(ejbMod.getName()));
        getDoc().getDocumentElement().appendChild(mod);
        return mod;
    }

}
