/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.tool.codegen;

//  Toolbox imports
import org.enhydra.tool.common.ResUtil;

// Standard imports
import java.util.ArrayList;
import java.util.Arrays;
import java.util.ResourceBundle;

/**
 * An OptionSet is a utility class for working with GeneratorOptions
 */
public class OptionSet {
    public static ResourceBundle res = ResourceBundle.getBundle("org.enhydra.tool.codegen.Res");  // nores
    //
    private GeneratorOption[] options = new GeneratorOption[0];

    /**
     * Create an empty option set.
     */
    public OptionSet() {}

    /**
     * Get the generator option that has the specified option name.
     *
     * @returns
     * A GeneratorOption that has the specified name.
     *
     * @throws GeneratorException
     * Thrown if the no GeneratorOption exists in the current set with
     * the specified name.
     *
     */
    public GeneratorOption lookup(String name) throws GeneratorException {
        GeneratorOption option = null;

        for (int i = 0; i < options.length; i++) {
            if (options[i].getName().equalsIgnoreCase(name)) {
                option = options[i];
                break;
            }
        }
        if (option == null) {
            throw new GeneratorException(
              ResUtil.format(res.getString("Unable_to_lookup"), name));
        }
        return option;
    }

    /**
     * Remove the generator option that has the specified option name.
     *
     * @throws GeneratorException
     * Thrown if the no GeneratorOption exists in the current set with
     * the specified name.
     */
    public void delete(String name) throws GeneratorException {
        throw new GeneratorException("OptionSet:delete() not implemented"); // nores
    }

    /**
     * Add a new generator option.
     *
     * @throws GeneratorException
     * Thrown if the no GeneratorOption exists in the current set with
     * the specified name.
     *
     */
    public void add(GeneratorOption newOp) throws GeneratorException {
        ArrayList list = null;
        list = new ArrayList(Arrays.asList(options));
        list.add(newOp);
        list.trimToSize();
        options = new GeneratorOption[list.size()];
        options = (GeneratorOption[]) list.toArray(options);
    }

    /**
     * Convert the option set into an array of individual
     * GeneratorOption objects.
     *
     * @return
     * The options as an GeneratorOption array.
     */
    public GeneratorOption[] toArray() {
        return options;
    }

}
