/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.tool.codegen;

// ToolBox imports
import org.enhydra.tool.ToolBoxInfo;
import org.enhydra.tool.codegen.wizard.CodeGenPanel;
import org.enhydra.tool.common.FileUtil;
import org.enhydra.tool.common.PathHandle;
import org.enhydra.tool.common.Replacement;
import org.enhydra.tool.common.ReplacementSet;
import org.enhydra.tool.common.ResUtil;
import org.enhydra.tool.common.Template;
import org.enhydra.tool.common.TemplateFilter;
import org.enhydra.tool.common.TemplateTool;
import org.enhydra.tool.common.ToolException;
import org.enhydra.tool.common.event.TemplateEvent;
import org.enhydra.tool.common.event.TemplateListener;
import org.enhydra.tool.configure.ConfigTool;

// Standard imports
import java.io.BufferedReader;
import java.io.IOException;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Properties;
import java.util.ResourceBundle;
import java.util.StringTokenizer;
import java.util.Vector;

//
abstract public class ProjectGenerator extends TemplateDrivenGenerator
    implements Constants {

    //
    private String   javaPath = ToolBoxInfo.getJavaPath();
    private String[] copyright = new String[0];
    private String[] shellSteps = null;
    private String[] addinSteps = null;
    private boolean  cli = true;

    public String[] getOutputExcludes() {
        ArrayList list = new ArrayList();
        String[]  ex = new String[0];

        if (cli) {
            if (ToolBoxInfo.isEnhydra3()) {
//include creating build.xml files
//                list.add(File.separator + FILE_BUILD_XML);
            } else {
                list.add('.' + TYPE_MK);
                list.add(File.separator + FILE_MAKEFILE);
                list.add(File.separator + FILE_BUILD_XML);
            }
        } else {
            list.add('.' + TYPE_MK);
            list.add(File.separator + FILE_MAKEFILE);
            list.add(File.separator + FILE_RUN_IN);
            list.add(File.separator + FILE_RUN_BAT_IN);
            list.add(File.separator + FILE_BUILD_XML);
        }
        list.add(".bak");
        list.trimToSize();
        ex = new String[list.size()];
        ex = (String[]) list.toArray(ex);
        list.clear();
        return ex;
    }

    //
    public ProjectGenerator() {
        super();
    }

    public File[] generate() throws GeneratorException {
        File[]    files = new File[0];
        File      readme = null;
        ArrayList list = null;

        setOverwrite(getOptionSet().lookup(ProjectOptionSet.OVERWRITE).isValue());
        cli = (!getOptionSet().lookup(ProjectOptionSet.NOCLI).isValue());
        files = super.generate();
        list = new ArrayList(Arrays.asList(files));
        readme = generateReadme(getDestination());
        list.add(readme);
        files = new File[list.size()];
        files = (File[]) list.toArray(files);
        list.clear();
        return files;
    }

    public void storeProperties() {
        try {
            getOptionSet().lookup(ProjectOptionSet.OVERWRITE).setValue(isOverwrite());
        } catch (GeneratorException e) {
            e.printStackTrace();
        }
        super.storeProperties();
    }

    public void setProperties(Properties p) throws GeneratorException {
        super.setProperties(p);

        // Ignore persisted make and script flags
        getOptionSet().lookup(ProjectOptionSet.NOCLI).setValue(false);

        // ensure unique projectName
        PathHandle handle = null;
        File       destDir = null;
        String     destPath = new String();
        String     projectName = new String();
        String[]   fileList = new String[0];
        boolean    newName = false;

        handle =
            PathHandle.createPathHandle(getOptionSet().lookup(ProjectOptionSet.ROOT).getValue());
        destPath = handle.getPath();
        projectName =
            getOptionSet().lookup(ProjectOptionSet.PROJECT).getValue();
        destDir = handle.getFile();
        if (!destDir.exists()) {
            destDir.mkdirs();
        }
        fileList = destDir.list();
        for (int i = 0; i < fileList.length; i++) {
            if (fileList[i].equalsIgnoreCase(projectName)) {
                newName = true;
                char   lastChar = projectName.charAt(projectName.length()
                                                     - 1);
                String digits = new String();

                while (Character.isDigit(lastChar)) {
                    projectName = projectName.substring(0,
                                                        projectName.length()
                                                        - 1);
                    digits = lastChar + digits;
                    lastChar = projectName.charAt(projectName.length() - 1);
                }
                int next = 1;

                try {
                    next = Integer.parseInt(digits);
                } catch (NumberFormatException e) {
                    next = 1;
                }
                next++;
                projectName = projectName + next;
                i = 0;    // start the search over again
            }
        }
        if (newName) {
            getOptionSet().lookup(ProjectOptionSet.PROJECT).setValue(projectName);
        }
    }

    public TemplateFilter createCopyFilter(TemplateTool tool)
            throws GeneratorException {
        ProjectTemplateFilter copyFilter = null;

        copyFilter = new ProjectTemplateFilter(tool, getDestination());
        copyFilter.setIncludeTemplate(false);
        copyFilter.setInputIncludes(getInputIncludes());
        copyFilter.setOutputExcludes(getOutputExcludes());
        return copyFilter;
    }

    public TemplateFilter createTemplateFilter(TemplateTool tool)
            throws GeneratorException {
        ProjectTemplateFilter templateFilter = null;

        templateFilter = new ProjectTemplateFilter(tool, getDestination());
        templateFilter.setIncludeTemplate(true);
        templateFilter.setInputIncludes(getInputIncludes());
        templateFilter.setOutputExcludes(getOutputExcludes());
        return templateFilter;
    }

    public String getDestination() throws GeneratorException {
        StringBuffer buf = new StringBuffer();
        PathHandle   path = null;

        buf.append(getOptionSet().lookup(ProjectOptionSet.ROOT).getValue());
        buf.append(File.separator);
        buf.append(getOptionSet().lookup(ProjectOptionSet.PROJECT).getValue());
        path = PathHandle.createPathHandle(buf.toString());
        return path.getPath();
    }

    public void setJavaPath(String p) {
        javaPath = PathHandle.createPathString(p);
    }

    public String getJavaPath() {
        return javaPath;
    }

    /**
     * Set the steps required to build the project from
     * a unix shell.
     *
     * @param steps
     * An array of strings, each string will be inserted into
     * a readme.html file as numbered step for building the
     * project from a unix shell.
     */
    public void setShellSteps(String[] steps) {
        shellSteps = steps;
    }

    /**
     * Get steps required to build the project from
     * a unix shell.
     *
     * @return
     * An array of strings, each string will be inserted into
     * a readme.html file as numbered step for building the
     * project using make files.
     */
    public String[] getShellSteps() {
        String[] steps = shellSteps;

        if (steps == null) {
            steps = getDefaultShellSteps();
        }
        return steps;
    }

    /**
     * Get steps required to build the project with the an IDE Addin.
     *
     * @return
     * An array of strings, each string will be inserted into
     * a readme.html file as numbered step for building the
     * project within an IDE.
     */
    public String[] getAddinSteps() {
        String[] steps = addinSteps;

        if (steps == null) {
            steps = getDefaultAddinSteps();
        }
        return steps;
    }

    /**
     * Set the steps required to build the project from
     * within an IDE.
     *
     * @param steps
     * An array of strings, each string will be inserted into
     * a readme.html file as numbered step for building the
     * project within an IDE.
     */
    public void setAddinSteps(String[] steps) {
        addinSteps = steps;
    }

    /**
     * Get the path where Java source files were generated.
     *
     * @return
     * The java source directory.
     */
    public String getProjectSourcePath() throws GeneratorException {
        String path = new String();

        path = findProjectSourcePath(getTemplateRoot());
        if (path == null) {
            throw new GeneratorException(res.getString("Project_source_not"));
        }
        return path;
    }

    /**
     * Method declaration
     *
     *
     * @exception GeneratorException
     */
    public void initReplacementSet() throws GeneratorException {
        try {
            String packageName = null;
            String shellPackageDir = null;
            String systemPackageDir = null;
            String content = null;
            int    index = 0;

            //
            packageName =
                getOptionSet().lookup(ProjectOptionSet.PACKAGE).getValue();
            shellPackageDir = packageName.replace('.', '/');
            systemPackageDir = packageName.replace('.', File.separatorChar);

            //
            readCopyright();
            getReplacementSet().lookup(ProjectReplacementSet.COPYRIGHT).setReplaceWith(copyright);
            getReplacementSet().lookup(ProjectReplacementSet.at_PROJECT_at).setReplaceWith(getOptionSet().lookup(ProjectOptionSet.PROJECT).getValue());
            getReplacementSet().lookup(ProjectReplacementSet.at_PACKAGE_DIR_at).setReplaceWith(systemPackageDir);
            getReplacementSet().lookup(ProjectReplacementSet.SHELL_PACKAGE_DIR).setReplaceWith(shellPackageDir);
            getReplacementSet().lookup(ProjectReplacementSet.JAVA_PACKAGE).setReplaceWith(packageName);
        } catch (ToolException e) {
            e.printStackTrace();
            throw new GeneratorException(e,
                                         res.getString("Unable_to_init_rep"));
        }
    }

    public void onTemplate(TemplateEvent event) {
        PathHandle     phOut = null;
        ReplacementSet eventSet = null;

        phOut = PathHandle.createPathHandle(event.getTemplate().getOutput());
        eventSet = new ReplacementSet(event.getTool().getReplacements());
        try {
            if (copyright.length >= 1) {
                int      last = 0;
                String[] newCR = new String[0];

                newCR = new String[copyright.length + 2];
                newCR[0] = new String();
                if (newCR.length >= 1) {
                    last = (newCR.length - 1);
                }
                if (phOut.endsWith('/' + FILE_MAKEFILE)
                        || phOut.hasExtension(TYPE_MK)) {
                    for (int i = 1; i < (newCR.length - 1); i++) {
                        newCR[i] = "#  " + copyright[i - 1];    // nores
                    }
                    newCR[last] = "#";                          // nores
                } else if (phOut.hasExtension(TYPE_JAVA)) {
                    for (int i = 1; i < (newCR.length - 1); i++) {
                        newCR[i] = " * " + copyright[i - 1];    // nores
                    }
                    newCR[last] = " *";                         // nores
                } else if (phOut.hasExtension(TYPE_XML)) {
                    for (int i = 1; i < (newCR.length - 1); i++) {
                        newCR[i] = "   " + copyright[i - 1];    // nores
                    }
                    newCR[last] = "   ";                        // nores
                } else {
                    newCR = new String[1];
                    newCR[0] = new String();
                }
                eventSet.lookup(ProjectReplacementSet.COPYRIGHT).setReplaceWith(newCR);
            }

            // Force class names to initial cap
            if (phOut.hasExtension(TYPE_JAVA)) {
                File out = phOut.getFile();
                char c0 = out.getName().charAt(0);

                if (Character.isLowerCase(c0)) {
                    StringBuffer buf = new StringBuffer();

                    buf.append(out.getParent());
                    buf.append(File.separator);
                    buf.append(Character.toUpperCase(c0));
                    if (out.getName().length() > 1) {
                        buf.append(out.getName().substring(1));
                    }
                    event.getTemplate().setOutput(new File(buf.toString()));
                    buf.setLength(0);
                    buf.append(eventSet.lookup(ProjectReplacementSet.at_PROJECT_at).getReplaceWith()[0]);
                    buf.setCharAt(0, Character.toUpperCase(c0));
                    eventSet.lookup(ProjectReplacementSet.at_PROJECT_at).setReplaceWith(buf.toString());
                }
                event.getTool().setReplacements(eventSet.toArray());
            }
        } catch (ToolException e) {
            e.printStackTrace();
        }
    }

    abstract protected String[] getDefaultShellSteps();
    abstract protected String[] getDefaultAddinSteps();

    //
    private void readCopyright() {
        String          copyText = new String();
        String          copyPath = new String();
        StringTokenizer tokenizer = null;
        File            copyFile = null;
        BufferedReader  reader = null;
        Vector          readVector = new Vector();

        try {
            copyText =
                getOptionSet().lookup(ProjectOptionSet.COPYRIGHT).getValue();
            if (copyText.trim().length() > 0) {
                final String NEWLINE = "\n";    // nores

                tokenizer = new StringTokenizer(copyText, NEWLINE);
                copyright = new String[tokenizer.countTokens()];
                for (int i = 0; i < copyright.length; i++) {
                    copyright[i] = tokenizer.nextToken();
                }
            }
        } catch (GeneratorException e) {
            e.printStackTrace();
        }
        if (copyright.length == 0) {
            try {
                copyPath =
                    getOptionSet().lookup(ProjectOptionSet.COPYRIGHTFILE).getValue();
                if (FileUtil.isFile(copyPath)) {
                    copyFile = new File(copyPath);
                    reader = new BufferedReader(new FileReader(copyFile));
                    String line = reader.readLine();

                    while (line != null) {
                        readVector.addElement(new String(line));
                        line = reader.readLine();
                    }
                    reader.close();
                    copyright = new String[readVector.size()];
                    copyright = (String[]) readVector.toArray(copyright);
                    readVector.clear();
                }
            } catch (GeneratorException e) {
                e.printStackTrace();
                copyright = new String[0];
            } catch (FileNotFoundException e) {
                e.printStackTrace();
                copyright = new String[0];
            } catch (IOException e) {
                e.printStackTrace();
                copyright = new String[0];
            }
        }
    }

    /**
     * Method declaration
     *
     *
     * @param target
     *
     * @exception GeneratorException
     */
    public Replacement[] createReplacementsForDirectory(PathHandle path)
            throws GeneratorException {
        Replacement[] pathReps = new Replacement[0];
        StringBuffer  rootPath = new StringBuffer();
        PathHandle    destHandle = null;
        PathHandle    target = null;
        PathHandle    cursor = null;

        destHandle = PathHandle.createPathHandle(getDestination());
        target = PathHandle.createPathHandle(path.getPath());
        cursor = PathHandle.createPathHandle(path.getPath());
        if (destHandle.parentOf(cursor)) {
            rootPath.append('.');
            rootPath.append('.');
            cursor =
                PathHandle.createPathHandle(cursor.getFile().getParent());
            while (destHandle.parentOf(cursor)) {
                rootPath.append('/');
                rootPath.append('.');
                rootPath.append('.');
                cursor =
                    PathHandle.createPathHandle(cursor.getFile().getParent());
            }
        } else {
            rootPath.append('.');
        }
        try {
            getReplacementSet().lookup(ProjectReplacementSet.SHELL_ROOT_PATH).setReplaceWith(rootPath.toString());
        } catch (ToolException e) {
            throw new GeneratorException(e,
                                         res.getString("Unable_to_init_root"));
        }
        pathReps = getReplacementSet().toArray();
        StringBuffer inputBuf = new StringBuffer();
        PathHandle   inputPath = null;

        inputBuf.append(getDestination().toLowerCase());
        inputBuf.append(File.separator);
        inputBuf.append(DIR_INPUT);
        inputBuf.append(File.separator);
        inputPath = PathHandle.createPathHandle(inputBuf.toString());
        if (!inputPath.parentOf(target)) {
            try {
                Replacement[]  confReps = new Replacement[0];
                ArrayList      newList = null;
                ReplacementSet pathSet = null;

                confReps = ConfigTool.createReplacements(getDestination()
                                                         + File.separator
                                                         + DIR_OUTPUT);
                newList = new ArrayList();
                newList.addAll(Arrays.asList(pathReps));
                newList.addAll(Arrays.asList(confReps));
                pathReps = (Replacement[]) newList.toArray(pathReps);
                newList.clear();
                pathSet = new ReplacementSet(pathReps);
                pathSet.lookup(ConfigTool.JAVA_PATH).setReplaceWith(getJavaPath());
                pathReps = pathSet.toArray();
            } catch (ToolException e) {
                throw new GeneratorException(e,
                                             res.getString("Unable_to_create_path"));
            }
        }
        return pathReps;
    }

    // /
    // /
    private String findProjectSourcePath(Template source)
            throws GeneratorException {
        String       found = null;
        StringBuffer buf = new StringBuffer();
        Template[]   list = new Template[0];

        list = source.listTemplates();
        buf.append(ProjectReplacementSet.at_PACKAGE_DIR_at);
        buf.append('/');
        for (int i = 0; i < list.length; i++) {
            if (list[i].isDirectory()) {
                if (list[i].getRelativePath().endsWith(buf.toString())) {
                    String       pack = new String();
                    TemplateTool tool = null;

                    tool = new TemplateTool();
                    tool.setReplacements(getReplacementSet().toArray());
                    tool.setInitDestination(getDestination());
                    tool.initOutput(list[i]);
                    found = list[i].getOutput().getAbsolutePath();
                    pack =
                        tool.lineSearchAndReplace(ProjectReplacementSet.at_PACKAGE_DIR_at);
                    found = found.substring(0,
                                            found.length() - pack.length());
                    found = found.replace('\\', '/');
                } else {
                    found = findProjectSourcePath(list[i]);
                }
            }
            if (found != null) {
                break;
            }
        }
        return found;
    }

    /**
     * Method declaration
     *
     *
     * @param destDir
     *
     * @exception java.io.IOException
     */
    private File generateReadme(String destPath) throws GeneratorException {
        File        readme = null;
        PrintWriter out = null;
        Replacement rep = null;
        String      projName = new String();

        readme = new File(destPath + File.separator + FILE_README_HTML);
        readme.getParentFile().mkdirs();
        try {
            out = new PrintWriter(new FileWriter(readme));
        } catch (IOException e) {
            throw new GeneratorException(e,
                                         res.getString("Unable_to_create_readme"));
        }
        try {
            rep =
                getReplacementSet().lookup(ProjectReplacementSet.at_PROJECT_at);
            projName = rep.getReplaceWith()[0];
        } catch (ToolException e) {
            projName = res.getString("The_project");
        }
        out.println("<html>");     // nores
        out.println("<head>");     // nores
        out.println("  "
                    + ResUtil.format(res.getString("Title_readme"),
                                     projName));
        out.println("</head>");    // nores
        out.println("<body>");     // nores
        out.println("<br>");       // nors
        if (getAddinSteps() != null && getAddinSteps().length >= 1) {
            out.println("<h2>"
                        + ResUtil.format(res.getString("To_build_with_kelp"), projName)
                        + "</h2>");
            out.println("<ol>");                             // nores
            for (int i = 0; i < getAddinSteps().length; i++) {
                out.println("  <li>");                       // nores
                out.println("    " + getAddinSteps()[i]);    // nores
                out.println("  </li>");                      // nores
            }
            out.println("</ol>");                            // nores
        }
        if (!cli) {

            // don't show shell build steps if not all required
            // files are present.
        } else if (getShellSteps() != null && getShellSteps().length >= 1) {
            out.println("<h2>"
                        + ResUtil.format(res.getString("To_build_with_shell"), projName)
                        + "</h2>");
            out.println("<ol>");                             // nores
            for (int i = 0; i < getShellSteps().length; i++) {
                out.println("  <li>");                       // nores
                out.println("    " + getShellSteps()[i]);    // nores
                out.println("  </li>");                      // nores
            }
            out.println("</ol>");                            // nores
        }
        out.println("</body>");    // nores
        out.println("</html>");    // nores
        out.flush();
        out.close();
        return readme;
    }

}
