/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.tool.codegen.wizard;

// ToolBox imports

import org.enhydra.tool.codegen.ProjectOptionSet;
import org.enhydra.tool.codegen.GeneratorOption;
import org.enhydra.tool.codegen.GeneratorException;
import org.enhydra.tool.codegen.ValidationException;
import org.enhydra.tool.common.ExtensionFilter;
import org.enhydra.tool.common.FileUtil;
import org.enhydra.tool.common.SwingUtil;

// Standard imports

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.beans.*;
import java.io.File;
import java.util.ResourceBundle;

/**
 * Panel for entering the default options: copyright and
 * copyrightfile.
 *
 * @author Paul Mahar
 */
public class ProjectOptionPanel2 extends CodeGenPanel {

    // not to be resourced

    private final String                        TXT_TYPE = "txt";       // nores

    // All swing controls are public to allow for easy modification in IDE
    // implementations.

    /**
     * Panel for copyright
     */
    transient public JPanel                     panelCopyright;

    /**
     * Button for setting copyright file
     */
    transient public JButton                    buttonSet;

    /**
     * Text field for entering copyright file
     */
    transient public JTextField                 textCopyrightFile;

    /**
     * Text area for entering copyright
     */
    transient public JTextArea                  textCopyright;

    /**
     * Radio button for selecting a copyright file
     */
    transient public JRadioButton               radioFile;

    /**
     * Radio button for entering copyright text
     */
    transient public JRadioButton               radioText;

    /**
     * Radio button for selecting a no copyright
     */
    transient public JRadioButton               radioNone;

    /**
     * Button group for grouping radio buttons
     */
    transient public ButtonGroup                radioGroup;

    //

    transient private JScrollPane               scroll;
    transient private JPanel                    panelFiller;
    transient private GridBagLayout             layoutMain;
    transient private GridBagLayout             layoutDest;
    transient private LocalRadioListener        radioListener;
    transient private LocalButtonListener       buttonListener;

    /**
     * Create default option panel 2 for entering the
     * following default options: copyright and copyright file.
     */
    public ProjectOptionPanel2 () {
        try {
            jbInit ();
            pmInit ();
        } catch (Exception ex) {
            ex.printStackTrace ();
        }
    }

    /**
     * Read copyright and copyrightfile values from the
     * option set into the swing controls.
     *
     * @exception GeneratorException
     * Thrown if unable to update Swing controls with option set values.
     */
    public void readOptionSet () throws GeneratorException {
        GeneratorOption option = null;

        // copyright

        option = getOptionSet ().lookup (ProjectOptionSet.COPYRIGHT);

        textCopyright.setText (option.getValue ());

        // copyrightfile

        option = getOptionSet ().lookup (ProjectOptionSet.COPYRIGHTFILE);

        if (FileUtil.isFile (option.getValue ())) {
            textCopyrightFile.setText (option.getValue ());
        } else {
            textCopyrightFile.setText (new String ());
        }

        if (textCopyright.getText ().trim ().length () > 0) {
            radioText.setSelected (true);
        } else if (textCopyrightFile.getText ().trim ().length () > 0) {
            radioFile.setSelected (true);
        } else {
            radioNone.setSelected (true);
        }
    }

    /**
     * Write copyright and copyrightfile values from the
     * swing controls into the option set.
     *
     * @exception GeneratorException
     * Thrown if unable to update option set from Swing control values.
     */
    public void writeOptionSet () throws GeneratorException {
        String  value = new String ();

        // copyright

        value = textCopyright.getText ().trim ();

        getOptionSet ().lookup (ProjectOptionSet.COPYRIGHT).setValue (value);

        // copyright file

        value = textCopyrightFile.getText ();

        getOptionSet ().lookup (ProjectOptionSet.COPYRIGHTFILE).setValue (value);
    }

    /**
     * Validate the copyrightfile and copyright values
     * in the swing controls.
     *
     * @exception ValidationException
     * Thrown if swing control values are not valid for the
     * current option set.
     */
    public void validateOptionSet () throws ValidationException {
        String  value = new String ();

        if (textCopyrightFile.getText ().trim ().length () > 0) {
            File        file = new File (textCopyrightFile.getText ());

            if ((!file.exists ()) || (!file.isFile ())) {
                throw new ValidationException (res.getString ("Copyright_file_is_not"));
            }
        }
    }

    /**
     * Get the title to use on the current page.
     *
     * @return
     * A string to place at the top of a CodeGen wizard panel.
     */
    public String getPageTitle () {
        return res.getString ("Copyright");
    }

    /**
     * Get the instructions for entering option values for the
     * current page.
     *
     * @return
     * A string to place below the page title.
     */
    public String getInstructions () {
        StringBuffer    buf = new StringBuffer ();

        buf.append (res.getString ("InstructCopy1"));
        buf.append (res.getString ("InstructCopy2"));
        buf.append (res.getString ("InstructCopy3"));
        buf.append (res.getString ("InstructCopy4"));

        return buf.toString ();
    }

    // /
    // / PRIVATE METHODS
    // /

    /**
     * Method declaration
     *
     *
     * @exception Exception
     */
    private void jbInit () throws Exception {
        panelCopyright =
            (JPanel) Beans.instantiate (getClass ().getClassLoader (),
                                        JPanel.class.getName ());
        layoutDest =
            (GridBagLayout) Beans.instantiate (getClass ().getClassLoader (),
                                               GridBagLayout.class.getName ());
        buttonSet =
            (JButton) Beans.instantiate (getClass ().getClassLoader (),
                                         JButton.class.getName ());
        textCopyrightFile =
            (JTextField) Beans.instantiate (getClass ().getClassLoader (),
                                            JTextField.class.getName ());
        layoutMain =
            (GridBagLayout) Beans.instantiate (getClass ().getClassLoader (),
                                               GridBagLayout.class.getName ());
        panelFiller =
            (JPanel) Beans.instantiate (getClass ().getClassLoader (),
                                        JPanel.class.getName ());
        textCopyright =
            (JTextArea) Beans.instantiate (getClass ().getClassLoader (),
                                           JTextArea.class.getName ());
        radioFile =
            (JRadioButton) Beans.instantiate (getClass ().getClassLoader (),
                                              JRadioButton.class.getName ());
        radioText =
            (JRadioButton) Beans.instantiate (getClass ().getClassLoader (),
                                              JRadioButton.class.getName ());
        radioNone =
            (JRadioButton) Beans.instantiate (getClass ().getClassLoader (),
                                              JRadioButton.class.getName ());
        scroll = new JScrollPane (textCopyright);

        buttonSet.setText (res.getString ("Set_"));
        panelCopyright.setLayout (layoutDest);
        radioFile.setText (res.getString ("Choose_file"));
        radioText.setText (res.getString ("Enter_copyright_text"));
        radioNone.setText (res.getString ("No_copyright"));
        panelCopyright.add (textCopyrightFile,
                            new GridBagConstraints (0, 2, 2, 1, 0.1, 0.1,
                                                    GridBagConstraints.WEST,
                                                    GridBagConstraints.HORIZONTAL,
                                                    new Insets (0, 5, 5, 5),
                                                    0, 0));
        panelCopyright.add (scroll,
                            new GridBagConstraints (0, 3, 4, 1, 0.1, 0.1,
                                                    GridBagConstraints.CENTER,
                                                    GridBagConstraints.BOTH,
                                                    new Insets (5, 5, 5, 5),
                                                    0, 50));
        panelCopyright.add (radioFile,
                            new GridBagConstraints (0, 0, 1, 1, 0.0, 0.0,
                                                    GridBagConstraints.CENTER,
                                                    GridBagConstraints.HORIZONTAL,
                                                    new Insets (5, 5, 5, 5),
                                                    0, 0));
        panelCopyright.add (radioText,
                            new GridBagConstraints (1, 0, 1, 1, 0.0, 0.0,
                                                    GridBagConstraints.CENTER,
                                                    GridBagConstraints.NONE,
                                                    new Insets (5, 5, 5, 5),
                                                    0, 0));
        panelCopyright.add (buttonSet,
                            new GridBagConstraints (2, 1, 2, 2, 0.0, 0.0,
                                                    GridBagConstraints.EAST,
                                                    GridBagConstraints.NONE,
                                                    new Insets (5, 5, 5, 5),
                                                    0, 0));
        panelCopyright.add (radioNone,
                            new GridBagConstraints (2, 0, 1, 1, 0.0, 0.0,
                                                    GridBagConstraints.EAST,
                                                    GridBagConstraints.NONE,
                                                    new Insets (5, 5, 5, 5),
                                                    0, 0));
        this.setLayout (layoutMain);
        this.add (panelCopyright,
                  new GridBagConstraints (0, 1, 2, 1, 0.1, 0.1,
                                          GridBagConstraints.CENTER,
                                          GridBagConstraints.BOTH,
                                          new Insets (5, 5, 5, 5), 0, 0));
        this.add (panelFiller,
                  new GridBagConstraints (0, 2, 2, 1, 0.4, 0.4,
                                          GridBagConstraints.CENTER,
                                          GridBagConstraints.BOTH,
                                          new Insets (0, 0, 0, 0), 0, 0));
    }

    /**
     * Method declaration
     *
     */
    private void pmInit () {
        radioGroup = new ButtonGroup ();

        radioGroup.add (radioFile);
        radioGroup.add (radioText);
        radioGroup.add (radioNone);
        radioNone.setSelected (true);

        radioListener = new LocalRadioListener ();

        radioText.addItemListener (radioListener);
        radioFile.addItemListener (radioListener);
        radioNone.addItemListener (radioListener);

        buttonListener = new LocalButtonListener ();

        buttonSet.addActionListener (buttonListener);
        refreshControls ();
    }

    /**
     * Method declaration
     *
     */
    private void refreshControls () {
        if (radioNone.isSelected ()) {
            textCopyright.setText (new String ());
            textCopyright.setEnabled (false);
            textCopyrightFile.setText (new String ());
            textCopyrightFile.setEnabled (false);
            buttonSet.setEnabled (false);
        } else if (radioText.isSelected ()) {
            textCopyright.setEnabled (true);
            textCopyrightFile.setText (new String ());
            textCopyrightFile.setEnabled (false);
            buttonSet.setEnabled (false);
        } else {
            textCopyright.setText (new String ());
            textCopyright.setEnabled (false);
            textCopyrightFile.setEnabled (true);
            buttonSet.setEnabled (true);
        }

        if (textCopyright.isEnabled ()) {
            textCopyright.setForeground (SystemColor.textText);
            textCopyright.setBackground (SystemColor.textHighlightText);
        } else {
            textCopyright.setForeground (SystemColor.inactiveCaptionText);
            textCopyright.setBackground (SystemColor.controlLtHighlight);
        }

        if (textCopyrightFile.isEnabled ()) {
            textCopyrightFile.setForeground (SystemColor.textText);
            textCopyrightFile.setBackground (SystemColor.textHighlightText);
        } else {
            textCopyrightFile.setForeground (SystemColor.inactiveCaptionText);
            textCopyrightFile.setBackground (SystemColor.controlLtHighlight);
        }
    }

    /**
     * Method declaration
     *
     */
    private void browseForFile () {
        File            file = null;
        ExtensionFilter filter = null;

        filter = new ExtensionFilter ();

        filter.addExtension (TXT_TYPE);
        filter.setDescriptionTitle (res.getString ("Copyright_text"));

        file =
            SwingUtil.getFileChoice (this, textCopyrightFile.getText (),
                                     filter,
                                     res.getString ("Choose_copyright_file"));

        if (file != null) {
            textCopyrightFile.setText (file.getAbsolutePath ());
        }
    }

    //
    //

    private class LocalRadioListener implements ItemListener {
        public void itemStateChanged (ItemEvent e) {
            if (e.getStateChange () == ItemEvent.SELECTED) {
                refreshControls ();
            }
        }

    }

    //
    //

    private class LocalButtonListener implements ActionListener {
        public void actionPerformed (ActionEvent event) {
            browseForFile ();
        }

    }

}

