/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.tool.codegen.wizard;

// ToolBox imports
import org.enhydra.tool.ToolBoxInfo;
import org.enhydra.tool.codegen.CodeGen;
import org.enhydra.tool.codegen.ProjectOptionSet;
import org.enhydra.tool.codegen.GeneratorOption;
import org.enhydra.tool.codegen.GeneratorException;
import org.enhydra.tool.codegen.ValidationException;

// Standard imports
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.*;
import java.io.File;
import java.io.FileFilter;
import java.util.ResourceBundle;

/**
 * Panel for entering the default options: make and script.
 *
 * @author Lutris Technologies
 */
public class ProjectOptionPanel3 extends CodeGenPanel {

    // All swing controls are public to allow for easy modification in IDE
    // implementations.

    /**
     * Panel for checkbox controls.
     */
    transient public JPanel         panelCheck;

    /**
     * Checkbox for script
     */
    transient public JCheckBox      checkCLI;

    //
    transient private GridBagLayout layoutMain;
    transient private GridBagLayout layoutDest;
    transient private JPanel        panelFiller;

    /**
     * Create default option panel 3 for entering the
     * following default options: make and script.
     */
    public ProjectOptionPanel3() {
        try {
            jbInit();
            pmInit();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    /**
     * Read make and script from the
     * option set into the swing controls.
     *
     * @exception GeneratorException
     * Thrown if unable to update Swing controls with option set values.
     */
    public void readOptionSet() throws GeneratorException {
        GeneratorOption option = null;

        // nocli
        option = getOptionSet().lookup(ProjectOptionSet.NOCLI);
        checkCLI.setSelected(!option.isValue());

    }

    /**
     * Write make and script values from the
     * swing controls into the option set.
     *
     * @exception GeneratorException
     * Thrown if unable to update option set from Swing control values.
     */
    public void writeOptionSet() throws GeneratorException {
        boolean bVal = false;
        String  sVal = new String();

        // CLI
        bVal = checkCLI.isSelected();
        getOptionSet().lookup(ProjectOptionSet.NOCLI).setValue(!bVal);
    }

    /**
     * Validate the make and script values
     * in the swing controls.
     *
     * @exception ValidationException
     * Thrown if swing control values are not valid for the
     * current option set.
     */
    public void validateOptionSet() throws ValidationException {}

    /**
     * Get the title to use on the current page.
     *
     * @return
     * A string to place at the top of a CodeGen wizard panel.
     */
    public String getPageTitle() {
        return res.getString("Supplemental_Files");
    }

    /**
     * Get the instructions for entering option values for the
     * current page.
     *
     * @return
     * A string to place below the page title.
     */
    public String getInstructions() {
        StringBuffer buf = new StringBuffer();

        buf.append(res.getString("InstructSub1"));
        buf.append(res.getString("InstructSub2"));
        buf.append(res.getString("InstructSub3"));
        return buf.toString();
    }

    // /
    // / PRIVATE METHODS
    // /
    private void jbInit() throws Exception {
        panelCheck = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                                JPanel.class.getName());
        layoutDest =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        layoutMain =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        panelFiller = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                                 JPanel.class.getName());
        checkCLI = (JCheckBox) Beans.instantiate(getClass().getClassLoader(),
                                                  JCheckBox.class.getName());
        panelCheck.setLayout(layoutDest);
        checkCLI.setSelected(true);
        checkCLI.setText(res.getString("Create_CLI"));
        panelCheck.add(checkCLI,
                       new GridBagConstraints(0, 0, 1, 1, 0.1, 0.1,
                                              GridBagConstraints.WEST,
                                              GridBagConstraints.HORIZONTAL,
                                              new Insets(5, 10, 2, 5), 0, 0));
        this.setLayout(layoutMain);
        this.add(panelCheck,
                 new GridBagConstraints(0, 1, 2, 1, 0.1, 0.1,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.BOTH,
                                        new Insets(5, 5, 5, 5), 0, 0));
        this.add(panelFiller,
                 new GridBagConstraints(0, 2, 2, 1, 0.4, 0.4,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.BOTH,
                                        new Insets(0, 0, 0, 0), 0, 0));
    }

    private void pmInit() {
        checkCLI.setSelected(true);
    }

}
