/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.tool.common;

// Toolbox imports
import org.enhydra.tool.ToolBoxInfo;

// Standard imports
import java.awt.Component;
import java.io.File;
import java.util.ResourceBundle;
import java.util.Properties;
public class Browser {
    static ResourceBundle res =
        ResourceBundle.getBundle("org.enhydra.tool.common.Res");    // nores
    private final String  MSIE = "IEXPLORE.EXE";                    // nores
    private final String  NETSCAPE = "netscape";                    // nores
    private Component     owner = null;

    public void open(String target) {
        StringBuffer buf = new StringBuffer();
        String       browser = getBrowser();
        File         file = null;

        if (browser == null) {

            //
        } else {
            file = new File(browser);
        }
        if ((file == null) || (!file.isFile())) {
            System.err.println("Help browser not found.");
        } else {
            buf.append(file.getAbsolutePath());
            buf.append(' ');
            buf.append(target);
            try {
                Runtime.getRuntime().exec(buf.toString());
            } catch (java.io.IOException e) {
                e.printStackTrace(System.err);
            }
        }
    }

    public void setOwner(Component o) {
        owner = o;
    }

    public Component getOwner() {
        return owner;
    }

    public String getBrowser() {
        PathHandle path = null;
        Properties props = null;

        if (File.pathSeparatorChar == '\\') {
            path = PathHandle.createPathHandle(MSIE);
        } else {
            path = PathHandle.createPathHandle(NETSCAPE);
        }
        try {
            props = ToolBoxInfo.loadProperties();
            path = PathHandle.createPathHandle(props.getProperty("browser"));
        } catch (ToolException e) {
            e.printStackTrace(System.err);
        }
        if (!path.isFile()) {
            path = selectBrowser();
            if (path.isFile()) {
                setBrowser(path.getPath());
            }
        }
        return path.getPath();
    }

    public void setBrowser(String browser) {
        Properties props = null;

        try {
            props = ToolBoxInfo.loadProperties();
            props.setProperty("browser", browser);
            ToolBoxInfo.storeProperties(props);
        } catch (ToolException e) {
            e.printStackTrace(System.err);
        }
    }

    private PathHandle selectBrowser() {
        PathHandle      path = null;
        File            choice = null;
        ExtensionFilter filter = null;

        filter = new ExtensionFilter();
        filter.setDescriptionTitle(res.getString("Browser"));
        if (File.separatorChar == '\\') {
            filter.addExtension("exe");    // nores
            path = PathHandle.createPathHandle(MSIE);
        } else {
            path = PathHandle.createPathHandle(NETSCAPE);
        }
        choice = SwingUtil.getFileChoice(getOwner(), path.getPath(), filter,
                                         res.getString("Select_browser"));
        path = PathHandle.createPathHandle(choice);
        return path;
    }

}
