/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.tool.common;

// Standard imports
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.IOException;
import java.io.File;
import java.io.FileFilter;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.ArrayList;
import java.util.ResourceBundle;

//
public class FileUtil {
    static ResourceBundle res =
        ResourceBundle.getBundle("org.enhydra.tool.common.Res");    // nores

    public static void copy(InputStream source,
                            OutputStream dest) throws IOException {
        BufferedInputStream  in = null;
        BufferedOutputStream out = null;
        int                  read = 1;
        int                  readSum = 0;
        int                  available = 1;
        int sleepCount = 0;

        if (source instanceof BufferedInputStream) {
            in = (BufferedInputStream) source;
        } else {
            in = new BufferedInputStream(source);
        }
        if (dest instanceof BufferedOutputStream) {
            out = (BufferedOutputStream) dest;
        } else {
            out = new BufferedOutputStream(dest);
        }
        while (true) {
            byte[] bytes = new byte[0];

            available = in.available();
            if (available < 1) {
                out.flush();
                if ((read < bytes.length) && (readSum > 0)) {
                    break;
                } else if (sleepCount > 10) {
                    break;
                } else {
                    FileUtil.sleep(100);
                    sleepCount++;
                    available = in.available();
                }
            }
            if (available > 2048) {
                bytes = new byte[2048];
            } else if (available > 1024) {
                bytes = new byte[1024];
            } else if (available > 256) {
                bytes = new byte[256];
            } else if (available > 16) {
                bytes = new byte[16];
            } else {
                bytes = new byte[1];
            }
            read = in.read(bytes, 0, bytes.length);
            if (read > 0) {
              readSum += read;
              out.write(bytes, 0, read);
           }
        }
        out.flush();
    }

    //
    public static File copy(InputStream source,
                            File dest) throws ToolException {
        FileOutputStream out = null;

        try {
            out = new FileOutputStream(dest);
            FileUtil.copy(source, out);
            source.close();
            out.flush();
            out.close();
        } catch (FileNotFoundException e) {
            String mess = new String();

            mess = ResUtil.format(res.getString("Unable_to_create_0_"), dest);
            throw new ToolException(e, mess);
        } catch (IOException e) {
            String mess = new String();

            mess = ResUtil.format(res.getString("Unable_to_create_0_"), dest);
            throw new ToolException(e, mess);
        }
        return dest;
    }

    public static File copy(Template source, File dest) throws ToolException {
        File        parentFile = null;
        InputStream in = null;

        if (source.isDirectory()) {
            if (dest.isDirectory()) {
                return dest;
            } else if (dest.exists()) {
                String mess = new String();

                mess = ResUtil.format(res.getString("Unable_to_copy"),
                                      source.toString(), dest);
                throw new ToolException(mess);
            } else {
                dest = source.getOutput();
                dest.mkdirs();
                return dest;
            }
        }
        if (source.isFile() && dest.isDirectory()) {
            parentFile = new File(dest.getAbsolutePath());
            dest = source.getOutput();
        } else {
            parentFile = new File(dest.getParent());
        }
        parentFile.mkdirs();
        try {
            in = source.getInputStream();
            dest = FileUtil.copy(in, dest);
        } catch (IOException e) {
            e.printStackTrace(System.err);
            String mess = new String();

            mess = ResUtil.format(res.getString("Unable_to_copy_file"),
                                  source.toString(), dest);
            throw new ToolException(e, mess);
        } catch (ToolException e) {
            e.printStackTrace(System.err);
            String mess = new String();

            mess = ResUtil.format(res.getString("Unable_to_copy_file"),
                                  source.toString(), dest);
            throw new ToolException(e, mess);
        }
        return dest;
    }

    public static File copy(File source, File dest) throws ToolException {
        File            parentFile = null;
        FileInputStream in = null;

        if (source.isDirectory()) {
            if (dest.isDirectory()) {
                return dest;
            } else if (dest.exists()) {
                String mess = new String();

                mess = ResUtil.format(res.getString("Unable_to_copy"),
                                      source, dest);
                throw new ToolException(mess);
            } else {
                dest = new File(dest, source.getName());
                dest.mkdirs();
                return dest;
            }
        }
        if (source.isFile() && dest.isDirectory()) {
            parentFile = new File(dest.getAbsolutePath());
            dest = new File(dest, source.getName());
        } else {
            parentFile = new File(dest.getParent());
        }
        parentFile.mkdirs();
        try {
            in = new FileInputStream(source);
            dest = FileUtil.copy(in, dest);
        } catch (FileNotFoundException e) {
            String mess = new String();

            mess = ResUtil.format(res.getString("Unable_to_copy_file"),
                                  source, dest);
            throw new ToolException(e, mess);
        } catch (ToolException e) {
            String mess = new String();

            mess = ResUtil.format(res.getString("Unable_to_copy_file"),
                                  source, dest);
            throw new ToolException(e, mess);
        }
        return dest;
    }

    public static boolean isDirectory(String path) {
        boolean dir = false;

        if (path != null) {
            File file = new File(path);

            dir = file.isDirectory();
        }
        return dir;
    }

    public static boolean isFile(String path) {
        boolean file = false;

        if (path != null) {
            File f = new File(path);

            file = f.isFile();
        }
        return file;
    }

    public static String toCanonicalPath(final String in) {
        final String DOT = new String() + '.';
        String       current = FileUtil.toCurrentPath(in);
        String       out = new String(current);
        int          index = -1;

        index = in.indexOf(DOT + DOT);
        if (index < 0) {
            index = current.indexOf(File.separator + '.');
        }
        if (index < 0) {
            index = current.indexOf('.' + File.separator);
        }
        if ((index > -1) || in.startsWith(DOT) || in.endsWith(DOT)) {
            File file = new File(current);

            try {
                out = file.getCanonicalPath();
            } catch (Exception e) {
                out = current;
                e.printStackTrace();
            }
        }
        return out;
    }

    public static String toJavaPath(final String in) {
        String path = new String(in);

        path = FileUtil.toCurrentPath(path);
        return path.replace('\\', '/');
    }

    public static String toCurrentPath(String path) {
        String cPath = path;
        File   file;

        if (File.separatorChar == '/') {
            cPath = FileUtil.toShellPath(cPath);
        } else {
            cPath = FileUtil.toWindowsPath(cPath);
        }
        file = new File(cPath);

        // Add default drive
        file = new File(file.getAbsolutePath());
        if (file.exists()) {
            cPath = file.getAbsolutePath();
        }
        return cPath.trim();
    }

    /**
     * Convert the given path to contain file seperators for
     * a Unix shell. This also converts paths that start with a
     * a Windows drive letter into one compatible with the
     * Cygnus shell.
     *
     * @param path
     * File path containing any combination of unix shell and Windows file
     * seperators.
     *
     * @return
     * The path compatible with the Cygnus shell.
     */
    public static String toShellPath(String inPath) {
        StringBuffer path = new StringBuffer();
        int          index = -1;

        inPath = inPath.trim();
        index = inPath.indexOf(":\\");    // nores
        inPath = inPath.replace('\\', '/');
        if (index > -1) {
            path.append("//");    // nores
            path.append(inPath.substring(0, index));
            path.append('/');
            path.append(inPath.substring(index + 2));
        } else {
            path.append(inPath);
        }
        return path.toString();
    }

    /**
     * Convert the given path to contain file seperators for
     * Windows. This also converts paths that start with a
     * '\\' into paths that start with a Windows drive letter.
     *
     * @param path
     * File path containing any combination of unix shell and Windows file
     * seperators.
     *
     * @return
     * The path compatible with the Windows.
     */
    public static String toWindowsPath(String path) {
        String winPath = path;
        int    index = winPath.indexOf("//");    // nores

        if (index > -1) {
            winPath = winPath.substring(0, index) + ":\\"    // nores
            + winPath.substring(index + 2);
        }
        index = winPath.indexOf(':');
        if (index == 1) {
            winPath = winPath.substring(0, 1).toUpperCase()
                      + winPath.substring(1);
        }
        winPath = winPath.replace('/', '\\');
        return winPath;
    }

    public static File findFirst(FileFilter filter, String path) {
        return FileUtil.findFirst(filter, new File(path), 0);
    }

    public static File findFirst(FileFilter filter, File dir, int count) {
        final String DIR_CLASSES = "classes";    // nores
        final String DIR_OUTPUT = "output";      // nores
        File[]       files = dir.listFiles(filter);
        File         found = null;
        File         candidate = null;

        if (count > 500) {

            // abort find if we've already searched through 500 directories
        } else {
            count++;
            if (files != null && files.length >= 1) {
                found = files[0];
            }
            if (found == null) {
                files = dir.listFiles();
                if (files != null) {
                    for (int i = 0; i < files.length; i++) {
                        if (files[i].isDirectory()) {
                            if (files[i].getName().equalsIgnoreCase(DIR_CLASSES)
                                    || files[i].getName().equalsIgnoreCase(DIR_OUTPUT)) {

                                // ignore
                            } else {
                                candidate = FileUtil.findFirst(filter,
                                                               files[i],
                                                               count);
                                if (found == null) {
                                    found = candidate;
                                } else if (candidate != null) {
                                    if (FileUtil.charCount(found.getAbsolutePath(), File.separatorChar)
                                            > FileUtil.charCount(candidate.getAbsolutePath(),
                                                                 File.separatorChar)) {
                                        found = candidate;
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
        return found;
    }

    public static String[] findJarPaths(String jar, ClassLoader loader) {

        //
        ArrayList      list = new ArrayList();
        String[]       paths = new String[0];
        URLClassLoader urlLoader = null;

        if (loader == null || (!(loader instanceof URLClassLoader))) {
            loader = ClassLoader.getSystemClassLoader();
        }
        if (loader instanceof URLClassLoader) {
            urlLoader = (URLClassLoader) loader;
            for (int i = 0; i < urlLoader.getURLs().length; i++) {
                String     path = null;
                PathHandle ph = null;

                path = urlLoader.getURLs()[i].getFile().toLowerCase();
                if (path.endsWith(jar.toLowerCase())) {
                    if (path.charAt(2) == ':') {
                        path = path.substring(1);
                    }
                    ph = PathHandle.createPathHandle(path);
                    if (ph.isFile()) {
                        list.add(ph.getPath());
                    }
                }
            }
            list.trimToSize();
            paths = new String[list.size()];
            paths = (String[]) list.toArray(paths);
            list.clear();
        }
        return paths;
    }

    private static int charCount(String in, char lookFor) {
        String search = new String(in);
        int    index = search.indexOf(lookFor);
        int    count = 0;

        while (index > 0) {
            count++;
            search = search.substring(index + 1);
            index = search.indexOf(lookFor);
        }
        return count;
    }

    private static void sleep(int m) {
        try {
            Thread.sleep(m);
        } catch (InterruptedException e) {
            e.printStackTrace(System.err);
        }
    }

}
