/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.tool.common;

// standard imports
import java.util.ArrayList;
import java.util.Arrays;
import java.util.ResourceBundle;

/**
 * ReplacementSet is a utility class for working with Replacement objects.
 */
public class ReplacementSet {
    static ResourceBundle res =
        ResourceBundle.getBundle("org.enhydra.tool.common.Res");    // nores

    //
    private Replacement[] replacements = new Replacement[0];

    /**
     * Create an empty replacement set.
     */
    public ReplacementSet() {}

    public ReplacementSet(Replacement[] reps) {
        replacements = reps;
    }

    /**
     * Get the generator replacement that has the specified replacement name.
     *
     * @returns
     * A Generatorreplacement that has the specified name.
     *
     * @throws ToolException
     * Thrown if the no Replacement exists in the current set with
     * the specified name.
     *
     */
    public Replacement lookup(String find) throws ToolException {
        Replacement replacement = null;

        for (int i = 0; i < replacements.length; i++) {
            if (replacements[i].getFind().equalsIgnoreCase(find)) {
                replacement = replacements[i];
                break;
            }
        }
        if (replacement == null) {
            throw new ToolException(ResUtil.format(res.getString("Unable_to_lookup"),
                                                   find));
        }
        return replacement;
    }

    /**
     * Remove the generator replacement that has the specified replacement name.
     *
     * @throws GeneratorException
     * Thrown if the no Replacement exists in the current set with
     * the specified find key.
     *
     */
    public void delete(String find) throws ToolException {}

    /**
     * Add a new generator replacement.
     *
     * @throws GeneratorException
     * Thrown if the no Generatorreplacement exists in the current set with
     * the specified name.
     *
     */
    public void add(Replacement newRep) throws ToolException {
        ArrayList list = null;

        list = new ArrayList(Arrays.asList(replacements));
        list.add(newRep);
        list.trimToSize();
        replacements = new Replacement[list.size()];
        replacements = (Replacement[]) list.toArray(replacements);
    }

    /**
     * Convert the option set into an array of individual
     * GeneratorOption objects.
     *
     * @return
     * The options as an GeneratorOption array.
     */
    public Replacement[] toArray() {
        return replacements;
    }

    public String[][] toStringArray() {
        String[][] strings = new String[replacements.length][2];

        for (int i = 0; i < replacements.length; i++) {
            strings[i][0] = replacements[i].getFind();
            strings[i][1] = replacements[i].getReplaceWith()[0];
        }
        return strings;
    }

}
