/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.tool.common;

// ToolBox imports
import org.enhydra.tool.ToolBoxInfo;
import org.enhydra.tool.common.event.TemplateEvent;
import org.enhydra.tool.common.event.TemplateListener;

// Standard imports
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.File;
import java.io.FileFilter;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.PrintWriter;
import java.net.URL;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.ResourceBundle;
import javax.swing.JOptionPane;
import javax.swing.JFrame;
import javax.swing.JButton;
import javax.swing.JDialog;

//
public class TemplateTool {

    //
    static ResourceBundle      res =
        ResourceBundle.getBundle("org.enhydra.tool.common.Res");    // nores

    //
    private boolean            echo = true;
    private boolean            overwrite = false;
    private Component          owner = null;
    private Template[]         templates = new Template[0];
    private Replacement[]      replacements = new Replacement[0];
    private ArrayList          templateList = new ArrayList();
    private String[]           initExtensions = new String[0];
    private String             initPathKey = new String();
    private String             initDestPath = new String();
    private FileFilter         filter = new TemplateFilter();
    private TemplateListener[] listeners = new TemplateListener[0];
    private PrintWriter        echoWriter = new PrintWriter(System.out);

    //
    public TemplateTool() {}

    public boolean isOverwrite() {
        return overwrite;
    }

    public void setOverwrite(boolean o) {
        overwrite = o;
    }

    public boolean isSwing() {
        return (owner != null);
    }

    public Component getOwner() {
        return owner;
    }

    public void setOwner(Component c) {
        owner = c;
    }

    public Template[] getTemplates() {
        return templates;
    }

    public void setTemplates(Template[] t) {
        templates = t;
    }

    public TemplateListener[] getTemplateListeners() {
        return listeners;
    }

    public void addTemplateListener(TemplateListener l) {
        ArrayList list = null;

        list = new ArrayList(Arrays.asList(listeners));
        if (!list.contains(l)) {
            list.add(l);
            list.trimToSize();
            listeners = new TemplateListener[list.size()];
            listeners = (TemplateListener[]) list.toArray(listeners);
        }
        list.clear();
    }

    public void removeTemplateListener(TemplateListener l) {
        ArrayList list = null;

        list = new ArrayList(Arrays.asList(listeners));
        if (list.contains(l)) {
            list.remove(l);
            list.trimToSize();
            listeners = new TemplateListener[list.size()];
            listeners = (TemplateListener[]) list.toArray(listeners);
        }
        list.clear();
    }

    //
    //
    public void initTemplates(File source, File dest, String[] exts) {
        setInitExtensions(exts);
        setInitDestination(dest.getAbsolutePath());
        templateList.clear();
        initPathKey = source.getAbsolutePath();
        fillTemplateList(source);
        templateList.trimToSize();
        templates = new Template[templateList.size()];
        templates = (Template[]) templateList.toArray(templates);
        templateList.clear();
    }

    public void initTemplates(Template[] in, File dest, String[] exts) {
        setInitExtensions(exts);
        setInitDestination(dest.getAbsolutePath());
        for (int i = 0; i < in.length; i++) {
            initOutput(in[i]);
        }
        setTemplates(in);
    }

    public Replacement[] getReplacements() {
        return replacements;
    }

    public void setReplacements(Replacement[] r) {
        replacements = r;
    }

    public PrintWriter getEchoWriter() {
        return echoWriter;
    }

    public void setEchoWriter(PrintWriter w) {
        if (w == null) {
            setEcho(false);
        } else {
            echoWriter = w;
        }
    }

    public boolean isEcho() {
        if (getEchoWriter() == null) {
            echo = false;
        }
        return echo;
    }

    public void setEcho(boolean e) {
        echo = e;
    }

    public File[] createOutput() throws ToolException {
        ArrayList outList = new ArrayList();
        File[]    outFiles = new File[0];

        for (int i = 0; i < templates.length; i++) {
            File out = null;

            out = templates[i].getOutput();
            if (templates[i].isDirectory()) {
                out.mkdirs();
            } else {
                if (isFileAvailable(out)) {
                    fileSearchAndReplace(templates[i]);
                    outList.add(templates[i].getOutput());
                }
            }
        }
        outList.trimToSize();
        outFiles = new File[outList.size()];
        outFiles = (File[]) outList.toArray(outFiles);
        outList.clear();
        return outFiles;
    }

    public String lineSearchAndReplace(String in) {
        String text = in;
        String prefix = new String();
        String suffix = new String();

        for (int i = 0; i < replacements.length; i++) {
            int indexOf = text.indexOf(replacements[i].getFind());
            int lastIndex = -1;
            int count = 0;

            while (indexOf != -1 && indexOf > lastIndex && count < 1000) {

                // Catch recursive replacements by maxing out at
                // after the 1000 replacement or break after one try
                // if imediate recursion.
                count++;
                prefix = text.substring(0, indexOf);
                suffix = text.substring(indexOf
                                        + replacements[i].getFind().length());
                StringBuffer buf = new StringBuffer();

                buf.append(prefix);
                for (int j = 0; j < replacements[i].getReplaceWith().length;
                        j++) {
                    String rep = replacements[i].getReplaceWith()[j];

                    if (j > 0) {
                        if (File.separatorChar == '\\') {    // Windows
                            buf.append((char) 13);           // CR
                        }
                        buf.append((char) 10);               // LF
                    }
                    buf.append(rep);
                }
                buf.append(suffix);
                text = buf.toString();
                indexOf = text.indexOf(replacements[i].getFind());
                if ((indexOf > -1)
                        && (replacements[i].getReplaceWith().length > 1)) {
                    break;
                }
            }
        }
        return text;
    }

    //
    //
    protected void echoWrite(String out) {
        if (isEcho()) {
            getEchoWriter().println(out);
            getEchoWriter().flush();
        }
    }

    protected void echoWrite(Exception e) {
        if (isEcho()) {
            getEchoWriter().println(e.getMessage());
        }
        e.printStackTrace();
    }

    //
    //
    public void setInitDestination(String path) {
        initDestPath = PathHandle.createPathString(path);
    }

    public String getInitDestination() {
        return initDestPath;
    }

    public void setInitExtensions(String[] exts) {
        ArrayList list = new ArrayList();

        if (exts == null) {
            initExtensions = new String[0];
        } else {
            for (int i = 0; i < exts.length; i++) {
                if (exts[i].length() > 0) {
                    if ((exts[i].charAt(0) == '.')
                            && (exts[i].length() > 1)) {
                        list.add(exts[i].trim().substring(1));
                    } else {
                        list.add(exts[i].trim());
                    }
                }
            }
            list.trimToSize();
            initExtensions = new String[list.size()];
            initExtensions = (String[]) list.toArray(initExtensions);
        }
    }

    public String[] getInitExtensions() {
        return initExtensions;
    }

    private void notify(Template in) {
        TemplateListener[] templateListeners = getTemplateListeners();

        if (templateListeners.length == 0) {

            // done
        } else {
            TemplateEvent event = null;

            event = new TemplateEvent(this, in);
            for (int i = 0; i < templateListeners.length; i++) {
                templateListeners[i].onTemplate(event);
            }
        }
    }

    private void fileSearchAndReplace(Template in) throws ToolException {
        BufferedReader reader = null;
        PrintWriter    writer = null;
        File           parent = null;
        PathHandle     ph = null;

        notify(in);
        ph = PathHandle.createPathHandle(in.getOutput());
        parent = ph.getParent().getFile();
        if ((parent != null) && (!parent.exists())) {
            parent.mkdirs();
        }
        try {
            reader =
                new BufferedReader(new InputStreamReader(in.getInputStream()));
            writer = new PrintWriter(new FileWriter(ph.getFile()));
            String line = reader.readLine();

            while (line != null) {
                line = lineSearchAndReplace(line);
                writer.println(line);
                line = reader.readLine();
            }
            reader.close();
            writer.close();
            echoWrite(ResUtil.format(res.getString("Creating_file_0_"), ph));
        } catch (IOException e) {
            throw (new ToolException(e,
                                     ResUtil.format(res.getString("Unable_to_create_0_"),
                                                    ph)));
        }
    }

    private void fillTemplateList(File root) {
        if (root.isDirectory()) {
            File[]   children = new File[0];
            Template template = null;
            File     out = null;

            if (filter == null) {
                children = root.listFiles();
            } else {
                children = root.listFiles(filter);
            }
            int count = 0;

            for (int i = 0; i < children.length; i++) {
                if (children[i].isDirectory()) {
                    fillTemplateList(children[i]);
                } else if (children[i].isFile()) {
                    count++;
                    template = new Template(children[i], initPathKey);
                    initOutput(template);
                    templateList.add(template);
                }
            }
            echoWrite(ResUtil.format(res.getString("_0_contains_1"), root,
                                     count));
        }
    }

    public void initOutput(Template template) {
        String       suffix = new String();
        String       ext = new String();
        StringBuffer pathBuf = new StringBuffer();
        File         out = null;
        int          index = -1;

        suffix = template.getRelativePath();
        ext = template.getExtension().toLowerCase();
        for (int i = 0; i < getInitExtensions().length; i++) {
            if (ext.startsWith(getInitExtensions()[i].toLowerCase())) {
                index = i;
                break;
            }
        }
        if (index > -1) {
            suffix = suffix.substring(0,
                                      suffix.length() - (ext.length() + 1));
        }
        pathBuf.append(initDestPath);
        if (!pathBuf.toString().endsWith("/")) {
            pathBuf.append('/');
        }
        pathBuf.append(suffix);
        out = new File(lineSearchAndReplace(pathBuf.toString()));
        template.setOutput(out);
    }

    /**
     * Method declaration
     *
     *
     * @param file
     *
     * @return
     */
    private boolean isFileAvailable(File file) throws ToolException {
        boolean available = true;

        if (file.exists() && (!file.isDirectory())) {
            if (isOverwrite()) {
                file.delete();
            } else {
                if (isSwing()) {
                    available = askForOverwrite(file);
                } else {

                    // command line
                    throw new ToolException(ResUtil.format(res.getString("File_already_exists_0"),
                                                           file));
                }
            }
        }
        return available;
    }

    /**
     * Method declaration
     *
     */
    private boolean askForOverwrite(File file) throws ToolException {
        final String      YES = res.getString("Yes");
        final String      ALL = res.getString("Yes_to_All");
        final String      NO = res.getString("No");
        final String      CANCEL = res.getString("Cancel");
        final JOptionPane optionPane = new JOptionPane();
        String            mess = new String();

        mess = ResUtil.format(res.getString("File_already_exists_n"), file);
        optionPane.setMessage(mess);
        final JDialog dialog = optionPane.createDialog(new JFrame(),
                res.getString("Confirm_Overwrite"));
        JButton[]     buttons = new JButton[4];
        JButton       selection = null;
        boolean       overwriteThis = false;
        boolean       cancel = false;

        for (int i = 0; i < buttons.length; i++) {
            buttons[i] = new JButton();
            buttons[i].setActionCommand(new String() + i);
            buttons[i].addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent event) {
                    optionPane.setValue(event.getSource());
                    dialog.setVisible(false);
                }

            });
        }
        buttons[0].setText(YES);
        buttons[1].setText(ALL);
        buttons[2].setText(NO);
        buttons[3].setText(CANCEL);
        optionPane.setOptions(buttons);
        dialog.pack();
        dialog.show();
        if (optionPane.getValue() instanceof JButton) {
            selection = (JButton) optionPane.getValue();
            if (selection.getText().equals(CANCEL)) {
                cancel = true;
            } else if (selection.getText().equals(YES)) {
                overwriteThis = true;
                file.delete();
            } else if (selection.getText().equals(ALL)) {
                overwriteThis = true;
                setOverwrite(true);
                file.delete();
            }
        }
        optionPane.removeAll();
        dialog.removeAll();
        dialog.dispose();
        if (cancel) {
            throw new ToolException(res.getString("Template_processing"));
        }
        return overwriteThis;
    }

    private class TemplateFilter implements FileFilter {
        public boolean accept(File file) {
            boolean    include = false;
            PathHandle path = null;

            if (file.isDirectory()) {
                include = true;
            } else if (getInitExtensions().length > 0) {
                path = PathHandle.createPathHandle(file);
                include = path.hasExtension(getInitExtensions());
            } else {
                include = true;
            }
            return include;
        }

    }
}
