/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.tool.common.wizard;

// ToolBox imports
import org.enhydra.tool.common.ToolException;

// Standard imports
import java.awt.CardLayout;
import java.beans.Beans;
import java.util.Arrays;
import java.util.ArrayList;
import java.util.ResourceBundle;
import javax.swing.JPanel;
import javax.swing.JOptionPane;

//
public class TBWizardDeck extends JPanel {
    static ResourceBundle res =
        ResourceBundle.getBundle("org.enhydra.tool.common.Res");    // nores

    //
    private CardLayout    cardLayout = null;
    private int           pageIndex = 0;
    private TBWizardPage[]  pages = new TBWizardPage[0];

    public TBWizardDeck() {
        try {
            jbInit();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * Method declaration
     *
     */
    public void next() {
        if (!isLastPageShowing()) {
            TBWizardPage currentPage = pages[pageIndex];
            TBWizardPage nextPage = pages[pageIndex +1];

            try {
                currentPage.validatePage();
                nextPage.preparePage();
                cardLayout.next(this);
                pageIndex++;
            } catch (ToolException e) {
                JOptionPane.showMessageDialog(this, e.getMessage(),
                                              res.getString("Validation_Error"),
                                              JOptionPane.ERROR_MESSAGE);
            }
        }
    }

    //
    protected TBWizardPage[] getPages() {
        return pages;
    }

    public int getPageIndex() {
        return pageIndex;
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    protected boolean isFirstPageShowing() {
        boolean atFirst = (pageIndex == 0);

        return (atFirst);
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    protected boolean isLastPageShowing() {
        return (pageIndex == (pages.length - 1));
    }

    /**
     * Method declaration
     *
     */
    protected void back() {
        if (!isFirstPageShowing()) {
            cardLayout.previous(this);
            pageIndex--;
        }
    }

    /**
     * Method declaration
     *
     *
     * @param page
     */
    protected void addWizardPage(TBWizardPage page) {
        ArrayList list = null;
        this.add(page, (new String()) + pages.length);
        list = new ArrayList(Arrays.asList(pages));
        list.add(page);
        list.trimToSize();
        pages = new TBWizardPage[list.size()];
        pages = (TBWizardPage[]) list.toArray(pages);
        list.clear();
    }

    protected void removeWizardPage(TBWizardPage page) {
        ArrayList list = null;
        list = new ArrayList(Arrays.asList(pages));
        if (list.contains(page)) {
            remove(page);
           list.remove(page);
        }
        list.trimToSize();
        pages = new TBWizardPage[list.size()];
        pages = (TBWizardPage[]) list.toArray(pages);
        list.clear();
    }


    /**
     * Method declaration
     *
     *
     * @exception Exception
     */
    private void jbInit() throws Exception {
        cardLayout =
            (CardLayout) Beans.instantiate(getClass().getClassLoader(),
                                           CardLayout.class.getName());
        this.setLayout(cardLayout);
    }

}
